@@ -60,9 +60,9 @@ let __init_popl = !Flag_cocci.popl
 let __init_timeout = !Flag_cocci.timeout
 let __init_selected_only = !Flag_cocci.selected_only
 let __init_use_saved_typedefs = !Flag_cocci.use_saved_typedefs
-let __init_std_iso = !Config.std_iso
-let __init_std_h = !Config.std_h
-let __init_python_interpreter = !Config.python_interpreter
+let __init_std_iso = !Cocciconfig.std_iso
+let __init_std_h = !Cocciconfig.std_h
+let __init_python_interpreter = !Cocciconfig.python_interpreter
 let __init_sgrep_mode2 = !Flag.sgrep_mode2
 let __init_show_misc = !Flag.show_misc
 let __init_show_transinfo = !Flag.show_transinfo
@@ -370,9 +370,9 @@ let coccinelle arglist =
   Flag_cocci.timeout := __init_timeout;
   Flag_cocci.selected_only := __init_selected_only;
   Flag_cocci.use_saved_typedefs := __init_use_saved_typedefs;
-  Config.std_iso := __init_std_iso;
-  Config.std_h := __init_std_h;
-  Config.python_interpreter := __init_python_interpreter;
+  Cocciconfig.std_iso := __init_std_iso;
+  Cocciconfig.std_h := __init_std_h;
+  Cocciconfig.python_interpreter := __init_python_interpreter;
   Flag.sgrep_mode2 := __init_sgrep_mode2;
   Flag.show_misc := __init_show_misc;
   Flag.show_transinfo := __init_show_transinfo;
