@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -32,7 +32,7 @@ type ('a,'b,'c) environment = (string, ('a,'b,'c) binding_val) Common.assoc
 and ('a, 'b, 'c) binding_val =
     Val of ('a,'b,'c) generic_ctl * ('a,'b,'c) environment
 
-let rec (pp_ctl:
+let (pp_ctl:
    ('pred -> unit) * ('mvar -> unit) -> bool ->
    ('pred, 'mvar, 'info) generic_ctl ->
    unit) =
