@@ -36,19 +36,14 @@ let contains_modif used_after x =
 	  bind (mcode r ((),(),bef,[])) res
       | Ast.Decl decl ->
 	  bind (mcode r ((),(),annotated_decl_bef decl,[])) res
-      | Ast.ForHeader(fr,lp,Ast.ForDecl decl,e2,sem2,e3,rp) ->
+      | Ast.ForHeader(fr,lp,Ast.ForDecl (decl,e2,sem2,e3),rp) ->
+	  bind (mcode r ((),(),annotated_decl_bef decl,[])) res
+      | Ast.ForHeader(fr,lp,Ast.ForRange (decl,exp),rp) ->
 	  bind (mcode r ((),(),annotated_decl_bef decl,[])) res
       | _ -> res in
     let recursor =
-      V.combiner bind option_default
-	mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-	mcode mcode mcode
-	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing
-	do_nothing do_nothing rule_elem do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing in
+      V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=do_nothing}
+	~rule:rule_elem do_nothing in
     recursor.V.combiner_rule_elem x
 
 (* contains an inherited metavariable or contains a constant *)
@@ -68,15 +63,8 @@ let contains_constant x =
 	  Ast.Constant(const) -> true
 	| _ -> k e in
       let recursor =
-	V.combiner bind option_default
-	  mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-	  mcode mcode mcode
-	  do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	  do_nothing
-	  ident expr do_nothing do_nothing do_nothing do_nothing do_nothing
-	  do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	  do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	  do_nothing do_nothing do_nothing do_nothing in
+	V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=do_nothing}
+	  ~ident:ident ~expr:expr do_nothing in
       recursor.V.combiner_rule_elem x
   | _ -> true
 
@@ -124,21 +112,22 @@ let strip x =
 	Ast.rewrap res
 	  (Ast.FunHeader(no_mcode,b,fninfo,name,lp,params,va,rp,endattrs))
     | Ast.Decl decl -> Ast.rewrap res (Ast.Decl(annotated_decl no_mcode decl))
-    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
+    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl,e2,sem2,e3),rp) ->
 	Ast.rewrap res
-	  (Ast.ForHeader(fr,lp,Ast.ForDecl(annotated_decl no_mcode decl),
-			 e2,sem2,e3,rp))
+	  (Ast.ForHeader(fr,lp,
+			 Ast.ForDecl(annotated_decl no_mcode decl,e2,sem2,e3),
+			 rp))
+    | Ast.ForHeader(fr,lp,Ast.ForRange(decl,e2),rp) ->
+	Ast.rewrap res
+	  (Ast.ForHeader(fr,lp,
+			 Ast.ForRange(annotated_decl no_mcode decl,e2),
+			 rp))
     | _ -> res in
   let r =
-    V.rebuilder
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      mcode mcode mcode mcode mcode
-      do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-      do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-      do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-      decl_or_field do_absolutely_nothing decl_or_field do_absolutely_nothing
-      do_nothing rule_elem
-      do_nothing do_nothing do_nothing do_nothing do_nothing do_absolutely_nothing in
+    V.rebuilder {V.rmcode=mcode} {V.rdonothing=do_nothing}
+      ~decl:decl_or_field ~annotated_decl:do_absolutely_nothing
+      ~field:decl_or_field ~annotated_field:do_absolutely_nothing
+      ~rule:rule_elem do_absolutely_nothing in
   r.V.rebuilder_rule_elem x
 
 (* --------------------------------------------------------------------- *)
@@ -198,16 +187,8 @@ let find_commonalities res : Ast_cocci.rule_elem option =
 	  bind (one sz e) (k e)
       | _ -> k e in
     let recursor =
-      V.combiner bind option_default
-	mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-	mcode mcode mcode
-	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing
-	expression do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing do_nothing
-	do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-	do_nothing in
+      V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=do_nothing}
+	~expr:expression do_nothing in
     recursor.V.combiner_rule_elem x in
   match res with
     [] -> failwith "bad disjunction"
@@ -283,6 +264,11 @@ and statement testfn mcode tail stmt : 'a list list =
       then conj (rule_elem header) (statement testfn mcode tail branch)
       else statement testfn mcode tail branch
 
+  | Ast.Do(header,branch,whiletail) ->
+     if testfn header || testfn whiletail
+     then conj_wrapped [header;whiletail] (statement testfn mcode tail branch)
+     else statement testfn mcode tail branch
+
   | Ast.Switch(header,lb,decls,cases,rb) ->
       let body_info =
 	conj
@@ -328,6 +314,12 @@ and statement testfn mcode tail stmt : 'a list list =
       then conj (rule_elem header) body_info
       else body_info
 
+  | Ast.TemplateDefinition(header,stmt) ->
+      let body_info = statement testfn mcode true stmt in
+      if testfn header
+      then conj (rule_elem header) body_info
+      else body_info      
+
   | Ast.Define(header,body) ->
       conj_one testfn header (statement_list testfn mcode tail body)
 
@@ -338,8 +330,6 @@ and statement testfn mcode tail stmt : 'a list list =
 
   | Ast.OptStm(stm) -> []
 
-  | _ -> failwith "not supported"
-
 and case_lines testfn mcode tail cases =
   match cases with
     [] -> []
