@@ -91,6 +91,7 @@ let dumb_astcocci_field = function
   | A.CppField _ -> ()
   | A.MetaField _ -> ()
   | A.MetaFieldList _ -> ()
+  | A.AccSpec _ -> ()
 
 let dumb_astcocci_ann_field = function
   | A.FElem(bef,allminus,fld) -> ()
@@ -132,6 +133,7 @@ let dumb_astcocci_expr = function
  | A.DeleteArr (_,_,_,ea) -> ()
  | A.New(_, _, _, _, _, _) -> ()
  | A.TemplateInst(_,_,_,_) -> ()
+ | A.TupleExpr(_) -> ()
  | A.TypeExp (typa) -> ()
  | A.Constructor (_, typa, _, ia) -> ()
  | A.Paren (_, ea, _) -> ()
