@@ -217,6 +217,11 @@ let labels_for_ctl (dropped_isos : string list)
 	  (function node ->
 	    match F.unwrap node with F.Exit | F.EndNode -> true | _ -> false)
 
+    | Lib_engine.PreExit ->
+	loop_nodes p
+	  (function node ->
+	    match F.unwrap node with F.PreExit _ -> true | _ -> false)
+
     | Lib_engine.ErrorExit ->
 	loop_nodes p
 	  (function node ->
@@ -324,7 +329,7 @@ let fix_flow_ctl2 (flow : F.cflow) : F.cflow =
 
   (* for the regular functions *)
   (try
-    let exitnodei  = F.find_node (fun x -> x = F.Exit) !g in
+    let exitnodei  = F.find_node (function F.Exit -> true | _ -> false) !g in
     let errornodei = F.find_node (fun x -> x = F.ErrorExit) !g in
 
     !g#add_arc ((exitnodei, exitnodei), F.Direct);
@@ -440,13 +445,33 @@ module ENV =
 	 Lib_engine.NormalMetaVal(Ast_c.MetaPosVal(min2,max2))) ->
 	   ((min1 <= min2) && (max1 >= max2)) ||
 	   ((min2 <= min1) && (max2 >= max1))
-      |	(Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal a),
-	 Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal b)) ->
+      |	(Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal(b,_))) ->
           C_vs_c.eq_type a b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaExprVal(a,_,_,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaExprVal(b,_,_,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaExprListVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaExprListVal(b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaAttrArgVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaAttrArgVal(b,_))) -> a = b
       | (Lib_engine.NormalMetaVal(Ast_c.MetaStmtVal(a,_,_)),
 	 Lib_engine.NormalMetaVal(Ast_c.MetaStmtVal(b,_,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaStmtListVal(a,_,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaStmtListVal(b,_,_))) -> a = b
       | (Lib_engine.NormalMetaVal(Ast_c.MetaDeclVal(a,_)),
 	 Lib_engine.NormalMetaVal(Ast_c.MetaDeclVal(b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaFieldVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaFieldVal(b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaFieldListVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaFieldListVal(b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaInitVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaInitVal(b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaInitListVal(_,a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaInitListVal(_,b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaParamVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaParamVal(b,_))) -> a = b
+      | (Lib_engine.NormalMetaVal(Ast_c.MetaParamListVal(a,_)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaParamListVal(b,_))) -> a = b
       |	(Lib_engine.LabelVal(l1),Lib_engine.LabelVal(l2)) ->
 	  compatible_labels l1 l2
       |	_ -> v = v'
@@ -461,9 +486,9 @@ module ENV =
 	     if (min2 <= min1) && (max2 >= max1)
 	     then Lib_engine.NormalMetaVal(Ast_c.MetaPosVal(min2,max2))
 	     else failwith "incompatible positions give to merge"
-      |	(Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal a),
-	 Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal b)) ->
-          Lib_engine.NormalMetaVal (Ast_c.MetaTypeVal (C_vs_c.merge_type a b))
+      |	(Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal(a,original)),
+	 Lib_engine.NormalMetaVal(Ast_c.MetaTypeVal(b,_))) ->
+          Lib_engine.NormalMetaVal (Ast_c.MetaTypeVal (C_vs_c.merge_type a b,original))
       |	(Lib_engine.LabelVal(l1),Lib_engine.LabelVal(l2)) ->
 	  Lib_engine.LabelVal(merge_labels l1 l2)
 
@@ -587,16 +612,16 @@ let strip env =
     (function (v,vl) ->
       let vl =
 	match vl with
-	  Ast_c.MetaExprVal (a,c,ty) ->
-	    Ast_c.MetaExprVal(Lib_parsing_c.al_inh_expr a,c,ty)
-	| Ast_c.MetaExprListVal a ->
-	    Ast_c.MetaExprListVal(Lib_parsing_c.al_inh_arguments a)
+	  Ast_c.MetaExprVal (a,original,c,ty) ->
+	    Ast_c.MetaExprVal(Lib_parsing_c.al_inh_expr a,original,c,ty)
+	| Ast_c.MetaExprListVal(a,original) ->
+	    Ast_c.MetaExprListVal(Lib_parsing_c.al_inh_arguments a,original)
 	| Ast_c.MetaStmtVal(a,orig,ty) ->
 	    Ast_c.MetaStmtVal(Lib_parsing_c.al_inh_statement a,orig,ty)
-	| Ast_c.MetaInitVal a ->
-	    Ast_c.MetaInitVal(Lib_parsing_c.al_inh_init a)
-	| Ast_c.MetaInitListVal a ->
-	    Ast_c.MetaInitListVal(Lib_parsing_c.al_inh_inits a)
+	| Ast_c.MetaInitVal(a,original) ->
+	    Ast_c.MetaInitVal(Lib_parsing_c.al_inh_init a,original)
+	| Ast_c.MetaInitListVal (newlines,a,original) ->
+	    Ast_c.MetaInitListVal(newlines,Lib_parsing_c.al_inh_inits a,original)
 	| x -> (*don't contain binding info*) x in
       (v,vl))
     env
@@ -604,6 +629,31 @@ let strip env =
 (*****************************************************************************)
 (* Call ctl engine *)
 (*****************************************************************************)
+
+let same_env e1 e2 =
+  let rec loop e1 e2 =
+    match (e1,e2) with
+      ((x1,x1v)::rest1,(x2,x2v)::rest2) ->
+	ENV.eq_mvar x1 x2 &&
+	ENV.eq_val
+	  (Lib_engine.NormalMetaVal x1v)
+	  (Lib_engine.NormalMetaVal x2v) &&
+	loop rest1 rest2
+    | ([],[]) -> true
+    | _ -> false in
+  loop e1 e2
+
+let same_env2 e1 e2 =
+  let rec loop e1 e2 =
+    match (e1,e2) with
+      ((x1,x1v)::rest1,(x2,x2v)::rest2) ->
+	ENV.eq_mvar x1 x2 &&
+	ENV.eq_val x1v x2v &&
+	loop rest1 rest2
+    | ([],[]) -> true
+    | _ -> false in
+  loop e1 e2
+
 let (mysat2:
   Lib_engine.model ->
   (Lib_engine.ctlcocci * (pred list list)) ->
@@ -619,7 +669,7 @@ let (mysat2:
             !Flag_matcher.allow_inconsistent_paths)
     then Check_reachability.check_reachability rulename triples flow;
     let (trans_info2,used_after_fresh_envs) =
-      Postprocess_transinfo.process used_after binding2 trans_info2 in
+      Postprocess_transinfo.process used_after binding2 same_env2 trans_info2 in
     let used_after_envs =
       Common.nub (List.map2 (@) used_after_fresh_envs used_after_envs) in
     let trans_info = satbis_to_trans_info trans_info2 in
