@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -16,7 +16,7 @@ open Common
 
 open Ast_c
 
-let rec (iso_e_e: expression -> expression -> bool) = fun a b ->
+let (iso_e_e: expression -> expression -> bool) = fun a b ->
   raise Todo
     (*
       let rec (=~=) a b =
