@@ -15,6 +15,14 @@ how we reached a particular match *)
 
 module Ast = Ast_cocci
 
+module ParamsType =
+  struct
+    type t = Ast_c.metavars_binding
+    let compare = Stdcompat.Stdlib.compare
+  end
+
+module ParamsMap = Map.Make(ParamsType)
+
 let extra_counter = ref 0
 
 let reset_fresh_counter () = extra_counter := 0
@@ -47,6 +55,9 @@ let string2val str = Lib_engine.NormalMetaVal(Ast_c.MetaIdVal str)
 (* ----------------------------------------------------------------------- *)
 (* Get values for fresh variables *)
 
+let paramsmap_find_opt key m =
+  try Some(ParamsMap.find key m) with Not_found -> None
+
 let process_tree inherited_env l =
   let (all_fresh,local_freshs,new_triples) =
     List.fold_left
@@ -87,39 +98,50 @@ let process_tree inherited_env l =
 			     failwith
 			       ("fresh: no binding for meta "^(Dumper.dump id)))
 		   seed in
-	    string2val(String.concat "" strings))
+	       string2val(String.concat "" strings))
         | ((r, n) as fresh, Ast.ScriptSeed(name, lang, params, pos, body)) ->
+            let res = ref ParamsMap.empty in
             let make_fresh_id env =
               let args =
                 List.map
                   (fun (((rule, name) as meta_name), _) ->
                     try match List.assoc meta_name env with
-                      | Lib_engine.NormalMetaVal v -> (meta_name, v)
-                      | _ ->
-                          failwith
-                            (Printf.sprintf
-                              "Undesired metavar_binding in line %d" (snd pos))
+                    | Lib_engine.NormalMetaVal v -> (meta_name, v)
+                    | _ ->
+                        failwith
+                          (Printf.sprintf
+                             "Undesired metavar_binding in line %d"
+                             (snd pos))
                     with
                     | Not_found ->
                         let get_meta_names l =
-                          List.map (fun (mn, _) -> Ast.string_of_meta_name mn)
-                          l in
+                          List.map
+                            (fun (mn, _) -> Ast.string_of_meta_name mn)
+                            l in
                         let string_of_list l =
                           "[" ^ String.concat "; " l ^ "]" in
                         failwith
                           (Printf.sprintf
-                            "%s: script on variable %s cannot be evaluated in line %d. available: %s\nwanted: %s"
-                            r n (snd pos) (string_of_list (get_meta_names env))
-                            (string_of_list (get_meta_names params)))
-                  )
+                             "%s: script on variable %s cannot be evaluated in line %d. available: %s\nwanted: %s"
+                             r n (snd pos)
+                             (string_of_list (get_meta_names env))
+                             (string_of_list (get_meta_names params)))
+                          )
                   params in
-              let args = (fresh, Ast_c.MetaIdVal n)::args in
-              let fresh_id =
-                match lang with
-                | "ocaml" -> Run_ocamlcocci.run_fresh_id name (List.map snd args)
-                | "python" -> Pycocci.run_fresh_id args pos body
-                | _ -> failwith "languages other than ocaml or python not supported" in
-              string2val fresh_id in
+              match paramsmap_find_opt args !res with
+		Some x -> x
+	      | None ->
+		  let fresh_id =
+                    match lang with
+                    | "ocaml" ->
+			Run_ocamlcocci.run_fresh_id name (List.map snd args)
+                    | "python" -> Pycocci.run_fresh_id args pos body
+                    | _ ->
+			failwith
+			  "languages other than ocaml or python not supported" in
+		  let r = string2val fresh_id in
+                  res := ParamsMap.add args r !res;
+		  r in
             (fresh, make_fresh_id)
       )
       all_fresh in
@@ -176,7 +198,9 @@ let fold_left_with_index f acc =
 	fold_lwi_aux (f acc x n) xs
   in fold_lwi_aux acc
 
-let numberify trees =
+let numberify same_env trees =
+  let same_env (a1,env1,b1) (a2,env2,b2) =
+    a1 = a2 && b1 = b2 && same_env env1 env2 in
   let trees =
     fold_left_with_index
       (function acc -> function xs -> function n ->
@@ -185,7 +209,7 @@ let numberify trees =
   List.fold_left
     (function res ->
       function (n,x) ->
-	let (same,diff) = List.partition (function (ns,xs) -> x = xs) res in
+	let (same,diff) = List.partition (function (ns,xs) -> same_env x xs) res in
 	match same with
 	  [(ns,xs)] -> (n::ns,xs)::diff
 	| _ -> ([n],x)::res)
@@ -194,8 +218,8 @@ let numberify trees =
 (* ----------------------------------------------------------------------- *)
 (* entry point *)
 
-let process used_after inherited_env l =
+let process used_after inherited_env same_env l =
   let (trees, fresh_envs) =
     List.split (List.map (process_tree inherited_env) l) in
-  let trees = numberify trees in
+  let trees = numberify same_env trees in
   (trees, collect_used_after used_after fresh_envs l inherited_env)
