@@ -20,6 +20,7 @@ type xinfo = {
   value_format_iso : bool;
   optional_declarer_semicolon_iso : bool;
   optional_attributes_iso : bool;
+  list_and_aggregate_initialization_iso : bool;
   current_rule_name : string; (* used for errors *)
   index : int list (* witness tree indices *)
 }
@@ -106,6 +107,9 @@ module XTRANS = struct
   let optional_attributes_flag f = fun tin ->
     f (tin.extra.optional_attributes_iso) tin
 
+  let list_and_aggregate_initialization_flag f = fun tin ->
+    f (tin.extra.list_and_aggregate_initialization_iso) tin
+
   let mode = Cocci_vs_c.TransformMode
 
   (* ------------------------------------------------------------------------*)
@@ -232,15 +236,15 @@ module XTRANS = struct
       Visitor_c.kdecl_s = (fun (k, bigf) expb ->
 	let iif ii = List.map (Visitor_c.vk_info_s bigf) ii in
 	match expb with
-	  Ast_c.DeclList(xs,iis) ->
+	  Ast_c.DeclList((xs, has_ender),iis) ->
 	    (match xs with
 	      (x,ii)::xs ->
 		Ast_c.DeclList
-		  ((Visitor_c.vk_onedecl_s bigf x,iif ii) ::
+		  (((Visitor_c.vk_onedecl_s bigf x,iif ii) ::
 		   (List.map
 		      (fun (x,ii) ->
 			(Visitor_c.vk_onedecl_opt_s false bigf x,iif ii))
-		      xs),
+		      xs), has_ender),
 		   iif iis)
 	    | _ -> failwith "no decls")
 	| _ -> k expb);
@@ -893,6 +897,8 @@ let (transform2: string (* rule name *) -> string list (* dropped_isos *) ->
        not(List.mem "optional_declarer_semicolon"   dropped_isos);
      optional_attributes_iso =
        not(List.mem "optional_attributes" dropped_isos);
+     list_and_aggregate_initialization_iso =
+       not(List.mem "list_and_aggregate_initialization" dropped_isos);
      current_rule_name = rule_name;
      index = [];
    } in
