@@ -16,7 +16,9 @@ let track_iso_usage = ref false
 
 let worth_trying_opt = ref true
 
-type scanner = IdUtils | Glimpse | CocciGrep | GitGrep | NoScanner
+type scanner =
+    IdUtils | Glimpse | CocciGrep | GitGrep
+  | PatchDiff | PatchDiffRange of string * string | NoScanner
 let scanner = ref NoScanner
 
 let pyoutput = ref "coccilib.output.Console"
@@ -26,6 +28,8 @@ let ocamlopt = ref Commands.ocamlopt_cmd
 let ocamldep = ref Commands.ocamldep_cmd
 let ocamlfind = ref Commands.ocamlfind_cmd
 
+let ocaml_lib_search_path = ref ([]: string list)
+
 (*"Some" value is the path with respect to which the patch should be created*)
 let patch = ref (None : string option)
 
@@ -50,6 +54,10 @@ let set_defined_virtual_rules s =
       defined_virtual_env := (name,vl) :: !defined_virtual_env
   | _ -> failwith "nothing defined"
 
+let reset_virt _ =
+  defined_virtual_rules := [];
+  defined_virtual_env := []
+
 type c_plus_plus = Off | On of int option (* release year *)
 let c_plus_plus = ref Off
 
@@ -63,6 +71,7 @@ let set_c_plus_plus version =
     | Some "1y" | Some "14" -> Some 2014
     | Some "1z" | Some "17" -> Some 2017
     | Some "2a" | Some "20" -> Some 2020
+    | Some "23" -> Some 2023
     | Some version -> failwith ("Invalid C++ version: " ^ version) in
   c_plus_plus := On version
 
