@@ -4,13 +4,16 @@ val show_transinfo : bool ref
 val show_trying : bool ref
 val track_iso_usage : bool ref
 val worth_trying_opt : bool ref
-type scanner = IdUtils | Glimpse | CocciGrep | GitGrep | NoScanner
+type scanner =
+    IdUtils | Glimpse | CocciGrep | GitGrep
+  | PatchDiff | PatchDiffRange of string * string | NoScanner
 val scanner : scanner ref
 val pyoutput : string ref
 val ocamlc : string ref
 val ocamlopt : string ref
 val ocamldep : string ref
 val ocamlfind : string ref
+val ocaml_lib_search_path : string list ref
 val patch : string option ref
 val make_hrule : string option ref
 val hrule_per_file : bool ref
@@ -22,6 +25,7 @@ val dir : string ref
 val defined_virtual_rules : string list ref
 val defined_virtual_env : (string * string) list ref
 val set_defined_virtual_rules : string -> unit
+val reset_virt : unit -> unit (* for testing *)
 type c_plus_plus = Off | On of int option (* release year of the version *)
 val c_plus_plus : c_plus_plus ref
 val set_c_plus_plus : string option -> unit
