@@ -33,8 +33,3 @@ val get_pending_instance : unit -> pending_info option
 
 (* for testing *)
 val clear_pending_instance : unit -> unit
-
-(* ----------------------------------------------------------------------- *)
-
-val check_virtual_rule : string -> unit
-val check_virtual_ident : string -> unit
