@@ -11,7 +11,7 @@ module Lib_parsing_c  = Lib_parsing_c  (* parsing_c/lib_parsing_c.ml *)
 module Visitor_c      = Visitor_c      (* parsing_c/visitor_c.ml *)
 
 module Regexp     = Regexp     (* globals/regexp.ml *)
-module Config     = Config     (* globals/config.ml *)
+module Cocciconfig = Cocciconfig     (* globals/cocciconfig.ml *)
 module Flag       = Flag       (* globals/flag.ml *)
 module Flag_parsing_c = Flag_parsing_c       (* parsing_c/flag_parsing_c.ml *)
 module Iteration  = Iteration  (* globals/iteration.ml *)
