@@ -6,9 +6,9 @@ open Common
 let caller s f a =
   let str = ref ([] : string list) in
   let pr_elem info = str := (Ast_c.str_of_info info) :: !str in
-  let pr_sp _ = () in
+  let pr_sp _ = str := " " :: !str in
   f ~pr_elem ~pr_space:pr_sp a;
-  String.concat s (List.rev !str)
+  String.concat "" (List.rev !str)
 
 let call_pretty f a = caller " " f a
 let call_pretty0 f a = caller "" f a
