@@ -1,6 +1,7 @@
+val ocaml_support : bool
 
 exception CompileFailure of string
-exception LinkFailure of string
+exception LinkFailure of string * string
 
 val prepare : string -> Ast_cocci.rule list -> string option
 val prepare_simple : string -> string
