@@ -67,9 +67,7 @@ and field =
     DeclarationField of field_declaration
   | EmptyField of info
   | FunctionField of definition (* C++ *)
-  | PublicLabel of info list (* C++ *)
-  | ProtectedLabel of info list (* C++ *)
-  | PrivateLabel of info list (* C++ *)
+  | AccSpec of info list (* C++ *)
   | ConstructDestructField of c_plus_plus_constructor
   | MacroDeclField of (string * argument wrap2 list * attribute list) wrap
   | MacroDeclFieldInit of (string * argument wrap2 list * attribute list * initialiser) wrap
@@ -132,6 +130,7 @@ and expressionbis =
   | New of (argument wrap2 list) option * fullType * (argument wrap2 list) option
   | Delete of bool (* true if [] *) * expression
   | TemplateInst of expression * argument wrap2 list
+  | TupleExpr of initialiser
   | Defined of name
 and argument = (expression, weird_argument) Common.either
 and weird_argument = ArgType of parameterType | ArgAction of action_macro
@@ -202,6 +201,9 @@ and declOrExpr =
   | ForExp of
       expression option wrap * exprStatement wrap * exprStatement wrap
   | ForRange of declaration * initialiser
+and whileDeclOrExpr =
+    WhileDecl of declaration
+  | WhileExp of expression
 and selection =
     If of expression * statement * statement
   | Switch of expression * statement
@@ -209,7 +211,7 @@ and selection =
   | Ifdef_Ite2 of expression * statement * statement * statement
   | TryCatch of statement * (parameterType * statement) wrap list
 and iteration =
-    While of expression * statement
+    While of whileDeclOrExpr * statement
   | DoWhile of statement * expression
   | For of declOrExpr * statement
   | MacroIteration of string * argument wrap2 list * statement
@@ -227,7 +229,7 @@ and colon_option_bis = ColonMisc | ColonExpr of expression
 and exec_code_bis = ExecEval of expression | ExecToken
 and exec_code = exec_code_bis wrap
 and declaration =
-    DeclList of onedecl wrap2 list wrap
+    DeclList of (onedecl wrap2 list * bool) wrap
   | MacroDecl of
       (storagebis * attribute list * string *
 	 argument wrap2 list * attribute list * bool)
@@ -300,7 +302,7 @@ and base_class = base_class_bis wrap
 and cpp_directive =
     Define of define
   | Include of includ
-  | Pragma of (name * string wrap list) wrap
+  | Pragma of (name * info) wrap
   | OtherDirective of il
   | UsingTypename of (name * fullType) wrap
   | UsingNamespace of name wrap
@@ -421,7 +423,6 @@ val nullQualif : typeQualifierbis * 'a list
 val nQ : typeQualifierbis * 'a list
 val defaultInt : typeCbis
 val noType : unit -> ('a option * test) ref
-val noInstr : statementbis * 'a list
 val noTypedefDef : unit -> 'a option
 val emptyMetavarsBinding : metavars_binding
 val emptyAnnotCocci : Ast_cocci.mcodekind * metavars_binding list
@@ -446,7 +447,6 @@ val unwrap_expr : ('a * 'b) * 'c -> 'a
 val rewrap_expr : ('a * 'b) * 'c -> 'd -> ('d * 'b) * 'c
 val unwrap_typeC : fullType -> typeCbis
 val rewrap_typeC : fullType -> typeCbis -> fullType
-val unwrap_typeCbis : typeC -> typeCbis
 val unwrap_st : 'a * 'b -> 'a
 val mk_e : 'a -> 'b -> ('a * ('c option * test) ref) * 'b
 val mk_e_bis : 'a -> 'b -> 'c -> ('a * 'b) * 'c
@@ -455,23 +455,19 @@ val mk_tybis : 'a -> 'b -> 'a * 'b
 val mk_st : 'a -> 'b -> 'a * 'b
 val get_ii_typeC_take_care : 'a * 'b -> 'b
 val get_ii_st_take_care : 'a * 'b -> 'b
-val get_ii_expr_take_care : 'a * 'b -> 'b
 val get_st_and_ii : 'a * 'b -> 'a * 'b
 val get_ty_and_ii : 'a * attribute list * ('b * 'c) -> 'b * 'c
 val get_e_and_ii : 'a * 'b -> 'a * 'b
 val get_type_expr : ('a * 'b ref) * 'c -> 'b
 val set_type_expr : ('a * 'b ref) * 'c -> 'b -> unit
 val get_onlytype_expr : ('a * (('b * 'c) option * 'd) ref) * 'e -> 'b option
-val get_onlylocal_expr : ('a * (('b * 'c) option * 'd) ref) * 'e -> 'c option
 val rewrap_str : string -> info -> info
 val rewrap_charpos : int -> info -> info
 val rewrap_col : int -> info -> info
 val rewrap_pinfo : parse_info -> info -> info
-val get_pi : parse_info -> Common.parse_info
 val get_opi : parse_info -> Common.parse_info
 val str_of_info : info -> string
 val get_info : (Common.parse_info -> 'a) -> info -> 'a
-val get_orig_info : (Common.parse_info -> 'a) -> info -> 'a
 val make_expanded : info -> info
 val pos_of_info : info -> int
 val opos_of_info : info -> int
@@ -490,11 +486,10 @@ val equal_posl : 'a * 'b -> 'a * 'b -> bool
 val compare_posl : int * int -> int * int -> int
 val info_to_fixpos : info -> Ast_cocci.fixpos
 val is_test : expression -> bool
+val al_info_of_string : int -> string -> info
 val al_info : int -> info -> info
 val semi_al_info : info -> info
-val magic_real_number : int
 val real_al_info : info -> info
-val al_comments : bool -> comments_around -> comments_around
 val al_info_cpp : int -> info -> info
 val semi_al_info_cpp : info -> info
 val real_al_info_cpp : bool -> info -> info
@@ -504,19 +499,11 @@ val split_nocomma : 'a list -> ('a, il) Common.either list
 val unsplit_nocomma : ('a, il) Common.either list -> 'a list
 val s_of_inc_file : inc_file -> string
 val s_of_inc_file_bis : inc_file -> string
-val fieldname_of_fieldkind : fieldkind -> name option
 val s_of_attr : (attributebis * info list) list -> string
 val str_of_name : name -> string
-val get_s_and_ii_of_name : name -> string * il
 val get_s_and_info_of_name : name -> string * info
 val info_of_name : name -> info
-val ii_of_name : name -> il
-val get_local_ii_of_expr_inlining_ii_of_name :
-  (expressionbis * 'a) * il -> il
-val get_local_ii_of_tybis_inlining_ii_of_name : typeCbis * il -> il
 val info_of_type : 'a * attribute list * (typeCbis * il) -> parse_info option
-val get_local_ii_of_st_inlining_ii_of_name :
-  statementbis * info list -> info list
 val name_of_parameter : parameterType -> string option
 val put_annot_info :
   info -> Token_annot.annot_key -> Token_annot.annot_val -> unit
@@ -524,5 +511,3 @@ val get_annot_info :
   info -> Token_annot.annot_key -> Token_annot.annot_val option
 val get_comments_before : info -> Token_c.comment_like_token list
 val get_comments_after : info -> Token_c.comment_like_token list
-val string_of_toplevel : toplevel -> string
-val string_of_inc_file : inc_file -> string
