@@ -3,7 +3,6 @@ val ast_to_control_flow :
     Ast_c.toplevel -> (Ast_c.toplevel outer * Control_flow_c.cflow option) list
 
 val deadcode_detection : Control_flow_c.cflow -> unit
-val check_control_flow : Control_flow_c.cflow -> unit
 
 val annotate_loop_nodes : Control_flow_c.cflow -> Control_flow_c.cflow
 
@@ -22,5 +21,3 @@ type error =
 exception Error of error
 
 val report_error : error -> unit
-
-val realstring : bool ref
