@@ -4,8 +4,6 @@ type cpp_option =
 
 val cpp_option_of_cmdline:
   Common.dirname list (* -I *) * string list (* -D *) -> cpp_option list
-val show_cpp_i_opts: string list -> unit
-val show_cpp_d_opts: string list -> unit
 
 
 (* ---------------------------------------------------------------------- *)
@@ -25,8 +23,6 @@ val _headers_hash:
 val _hcandidates:
   (string, Common.filename) Hashtbl.t
 
-val init_adjust_candidate_header_files: Common.dirname -> unit
-
 (* ---------------------------------------------------------------------- *)
 
 (* #include *)
@@ -35,8 +31,3 @@ val cpp_expand_include:
   ?threshold_cache_nb_files:int ->
   cpp_option list -> Common.dirname (* start point for relative paths *) ->
   Ast_c.program -> Ast_c.program
-
-(* #define *)
-val cpp_expand_macro_expr:
-  Ast_c.define_kind -> Ast_c.argument Ast_c.wrap2 list ->
-  Ast_c.expression option
