@@ -83,7 +83,7 @@ let assoc_hint_string = [
   "YACFE_STRING"     , HintMacroString;
   "YACFE_STATEMENT"  , HintMacroStatement;
   "YACFE_ATTRIBUTE"  , HintAttribute;
-  "YACFE_END_ATTRIBUTE" , HintEndAttribute;
+  "YACFE_END_ATTRIBUTE" , HintAttribute;
   "YACFE_IDENT_BUILDER" , HintMacroIdentBuilder;
 
   "MACROANNOTATION"   , HintAttribute;
@@ -104,8 +104,8 @@ let (is_parsinghack_hint: string -> bool) = fun s ->
   parsinghack_hint_of_string s <> None
 
 let (token_from_parsinghack_hint:
-     (string * Ast_c.info) -> parsinghack_hint -> Parser_c.token) =
- fun (s,ii) hint ->
+     bool -> (string * Ast_c.info) -> parsinghack_hint -> Parser_c.token) =
+ fun hasargs (s,ii) hint ->
    match hint with
    | HintIterator ->
        Parser_c.TMacroIterator (s, ii)
@@ -116,7 +116,9 @@ let (token_from_parsinghack_hint:
    | HintMacroStatement ->
        Parser_c.TMacroStmt (s, ii)
    | HintAttribute ->
-       Parser_c.TMacroAttr (s, ii)
+       if hasargs
+       then Parser_c.TMacroAttrArgs (s, ii)
+       else Parser_c.TMacroAttr (s, ii)
    | HintEndAttribute -> failwith "not supported"
    | HintMacroIdentBuilder ->
        Parser_c.TMacroIdentBuilder (s, ii)
@@ -206,24 +208,13 @@ let rec get_ident_in_concat_op xs =
   | [] ->
       pr2_once "weird: ident after ## operator not found";
       "", []
-  | [x] ->
-      (match x with
-      | Parser_c.TIdent (s, i1) -> s, []
-      | _ ->
-          pr2_once "weird: ident after ## operator not found";
-          "", [x]
-      )
+  | [x] -> TH.str_of_tok x, []
   | x::y::xs ->
-      (match x, y with
-      | Parser_c.TIdent (s,i1), Parser_c.TCppConcatOp (i2) ->
+      (match y with
+	Parser_c.TCppConcatOp (i2) ->
           let (s2, rest) = get_ident_in_concat_op xs in
-          s ^ s2, rest
-      | Parser_c.TIdent (s, i1), _ ->
-          s, (y::xs)
-      | _ ->
-          pr2_once "weird: ident after ## operator not found";
-          "", x::y::xs
-      )
+          TH.str_of_tok x ^ s2, rest
+      | _ -> TH.str_of_tok x, (y::xs))
 
 (* must be run after the expansion has been done for the parameter so
  * that all idents are actually ident, not macro parameter names.
@@ -390,7 +381,7 @@ let apply_macro_defs
           | DefineHintBody (hint,  _, {contents = false})
           | DefineHint hint ->
               msg_apply_known_macro_hint s;
-              id.tok <- token_from_parsinghack_hint (s,i1) hint;
+              id.tok <- token_from_parsinghack_hint false (s,i1) hint;
           )
       | Params params ->
           (match body with
@@ -429,7 +420,10 @@ let apply_macro_defs
 	      |	Some bind ->
 
                   let xxs' = xxs +> List.map (fun x ->
-		    (tokens_of_paren_ordered x) +> List.map (fun x ->
+		    (tokens_of_paren_ordered x) +>
+		    (* no need to expand fake tokens, just drop them *)
+		    List.filter (fun x -> not(TH.is_fake x.tok)) +>
+		    List.map (fun x ->
 		      TH.visitor_info_of_tok Ast_c.make_expanded x.tok
 			)
 		      ) in
@@ -501,7 +495,7 @@ let apply_macro_defs
             | DefineHintBody (hint,  _, {contents = false})
             | DefineHint hint ->
                 msg_apply_known_macro_hint s;
-                id.tok <- token_from_parsinghack_hint (s,i1) hint;
+                id.tok <- token_from_parsinghack_hint true (s,i1) hint;
             )
       );
       (match body with
@@ -552,8 +546,8 @@ let apply_macro_defs
   | PToken ({tok = TIdent (s,i1)} as id)::
     PToken ({tok = TPtVirg _} | {tok = TEq _})::xs
       when List.mem s !Flag.cocci_attribute_names ->
-	id.tok <- TMacroEndAttr (s, i1);
-	apply_macro_defs dynamic_macs pos xs
+        id.tok <- TMacroAttr (s, i1);
+        apply_macro_defs dynamic_macs pos xs
 
   | PToken ({tok = TIdent (s,i1)} as id)::xs
       when List.mem s !Flag.cocci_attribute_names ->
@@ -588,7 +582,7 @@ let apply_macro_defs
           | DefineHintBody (hint,  _, {contents = false})
           | DefineHint hint ->
                 msg_apply_known_macro_hint s;
-                id.tok <- token_from_parsinghack_hint (s,i1) hint;
+                id.tok <- token_from_parsinghack_hint false (s,i1) hint;
           )
       );
       apply_macro_defs dynamic_macs pos xs
@@ -620,7 +614,7 @@ let macro_body_to_maybe_hint body =
       let lexbuf_fake = Lexing.from_function (fun _buf _n -> assert false) in
       let read_tokens tokens _lexbuf =
         match !tokens with
-        | [] -> TDefEOL (Ast_c.fakeInfo ())
+        | [] -> TDefEOL (Ast_c.fakeAfterInfo())
         | tok::toks ->
             tokens := toks;
             tok
@@ -634,17 +628,20 @@ let macro_body_to_maybe_hint body =
        * Currently, the get_macro_* functions rely on the parser to verify if
        * the body belongs to certain primary constructs *)
       let get_macro_iterator () =
-        let macro_tokens = body@[TPtVirg (Ast_c.fakeInfo())] in
-        try
-          match Parser_c.iteration (read_tokens (ref macro_tokens)) lexbuf_fake with
-          | (Ast_c.While _ | Ast_c.For _), _ ->
-              Some HintIterator
-          | _ -> None
-        with _ -> None
+	match body with
+	  (Twhile _ | Tfor _)::_ -> (* need these tokns to have an iterator *)
+            let macro_tokens = body@[TPtVirg (Ast_c.fakeAfterInfo())] in
+            (try
+              match Parser_c.iteration (read_tokens (ref macro_tokens)) lexbuf_fake with
+              | (Ast_c.While _ | Ast_c.For _), _ ->
+		  Some HintIterator
+              | _ -> None
+            with _ -> None)
+	| _ -> None
       in
       let get_macro_stmt () =
         let macro_tokens =
-          TDefine (Ast_c.fakeInfo())::TIdentDefine ("Fake ident", Ast_c.fakeInfo())::body
+          TDefine (Ast_c.fakeBeforeInfo())::TIdentDefine ("Fake ident", Ast_c.fakeAfterInfo())::body
         in
         try
           match Parser_c.cpp_directive (read_tokens (ref macro_tokens)) lexbuf_fake with
@@ -653,16 +650,22 @@ let macro_body_to_maybe_hint body =
           | _ -> None
         with _ -> None
       in
-      match Common.find_map (fun f -> f()) [get_macro_iterator; get_macro_stmt] with
-      | Some hint -> DefineHintBody (hint, body, ref false)
-      | None -> DefineBody body
+      let save_verbose_parsing = !Flag_parsing_c.verbose_parsing in
+      Flag_parsing_c.verbose_parsing := false;
+      let res =
+	match Common.find_map (fun f -> f()) [get_macro_iterator; get_macro_stmt] with
+	| Some hint -> DefineHintBody (hint, body, ref false)
+	| None -> DefineBody body in
+      Flag_parsing_c.verbose_parsing := save_verbose_parsing;
+      res
 
 exception Bad_param
 
-let rec (define_parse: Parser_c.token list -> (string * define_def) list) =
- fun xs ->
+let rec (define_parse: (string * define_def) list -> Parser_c.token list ->
+                       (string * define_def) list) =
+ fun acc xs ->
   match xs with
-  | [] -> []
+  | [] -> List.rev acc
   | TDefine i1::TIdentDefine (s,i2)::TOParDefine i3::xs ->
       (* note: the macro could be badly written and have no closing ')' for
        * its param, which would make us go too far away, but I don't think
@@ -674,28 +677,32 @@ let rec (define_parse: Parser_c.token list -> (string * define_def) list) =
 	  let (body, _, xs) =
             xs +> Common.split_when (function TDefEOL _ -> true | _ -> false) in
 	  let params =
-            tokparams +> Common.map_filter (function
-              |	 TComma _ -> None
-              |	 TIdent (s, _) -> Some (FixedArg s)
+	    let rec loop = function
+		[] -> []
+	      | TComma _ :: rest -> loop rest
+                (* TODO *)
+	      | (TIdent (s, _)|TypedefIdent (s, _)) :: TEllipsis _ :: rest ->
+		  (VariadicArg (s^"...")) :: loop rest
+                (* TODO *)
+	      | TEllipsis _ :: rest -> (VariadicArg "...") :: loop rest
 
-              (* TODO *)
-              |	 TDefParamVariadic (s, _) -> Some (VariadicArg s)
-              (* TODO *)
-              |	 TEllipsis _ -> Some (VariadicArg "...")
+	      | TIdent (s, _) :: rest -> (FixedArg s) :: loop rest
 
-              |	 x ->
-              (* bugfix: param of macros can be tricky *)
-                  let s = TH.str_of_tok x in
-                  if s ==~ Common.regexp_alpha
-                  then begin
-                    pr2 (spf "remapping: %s to a macro parameter" s);
-                    Some (FixedArg s)
-                  end
-                  else
-                    begin
-                      pr2 (spf "bad character %s in macro parameter list" s);
-                      raise Bad_param
-                    end) in
+	      | x::rest ->
+                  (* bugfix: param of macros can be tricky *)
+		  let s = TH.str_of_tok x in
+		  if s ==~ Common.regexp_alpha
+		  then
+		    begin
+		      pr2 (spf "remapping: %s to a macro parameter" s);
+		      (FixedArg s) :: loop rest
+		    end
+		  else
+		    begin
+		      pr2 (spf "bad character %s in macro parameter list" s);
+		      raise Bad_param
+		    end in
+	    loop tokparams in
           (* bugfix: also substitute to ident in body so cpp_engine will
              * have an easy job.
           *)
@@ -719,8 +726,8 @@ let rec (define_parse: Parser_c.token list -> (string * define_def) list) =
 	  Some (s, (s, Params params, macro_body_to_maybe_hint body))
 	with Bad_param -> None in
       (match def with
-	Some def -> def::define_parse xs
-      |	None -> define_parse xs)
+	Some def -> define_parse (def::acc) xs
+      | None -> define_parse acc xs)
 
   | TDefine i1::TIdentDefine (s,i2)::xs ->
       let (body, _, xs) =
@@ -728,15 +735,15 @@ let rec (define_parse: Parser_c.token list -> (string * define_def) list) =
       let body = body +> List.map
         (TH.visitor_info_of_tok Ast_c.make_expanded) in
       let def = (s, (s, NoParam, macro_body_to_maybe_hint body)) in
-      def::define_parse xs
+      define_parse (def::acc) xs
 
   (* cf tests-bis/define_plus.c *)
   | TDefine i1::xs ->
       let line = Ast_c.line_of_info i1 in
       pr2 (spf "WEIRD: no ident in define at line %d" line);
-      define_parse xs
+      define_parse acc xs
 
-  | x::xs -> define_parse xs
+  | x::xs -> define_parse acc xs
 
 
 
@@ -744,4 +751,4 @@ let extract_macros xs =
   let cleaner = xs +> List.filter (fun x ->
     not (TH.is_comment x)
   ) in
-  define_parse cleaner
+  define_parse [] cleaner
