@@ -213,7 +213,8 @@ let mk_pretty_printers
         pp_expression name; pr_elem lab;
 	pp_arg_list es;
         pr_elem rab
-
+    | TupleExpr(init), [] ->
+	pp_init init
     | Defined name, [i1] ->
         pr_elem i1; (* defined *) pr_space();
         pp_name name;
@@ -229,7 +230,7 @@ let mk_pretty_printers
     | ArrayAccess (_,_) | RecordAccess (_,_) | RecordPtAccess (_,_) | QualifiedAccess(_,_)
     | SizeOfExpr (_) | SizeOfType (_) | Cast (_,_)
     | StatementExpr (_) | Constructor _
-    | ParenExpr (_) | New (_) | Delete (_,_) | TemplateInst(_,_)
+    | ParenExpr (_) | New (_) | Delete (_,_) | TemplateInst(_,_) | TupleExpr(_)
     | Defined (_)),_ -> raise (Impossible 95)
     );
 
@@ -394,9 +395,12 @@ and pp_string_format (e,ii) =
     | Selection  (Switch (e, st)), [i1;i2;i3;iifakend] ->
         pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3;
 	indent_if_needed st (function _-> pp_statement st); pr_elem iifakend
-    | Iteration  (While (e, st)), [i1;i2;i3;iifakend] ->
+    | Iteration  (While (WhileExp (e), st)), [i1;i2;i3;iifakend] ->
         pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3;
 	indent_if_needed st (function _-> pp_statement st); pr_elem iifakend
+    | Iteration  (While (WhileDecl (d), st)), [i1;i2;i3;iifakend] ->
+        pr_elem i1; pr_space(); pr_elem i2; pp_decl d; pr_elem i3;
+	indent_if_needed st (function _-> pp_statement st); pr_elem iifakend
     | Iteration  (DoWhile (st, e)), [i1;i2;i3;i4;i5;iifakend] ->
         pr_elem i1;
 	indent_if_needed st (function _ -> pp_statement st);
@@ -929,7 +933,7 @@ and pp_string_format (e,ii) =
 
     (* C++ *)
     | FunctionField def -> pp_def def
-    | PublicLabel ii | ProtectedLabel ii | PrivateLabel ii ->
+    | AccSpec ii ->
 	let (kwd,dotdot) = Common.tuple_of_list2 ii in
 	pr_elem kwd; pr_elem dotdot
     | ConstructDestructField cd -> pp_construct_destruct cd
@@ -1177,14 +1181,17 @@ and pp_string_format (e,ii) =
                    v_storage = storage;
                    v_attr = attrs;
                    v_endattr = endattrs;
-                  },[])::xs),
-	       iivirg::ifakestart::iisto) ->
-
-	pr_elem ifakestart;
+                  },[])::xs, has_ender),
+	       vfs) ->
 
+        let (iivirg,ifakestart,iisto) =
+          match vfs with
+            iivirg::ifakestart::iisto when has_ender -> (Some iivirg,ifakestart,iisto)
+          |         ifakestart::iisto -> (None,ifakestart,iisto)
+	  | _ -> failwith "UsingTypename: wrong number of elements" in
+        pr_elem ifakestart;
         (* old: iisto +> List.iter pr_elem; *)
 
-
         (* handling the first var. Special case, we print the whole type *)
 	firstvar attrs storage var returnType endattrs iisto;
 
@@ -1199,7 +1206,7 @@ and pp_string_format (e,ii) =
 	       assert (storage2 = storage);
 	       restvar iivirg var returnType attrs endattrs);
 
-	pr_elem iivirg;
+	do_option pr_elem iivirg;
 
     | MacroDecl
       ((sto, preattrs, s, es, attrs, true), iis::lp::rp::iiend::ifakestart::iisto) ->
@@ -1487,8 +1494,7 @@ and pp_init (init, iinit) =
     | Pragma((name,rest), ii) ->
 	let (ipragma,ieol) = Common.tuple_of_list2 ii in
 	pr_elem ipragma; pr_space();
-	pp_name name; pr_space();
-	List.iter (List.iter pr_elem) (List.map snd rest); pr_elem ieol
+	pp_name name; pr_space(); pr_elem rest; pr_elem ieol
 
     | OtherDirective (ii) ->
 	List.iter pr_elem ii
@@ -1609,9 +1615,12 @@ and pp_init (init, iinit) =
     | F.SwitchHeader (_, (e,ii)) ->
 	let (i1,i2,i3) = tuple_of_list3 ii in
 	pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3
-    | F.WhileHeader (_, (e,ii)) ->
+    | F.WhileHeader (_, (WhileExp e,ii)) ->
 	let (i1,i2,i3) = tuple_of_list3 ii in
 	pr_elem i1; pr_space(); pr_elem i2; pp_expression e; pr_elem i3
+    | F.WhileHeader (_, (WhileDecl d,ii)) ->
+	let (i1,i2,i3) = tuple_of_list3 ii in
+	pr_elem i1; pr_space(); pr_elem i2; pp_decl d; pr_elem i3
     | F.DoWhileTail (e,ii) ->
 	let (i1,i2,i3,i4) = tuple_of_list4 ii in
 	pr_elem i1; pr_elem i2; pp_expression e;
@@ -1926,18 +1935,12 @@ let pp_define_param_list_gen ~pr_elem ~pr_space =
 let pp_type_gen ~pr_elem ~pr_space =
   (pp_elem_sp ~pr_elem:pr_elem ~pr_space:pr_space).ty
 
-let pp_type_with_ident_gen pr_elem pr_space =
-  (pp_elem_sp ~pr_elem:pr_elem ~pr_space:pr_space).type_with_ident
-
 let pp_string_fragment_list_gen ~pr_elem ~pr_space =
   (pp_elem_sp ~pr_elem:pr_elem ~pr_space:pr_space).fragment_list
 
 let pp_string_format_gen ~pr_elem ~pr_space =
   (pp_elem_sp ~pr_elem:pr_elem ~pr_space:pr_space).format
 
-let pp_attribute_gen ~pr_elem ~pr_space =
-  (pp_elem_sp ~pr_elem:pr_elem ~pr_space:pr_space).attribute
-
 let pp_attr_arg_gen ~pr_elem ~pr_space =
   (pp_elem_sp ~pr_elem:pr_elem ~pr_space:pr_space).attr_arg
 
@@ -1962,21 +1965,11 @@ let string_of_flow n =
     pp_flow_simple n
   )
 
-let string_of_toplevel top =
-  Common.format_to_string (fun () ->
-    pp_toplevel_simple top
-  )
-
 let string_of_fullType t =
   Common.format_to_string (fun () ->
     pp_type_simple t
   )
 
-let string_of_name name =
-  Common.format_to_string (fun () ->
-    pp_name name
-  )
-
 let (debug_info_of_node:
        Control_flow_c.G.key -> Control_flow_c.cflow -> string) =
   fun nodei flow ->
