@@ -194,7 +194,7 @@ let test_parse_gen xs ext =
   end;
   ()
 
-
+(* parse and handle some regression information when called with dirmode *)
 let test_parse_c xs =
   test_parse_gen xs "c"
 let test_parse_h xs =
@@ -437,7 +437,7 @@ let test_attributes file =
     );
     Visitor_c.kdecl = (fun (k, bigf) decl ->
       match decl with
-      | DeclList (xs, ii) ->
+      | DeclList ((xs, has_ender), ii) ->
           xs +> List.iter (fun (onedecl, iicomma) ->
 
             let sattr  = Ast_c.s_of_attr onedecl.v_attr in
