@@ -316,7 +316,7 @@ let info_of_tok = function
 
   | TUndef (ii) -> ii
   | TPrePragma (ii,_,_,_,_,_) -> ii
-  | TPragmaString (s,ii) -> ii
+  | TPragmaString ii -> ii
   | TPragma (ii) -> ii
   | TCppDirectiveOther (ii) -> ii
 
@@ -517,9 +517,8 @@ let visitor_info_of_tok f = function
   | TUndef (i1) -> TUndef(f i1)
   | TPragma (i1) -> TPragma(f i1)
   | TPrePragma (i1,wss1,a,b,wss2,c) ->
-      TPrePragma(f i1,f wss1,a,f b,f wss2,
-		 List.map (fun (c,d) -> (c,f d)) c)
-  | TPragmaString (s,ii) -> TPragmaString(s,f ii)
+      TPrePragma(f i1,f wss1,a,f b,f wss2,f c)
+  | TPragmaString ii -> TPragmaString(f ii)
   | TCppDirectiveOther (i1) -> TCppDirectiveOther(f i1)
 
   | TInclude (includes, filename, inifdef, i1) ->
@@ -906,8 +905,6 @@ let is_expanded x =
   match pinfo_of_tok x with Ast_c.ExpandedTok _ -> true | _ -> false
 let is_fake x =
   match pinfo_of_tok x with Ast_c.FakeTok _ -> true | _ -> false
-let is_abstract x =
-  match pinfo_of_tok x with Ast_c.AbstractLineTok _ -> true | _ -> false
 
 (*****************************************************************************)
 (* Helpers *)
