@@ -19,6 +19,7 @@ open Common
 open Ast_c
 
 module Lib = Lib_parsing_c
+module IC = Includes_cache
 
 (*****************************************************************************)
 (* Prelude *)
@@ -154,7 +155,7 @@ let pr2, pr2_once = Common.mk_pr2_wrappers Flag_parsing_c.verbose_type
  *)
 type namedef =
   | VarOrFunc of string * Ast_c.exp_type
-  | EnumConstant of string * string option
+  | EnumConstant of string * fullType
 
   (* also used for macro type aliases *)
   | TypeDef   of string * fullType
@@ -178,7 +179,7 @@ type typedefs = { defs : (fullType * typedefs * int) StringMap.t }
 type nameenv = {
     level : int;
     var_or_func : Ast_c.exp_type StringMap.t;
-    enum_constant : string option StringMap.t;
+    enum_constant : Ast_c.fullType StringMap.t;
     typedef : typedefs;
     struct_union_name_def : ((structUnion * structType) wrap) StringMap.t;
     macro : (define_kind * define_val) StringMap.t
@@ -186,6 +187,13 @@ type nameenv = {
 
 type environment = nameenv list
 
+let includes_parse_fn file =
+  let choose_includes = Includes.get_parsing_style () in
+  Includes.set_parsing_style Includes.Parse_no_includes;
+  let ret = Parse_c.parse_c_and_cpp false false file in
+  Includes.set_parsing_style choose_includes;
+  List.map fst (fst ret)
+
 (* ------------------------------------------------------------ *)
 (* can be modified by the init_env function below, by
  * the file environment_unix.h
@@ -240,10 +248,12 @@ let member_env lookupf env =
 
 (* ------------------------------------------------------------ *)
 
-let lookup_var s env =
+let rec lookup_var s env =
   match env with
   | [] -> raise Not_found
-  | env :: _ -> StringMap.find s env.var_or_func
+  | env :: rest ->
+      try StringMap.find s env.var_or_func
+      with _ -> lookup_var s rest
 
 let member_env_lookup_var s env =
   match env with
@@ -294,6 +304,39 @@ let member_env_lookup_enum s env =
   | [] -> false
   | env :: _ -> StringMap.mem s env.enum_constant
 
+(* ------------------------------------------------------------ *)
+
+let add_cache_binding_in_scope namedef =
+  let (current, older) = Common.uncons !_scoped_env in
+  let new_frame fr =
+    match namedef with
+      | IC.RetVarOrFunc (s, typ) ->
+	  {fr with
+	   var_or_func = StringMap.add s typ fr.var_or_func}
+      | IC.RetTypeDef   (s, typ) ->
+	  let cv = typ, fr.typedef, fr.level in
+	  let new_typedef_c : typedefs = { defs = StringMap.add s cv fr.typedef.defs } in
+	  {fr with typedef = new_typedef_c}
+      | IC.RetStructUnionNameDef (s, (su, typ)) ->
+	  {fr with
+	   struct_union_name_def = StringMap.add s (su, typ) fr.struct_union_name_def}
+      | IC.RetEnumConstant (s, body) ->
+	  {fr with
+	   enum_constant = StringMap.add s body fr.enum_constant} in
+  (* These are global, so have to reflect them in all the frames. *)
+  _scoped_env := (new_frame current)::(List.map new_frame older)
+
+(* Has side-effects on the environment.
+ * TODO: profile? *)
+let get_type_from_includes_cache file name exp_types on_success on_failure =
+  let file_bindings =
+    IC.get_types_from_name_cache
+      file name exp_types includes_parse_fn in
+  List.iter add_cache_binding_in_scope file_bindings;
+  match file_bindings with
+    [] -> on_failure ()
+  | _ -> on_success ()
+
 
 (*****************************************************************************)
 (* "type-lookup"  *)
@@ -366,17 +409,19 @@ let rec type_unfold_one_step ty env =
   | Array (e, t)  -> ty
   | Decimal (len,prec_opt) -> ty
 
-  | StructUnion (sopt, su, fields) -> ty
+  | StructUnion (sopt, su, optfinal, base_classes, fields) -> ty
 
   | FunctionType t   -> ty
-  | Enum  (s, enumt) -> ty
+  | EnumDef  (ename, base, enumt) -> ty
 
-  | EnumName s       -> ty (* todo: look in env when will have EnumDef *)
+  | EnumName (key, id)       -> ty (* todo: look in env when will have EnumDef *)
+
+  | ParenType t      -> ty
 
   | StructUnionName (su, s) ->
       (try
           let ((su,fields),ii) = lookup_structunion (su, s) env in
-          Ast_c.mk_ty (StructUnion (su, Some s, fields)) ii
+          Ast_c.mk_ty (StructUnion (su, Some s, None, [], fields)) ii
           (* old: +> Ast_c.rewrap_typeC ty
            * but must wrap with good ii, otherwise pretty_print_c
            * will be lost and raise some Impossible
@@ -394,25 +439,54 @@ let rec type_unfold_one_step ty env =
 	  then type_unfold_one_step t' env'
           else loop (s::seen) t' env
        with Not_found ->
-          ty
+          let f = Ast_c.file_of_info (Ast_c.info_of_name name) in
+          get_type_from_includes_cache
+            f s [IC.CacheTypedef]
+            (fun () ->
+              let (t', env') = lookup_typedef s !_scoped_env in
+              TypeName (name, Some t') +>
+              Ast_c.rewrap_typeC ty)
+            (fun () -> ty)
       )
+  | QualifiedType(typ, _name) -> ty
 
   | FieldType (t, _, _) -> type_unfold_one_step t env
 
-  | ParenType t -> type_unfold_one_step t env
-  | TypeOfExpr e ->
-      pr2_once ("Type_annoter: not handling typeof");
-      ty
-  | TypeOfType t -> type_unfold_one_step t env in
+  | TypeOfExpr e -> type_of_expr (fun t -> t) ty e
+  | TypeOfType t -> type_unfold_one_step t env
+  | AutoType -> ty
+  | TemplateType _ -> ty in
   loop [] ty env
 
-
-
-
-
-
-
-
+and type_of_expr extra ty e =
+  (* hackish, but this is the only place where the
+     difference between decltype and typeof matters *)
+  let (qu, attr, ii) =
+    let (qu, attr, (typeC, ii)) = ty in
+    (qu, attr, List.hd ii) in
+  let etype _ =
+    match Type_c.get_opt_type e with
+      Some t -> t
+    | None -> ty in
+  if str_of_info ii = "decltype"
+  then
+    (match Ast_c.unwrap_expr e with
+      ParenExpr e ->
+	(match Type_c.get_opt_type e with
+	  Some t ->
+	    (* position shouldn't matter because this is a type *)
+	    let vp = ({str="";charpos=0;line=0;column=0;file=""},0) in
+	    let ander =
+	      { pinfo = FakeTok ("&",vp,Ast_c.After);
+		cocci_tag = ref Ast_c.emptyAnnot;
+		annots_tag = Token_annot.empty;
+		comments_tag = ref Ast_c.emptyComments;
+		danger = ref Ast_c.NoDanger;
+	      } in
+	    (qu, attr, (Pointer (extra t), [ander]))
+	| None -> ty)
+    | _ -> etype())
+  else etype()
 
 (* normalizer. can be seen as the opposite of the previous function as
  * we "fold" at least for the structUnion. Should return something that
@@ -431,17 +505,17 @@ let rec typedef_fix ty env =
 	Pointer (typedef_fix t env)  +> Ast_c.rewrap_typeC ty
     | Array (e, t) ->
 	Array (e, typedef_fix t env) +> Ast_c.rewrap_typeC ty
-    | StructUnion (su, sopt, fields) ->
+    | StructUnion (su, sopt, optfinal, base_classes, fields) ->
       (* normalize, fold.
 	 * todo? but what if correspond to a nested struct def ?
       *)
 	Type_c.structdef_to_struct_name ty
     | FunctionType ft ->
 	(FunctionType ft) (* todo ? *) +> Ast_c.rewrap_typeC ty
-    | Enum  (s, enumt) ->
-	(Enum  (s, enumt)) (* todo? *) +> Ast_c.rewrap_typeC ty
-    | EnumName s ->
-	(EnumName s) (* todo? *) +> Ast_c.rewrap_typeC ty
+    | EnumDef  (ename, base, enumt) ->
+	(EnumDef  (ename, base, enumt)) (* todo? *) +> Ast_c.rewrap_typeC ty
+    | EnumName (key, id) ->
+	(EnumName (key, id)) (* todo? *) +> Ast_c.rewrap_typeC ty
     | Decimal(l,p) ->
 	(Decimal(l,p)) (* todo? *) +> Ast_c.rewrap_typeC ty
 
@@ -473,20 +547,31 @@ let rec typedef_fix ty env =
 	      TypeName (name, Some fixed) +>
 	      Ast_c.rewrap_typeC ty
             with Not_found ->
-              ty))
-
+              let f = Ast_c.file_of_info (Ast_c.info_of_name name) in
+              get_type_from_includes_cache
+                f s [IC.CacheTypedef]
+                (fun () ->
+                   let (t', env') = lookup_typedef s !_scoped_env in
+                   TypeName (name, Some t') +>
+                   Ast_c.rewrap_typeC ty)
+                (fun () -> ty)
+              ))
+    | QualifiedType (typ,_name) ->
+    QualifiedType (typ,_name) +> Ast_c.rewrap_typeC ty
     | FieldType (t, a, b) ->
 	FieldType (typedef_fix t env, a, b) +> Ast_c.rewrap_typeC ty
 
-  (* remove paren for better matching with typed metavar. kind of iso again *)
     | ParenType t ->
-	typedef_fix t env
-    | TypeOfExpr e ->
-	pr2_once ("Type_annoter: not handling typeof");
-	ty
+	ParenType (typedef_fix t env) +> Ast_c.rewrap_typeC ty
+
+    | TypeOfExpr e -> type_of_expr (fun t -> typedef_fix t env) ty e
 
     | TypeOfType t ->
-	typedef_fix t env in
+	typedef_fix t env
+    | AutoType ->
+	pr2_once ("Type_annoter: not handling unresolved auto");
+	ty
+    | TemplateType _ -> ty in
   loop [] ty env
 
 
@@ -545,7 +630,7 @@ let _notyped_var = Hashtbl.create 101
 let new_scope() =
   match !_scoped_env with
   | hd :: _ ->
-      _scoped_env := { hd with level = succ hd.level; } :: !_scoped_env
+      _scoped_env := { hd with level = succ hd.level; var_or_func = StringMap.empty; } :: !_scoped_env
   | [] ->
       _scoped_env := [ empty_frame ]
 let del_scope() =
@@ -675,6 +760,15 @@ let make_info_def = Type_c.make_info_def
 (*****************************************************************************)
 
 let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
+  let ret_of_functiontype typ =
+    let rec loop = function
+    | FunctionType (ret, _) -> Some ret
+    (* can be function pointer, C have an iso for that,
+     * same pfn() syntax than regular function call. *)
+    | Pointer typ | ParenType typ -> loop (unwrap_unfold_env typ)
+    | _ -> None in
+    loop (unwrap_unfold_env typ) in
+
 
   let ty =
     match Ast_c.unwrap_expr expr with
@@ -699,12 +793,20 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
 	  | Si(UnSigned,CLongLong) -> type_of_s "unsigned long long"
 	  | _ -> failwith "unexpected kind for constant")
     | Constant (Float (s,kind)) ->
-        let fake = Ast_c.fakeInfo (Common.fake_parse_info) in
-        let fake = Ast_c.rewrap_str "float" fake in
-        let iinull = [fake] in
+	let names =
+	  match kind with
+	    Ast_c.CFloat -> ["float"]
+	  | Ast_c.CDouble -> ["double"]
+	  | Ast_c.CFloatComplex -> ["float";"complex"]
+	  | Ast_c.CDoubleComplex -> ["double";"complex"]
+	  | Ast_c.CLongDouble -> ["long";"double"]
+	  | Ast_c.CLongDoubleComplex -> ["long";"double";"complex"]
+	  | Ast_c.CUnknownComplex -> ["complex"] in
+        let fake = Ast_c.fakeAfterInfo() in
+        let iinull = List.map (fun s -> Ast_c.rewrap_str s fake) names in
         make_info_def (Ast_c.mk_ty (BaseType (FloatType kind)) iinull)
     | Constant (DecimalConst(s,n,p)) ->
-        let fake = Ast_c.fakeInfo (Common.fake_parse_info) in
+        let fake = Ast_c.fakeAfterInfo() in
         let fake1 = Ast_c.rewrap_str "decimal" fake in
         let fake2 = Ast_c.rewrap_str "(" fake in
         let fake3 = Ast_c.rewrap_str "," fake in
@@ -737,26 +839,18 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
           Visitor_c.vk_argument bigf e
         );
         let s = Ast_c.str_of_name ident in
+
+	let get_return_type typ local =
+          (* set type for ident *)
+          let tyinfo = make_info_fix (typ, local) in
+          Ast_c.set_type_expr e1 tyinfo;
+
+          (match ret_of_functiontype typ with
+          | Some ret -> make_info_def ret
+          | None -> Type_c.noTypeHere) in
+
         (match lookup_opt_env lookup_var s with
-        | Some (typ,local) ->
-
-            (* set type for ident *)
-            let tyinfo = make_info_fix (typ, local) in
-            Ast_c.set_type_expr e1 tyinfo;
-
-            (match unwrap_unfold_env typ  with
-            | FunctionType (ret, params) -> make_info_def ret
-
-            (* can be function pointer, C have an iso for that,
-             * same pfn() syntax than regular function call.
-             *)
-            | Pointer (typ2) ->
-                (match unwrap_unfold_env typ2 with
-                | FunctionType (ret, params) -> make_info_def ret
-                | _ -> Type_c.noTypeHere
-                )
-            | _ -> Type_c.noTypeHere
-            )
+        | Some (typ,local) -> get_return_type typ local
         | None  ->
 
             (match lookup_opt_env lookup_macro s with
@@ -793,8 +887,16 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                     Type_c.noTypeHere
                 )
             | None ->
-                pr2_once ("type_annotater: no type for function ident: " ^ s);
-                Type_c.noTypeHere
+                let f =
+                  Ast_c.file_of_info
+                    (Ast_c.info_of_name ident) in
+                get_type_from_includes_cache
+                  f s [IC.CacheVarFunc]
+                  (fun () ->
+                     match lookup_opt_env lookup_var s with
+                       Some (typ, local) -> get_return_type typ local
+                     | None -> Type_c.noTypeHere)
+                  (fun () -> Type_c.noTypeHere)
             )
         )
 
@@ -804,14 +906,9 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
 
         (Ast_c.get_type_expr e1) +> Type_c.do_with_type (fun typ ->
           (* copy paste of above *)
-          (match unwrap_unfold_env typ with
-          | FunctionType (ret, params) -> make_info_def ret
-          | Pointer (typ) ->
-              (match unwrap_unfold_env typ with
-              | FunctionType (ret, params) -> make_info_def ret
-              | _ -> Type_c.noTypeHere
-              )
-          | _ -> Type_c.noTypeHere
+          (match ret_of_functiontype typ with
+          | Some ret -> make_info_def ret
+          | None -> Type_c.noTypeHere
           )
         )
      )
@@ -844,22 +941,36 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                 | Some _ ->
                     make_info_def (type_of_s "int")
                 | None ->
-                    if not (s =~ "[A-Z_]+") (* if macro then no warning *)
-                    then
-                      if !Flag_parsing_c.check_annotater then
-                        if not (Hashtbl.mem _notyped_var s)
-                        then begin
-                          pr2 ("Type_annoter: no type found for: " ^ s);
-                          Hashtbl.add _notyped_var s true;
-                        end
-                        else ()
-                      else
-                        pr2 ("Type_annoter: no type found for: " ^ s)
-                    ;
-                    Type_c.noTypeHere
-                )
+                    let f = Ast_c.file_of_info (Ast_c.info_of_name ident) in
+                    let failure_fn =
+                      (fun () ->
+                         if not (s =~ "[A-Z_]+") (* if macro then no warning *)
+                         then
+                           if !Flag_parsing_c.check_annotater
+                           then
+                             if not (Hashtbl.mem _notyped_var s)
+                             then
+                               begin
+                                 pr2
+                                   ("Type_annoter: no type found for: " ^ s);
+                                 Hashtbl.add _notyped_var s true;
+                               end
+                             else ()
+                           else pr2 ("Type_annoter: no type found for: " ^ s);
+                         Type_c.noTypeHere) in
+                    get_type_from_includes_cache
+                      f s [IC.CacheEnumConst;IC.CacheVarFunc]
+                      (fun () ->
+                         match lookup_opt_env lookup_enum s with
+                           Some _ -> make_info_def (type_of_s "int")
+                         | None ->
+                             (match lookup_opt_env lookup_var s with
+                                Some (typ,local) -> make_info_fix (typ,local)
+                              | None -> failure_fn ()))
+                      failure_fn
             )
         )
+    )
 
     (* -------------------------------------------------- *)
     (* C isomorphism on type on array and pointers *)
@@ -880,19 +991,20 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
     | Unary (e, GetRef) ->
         k expr; (* recurse to set the types-ref of sub expressions *)
 
-	let inherited_fn tq1 ((tq2,ii),ty) =
+	let inherited_fn tq1 ((tq2,ii),attr,ty) = (* should we union the attrs? *)
 	  let cst = tq1.Ast_c.const || tq2.Ast_c.const in
 	  let vol = tq1.Ast_c.volatile || tq2.Ast_c.volatile in
-	  (({Ast_c.const=cst;Ast_c.volatile=vol},ii),ty) in
+	  let res = tq1.Ast_c.restrict || tq2.Ast_c.restrict in
+	  (({Ast_c.const=cst;Ast_c.volatile=vol;Ast_c.restrict=res},ii),attr,ty) in
 	let rec inherited e =
 	  match Ast_c.unwrap_expr e with
 	    RecordAccess (e, _) ->
 	      (match Ast_c.get_onlytype_expr e with
-		Some ((tq1,_),_) -> inherited_fn tq1
+		Some ((tq1,_),_,_) -> inherited_fn tq1
 	      | None -> (fun x -> x))
 	  | RecordPtAccess (e, _) ->
 	      (match Ast_c.get_onlytype_expr e with
-		Some(_,(Ast_c.Pointer ((tq1,_),_),_)) -> inherited_fn tq1
+		Some(_,_,(Ast_c.Pointer ((tq1,_),_,_),_)) -> inherited_fn tq1
 	      | Some t -> (fun x -> x)
 	      | None -> (fun x -> x))
 	  | ArrayAccess (e, _) -> inherited e
@@ -902,10 +1014,25 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
           (* must generate an element so that '=' can be used
            * to compare type ?
            *)
-          let fake = Ast_c.fakeInfo Common.fake_parse_info in
-          let fake = Ast_c.rewrap_str "*" fake in
-
-          let ft = Ast_c.mk_ty (Pointer ((inherited e) t)) [fake] in
+	  let ft =
+	    let is_fn_ar = (* & has no impact for fn or array *)
+	      match Ast_c.unwrap_typeC t with
+		ParenType pt ->
+		  (match Ast_c.unwrap_typeC pt with
+		    Pointer t ->
+		      (match Ast_c.unwrap_typeC t with
+			FunctionType (returnt, paramst) -> true
+		      | _ -> false)
+		  | _ -> false)
+	      | Array _ -> true
+	      | _ -> false in
+	    if is_fn_ar
+	    then t
+	    else
+	      (* NoPos because a type generated on the fly *)
+	      let fake = Ast_c.fakeAfterInfoNoPos() in
+              let fake = Ast_c.rewrap_str "*" fake in
+              Ast_c.mk_ty (Pointer ((inherited e) t)) [fake] in
           make_info_def_fix ft
         )
 
@@ -934,7 +1061,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
           | None -> Type_c.noTypeHere
           | Some t ->
               match unwrap_unfold_env t with
-              | StructUnion (su, sopt, fields) ->
+              | StructUnion (su, sopt, optfinal, base_classes, fields) ->
                   (try
                       (* todo: which env ? *)
                       make_info_def_fix
@@ -950,12 +1077,32 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
                         Type_c.noTypeHere
                   )
               | _ ->
-		  Type_c.noTypeHere
+                let s = Ast_c.str_of_name namefld in
+                let f = Ast_c.file_of_info (Ast_c.info_of_name namefld) in
+                let ret_typ =
+                  (match Ast_c.unwrap_typeC t with
+                    Ast_c.StructUnionName(su, sname) ->
+                      get_type_from_includes_cache
+                        f s [IC.CacheField sname]
+                        (fun () ->
+                           try
+                             let ((su,fields),ii) =
+                               lookup_structunion (su, sname) !_scoped_env in
+                               try
+                                  make_info_def_fix
+                                    (Type_c.type_field fld (su, fields))
+                                with _ -> Type_c.noTypeHere
+                           with Not_found -> Type_c.noTypeHere)
+                        (fun () -> Type_c.noTypeHere)
+                  | _ -> Type_c.noTypeHere)
+                in ret_typ
           )
         )
 
-
-
+    | QualifiedAccess(Some ty,_) ->
+        k expr;
+        make_info_def_fix(Lib.al_type ty)
+    | QualifiedAccess(None,_) -> Type_c.noTypeHere
     (* -------------------------------------------------- *)
     | Cast (t, e) ->
         k expr;
@@ -1041,7 +1188,7 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
             | _ -> k expr; Type_c.noTypeHere
           *)
 
-    | New (_, ty) ->
+    | New (_, ty, _) ->
 	k expr;
 	pr2_once "Type annotater:not handling New";
 	Type_c.noTypeHere (* TODO *)
@@ -1051,6 +1198,15 @@ let annotater_expr_visitor_subpart = (fun (k,bigf) expr ->
 	pr2_once "Type annotater:not handling Delete";
 	Type_c.noTypeHere (* TODO *)
 
+    | TemplateInst(name,args) ->
+	k expr;
+	Type_c.noTypeHere
+    
+    (* TODO: Make a proper tuple type *)
+    | TupleExpr(args) ->
+    k expr;
+    Type_c.noTypeHere
+
     | Defined _ ->
 	make_info_def (type_of_s "int")
 
@@ -1104,6 +1260,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
       (* do error messages for type annotater only for the real body of the
        * file, not inside include.
        *)
+      | UsingNamespace _ -> ()
       | Include {i_content = opt} ->
           opt +> Common.do_option (fun (filename, program) ->
             Common.save_excursion Flag_parsing_c.verbose_type (fun () ->
@@ -1127,31 +1284,39 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
            * the macro has no type and so we infer its type from its
            * body (and one day later maybe from its use).
            *)
-          (match defval with
-          (* can try to optimize and recurse only when the define body
-           * is simple ?
-           *)
+	  do_in_new_scope (fun () ->
+	    (* prevent macro-declared variables from leaking out *)
+	    (match defkind with
+	      DefineFunc (params,ii) ->
+		List.iter
+		  (function ((id,ii),commaii) ->
+		    let local = Ast_c.LocalVar (List.hd ii).pinfo in
+		    let bind =
+		      VarOrFunc (id,(Lib.al_type (mk_ty NoType []),local)) in
+		    add_binding bind true)
+		  params
+	    | _ -> ());
+	    match defval with
+            (* can try to optimize and recurse only when the define body
+             * is simple ?
+             *)
 
-          | DefineExpr expr ->
-	      (* prevent macro-declared variables from leaking out *)
-	      do_in_new_scope (fun () ->
+            | DefineExpr expr ->
 		if is_simple_expr expr
-             (* even if not need_annotate_body, still recurse*)
+                (* even if not need_annotate_body, still recurse*)
 		then k directive
 		else
                   if need_annotate_body
-                  then k directive)
-          | _ ->
-	      do_in_new_scope (fun () ->
+                  then k directive
+	    | _ ->
 		if need_annotate_body
-		then k directive)
-          );
+		then k directive);
 
           add_binding (Macro (s, (defkind, defval) )) true;
 
       |	Pragma((name,rest), ii) -> ()
 
-      | OtherDirective _ -> ()
+      | OtherDirective _ | UsingTypename _ | UsingMember _ -> ()
     );
 
     (* ------------------------------------------------------------ *)
@@ -1168,7 +1333,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
     (* ------------------------------------------------------------ *)
     Visitor_c.kdecl = (fun (k, bigf) d ->
       (match d with
-      | (DeclList (xs, ii)) ->
+      | (DeclList ((xs, has_ender), ii)) ->
           xs +> List.iter (fun ({v_namei = var; v_type = t;
                                  v_storage = sto; v_local = local} as x
                                    , iicomma) ->
@@ -1180,7 +1345,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 	    let local =
 	      match (sto,local) with
 	      | (_,Ast_c.NotLocalDecl) -> Ast_c.NotLocalVar
-	      |	((Ast_c.Sto Ast_c.Static, _), Ast_c.LocalDecl) ->
+	      |	((Ast_c.Sto Ast_c.Static, _, _), Ast_c.LocalDecl) ->
 		  (match Ast_c.info_of_type t with
 		    (* if there is no info about the type it must not be
 		       present, so we don't know what the variable is *)
@@ -1209,7 +1374,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 
 
               match sto with
-              | StoTypedef, _inline ->
+              | StoTypedef, _inline, _align ->
                   add_binding (TypeDef (s,Lib.al_type t)) true;
               | _ ->
                   add_binding (VarOrFunc (s, (Lib.al_type t, local))) true;
@@ -1221,11 +1386,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
                     (* int x = sizeof(x) is legal so need process ini *)
 		    match iniopt with
 		      Ast_c.NoInit -> ()
-		    | Ast_c.ValInit(iini,init) -> Visitor_c.vk_ini bigf init
-		    | Ast_c.ConstrInit((args,_)) ->
-			args +> List.iter (fun (e,ii) ->
-			  Visitor_c.vk_argument bigf e
-			)
+		    | Ast_c.ValInit(init,iini) -> Visitor_c.vk_ini bigf init
                   end
             );
           );
@@ -1243,18 +1404,18 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
        * the ref of abstract-lined types, but the real one, so
        * don't al_type here
        *)
-      let (_q, tbis) = typ in
+      let (_q, _attr, tbis) = typ in
       match Ast_c.unwrap_typeC typ with
-      | StructUnion  (su, Some s, structType) ->
+      | StructUnion  (su, Some s, optfinal, base_classes, structType) ->
           let structType' = Lib.al_fields structType in
           let ii = Ast_c.get_ii_typeC_take_care tbis in
           let ii' = Lib.al_ii ii in
           add_binding (StructUnionNameDef (s, ((su, structType'),ii')))  true;
 
           if need_annotate_body
-          then k typ (* todo: restrict ? new scope so use do_in_scope ? *)
+          then do_in_new_scope (fun () -> k typ) (* todo: restrict ? new scope so use do_in_scope ? *)
 
-      | Enum (sopt, enums) ->
+      | EnumDef (sopt, base, enums) ->
 
           enums +> List.iter (fun ((name, eopt), iicomma) ->
 
@@ -1303,8 +1464,13 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
 
           (match oldstyle with
           | None ->
-              let typ' =
-                Lib.al_type (Ast_c.mk_ty (FunctionType ftyp) [i1;i2]) in
+              let fake = Ast_c.fakeAfterInfo() in
+              let fakea = Ast_c.rewrap_str "*" fake in
+              let fakeopar = Ast_c.rewrap_str "(" fake in
+              let fakecpar = Ast_c.rewrap_str ")" fake in
+
+              let typ' = Lib.al_type (Ast_c.mk_ty (ParenType (Ast_c.mk_ty
+                (Pointer ( Ast_c.mk_ty (FunctionType ftyp) [i1;i2])) [fakea])) [fakeopar;fakecpar]) in
 
               add_binding (VarOrFunc (funcs, (typ',islocal i1.Ast_c.pinfo)))
                 false;
@@ -1361,6 +1527,7 @@ let rec visit_toplevel ~just_add_in_env ~depth elem =
       | NotParsedCorrectly _
       | FinalDef _
       | Namespace _
+      | TemplateDefinition _
           ->
           k elem
     );
@@ -1445,10 +1612,20 @@ let annotate_test_expressions prog =
 	  k st;
       |	Iteration(i) ->
 	  (match i with
-	    While(e,s) -> propagate_test e
+	    While(WhileExp (e),s) -> propagate_test e
+	  | While(WhileDecl (DeclList dl),s) ->
+              (match unwrap dl with
+                ([x], has_ender) ->
+                  (match (unwrap2 x).v_namei with
+                    Some (n,ValInit z) ->
+                      (match unwrap (unwrap z ) with
+                        InitExpr e -> propagate_test e
+                      | _ -> ())
+                  | _ -> ())
+              | _ -> ())
 	  | DoWhile(s,e) -> propagate_test e
-	  | For(_,es,_,_) ->
-	      (match unwrap es with Some e -> propagate_test e | None -> ())
+	  | For(ForExp(_,(Some e,_),_),_) -> propagate_test e
+	  | For(ForDecl(_,(Some e,_),_),_) -> propagate_test e
 	  | _ -> ());
 	  k st
       | _ -> k st
@@ -1470,7 +1647,3 @@ let annotate_program env prog =
       annotate_test_expressions prog;
       res
     )
-
-let annotate_type_and_localvar env prog =
-  Common.profile_code "TAC.annotate_type"
-    (fun () -> annotate_program2 env prog)
