@@ -463,6 +463,7 @@ let rec expression e =
     | Ast_c.New (_, t, _) -> unary
     | Ast_c.Delete(_,t) -> unary
     | Ast_c.TemplateInst(tn,args) -> unary
+    | Ast_c.TupleExpr(args) -> unary
     | Ast_c.Defined _ -> primary
   in
 
@@ -532,6 +533,8 @@ let rec expression e =
       loop tn postfix; mcode (print_string_with_hint StartBox) lp;
       dots (function _ -> ()) arg_expression args;
       mcode (print_string_with_hint EndBox) rp
+  | Ast.TupleExpr(init) ->
+      initialiser true init
   | Ast.TypeExp(ty) -> fullType ty
   | Ast.Constructor(lp,ty,rp,init) ->
       mcode print_string_box lp; fullType ty; close_box();
@@ -1045,7 +1048,7 @@ and declaration d =
       print_named_type ty (fun _ -> ident id);
       print_attribute_list endattr;
       pr_space(); mcode print_string eq;
-      pr_space(); initialiser true ini; mcode print_string sem
+      pr_space(); initialiser true ini; print_option (mcode print_string) sem
   | Ast.UnInit(al,stg,ty,id,endattr,sem) ->
       print_option alignas al;
       print_option (mcode storage) stg;
@@ -1135,6 +1138,7 @@ and field d =
       print_attribute_list attr;
       mcode print_string sem
   | Ast.CppField(di) -> directive di
+  | Ast.AccSpec(decl,dd) -> mcode print_string_box decl; mcode print_string_box dd
 
 and pragmainfo pi =
   match Ast.unwrap pi with
@@ -1145,6 +1149,11 @@ and pragmainfo pi =
         | Ast_c.MetaPragmaInfoVal(rest) -> print_text (Ast_c.str_of_info rest)
         | _ -> error name pi "pragma info value expected")
 
+and whileinfo cond =
+  match cond with
+    Ast.WhileExp(e) -> expression e
+  | Ast.WhileDecl(d) -> annotated_decl d
+
 and directive di =
   match Ast.unwrap di with
     Ast.Include(inc,s) ->
@@ -1382,10 +1391,10 @@ and rule_elem arity re =
   | Ast.Else(els) ->
       pr_arity arity; mcode print_string els
 
-  | Ast.WhileHeader(whl,lp,exp,rp) ->
+  | Ast.WhileHeader(whl,lp,cond,rp) ->
       pr_arity arity;
       mcode print_string whl; pr_space(); mcode print_string_box lp;
-      expression exp; close_box(); mcode print_string rp
+      whileinfo cond; close_box(); mcode print_string rp
   | Ast.DoHeader(d) ->
       pr_arity arity; mcode print_string d
   | Ast.WhileTail(whl,lp,exp,rp,sem) ->
