@@ -348,6 +348,8 @@ let rec vk_expr = fun bigf expr ->
     | TemplateInst(nm,args) ->
 	exprf nm;
 	vk_argument_list bigf args
+    | TupleExpr(args) ->
+        vk_ini bigf args
     | Defined name -> vk_name bigf name
 
 
@@ -429,8 +431,14 @@ and vk_statement = fun bigf (st: Ast_c.statement) ->
         cal +> List.iter (fun ((param,st),ii) -> vk_param bigf param; statf st; iif ii)
     | Selection  (Switch (e, st)) ->
         vk_expr bigf e; statf st;
-    | Iteration  (While (e, st)) ->
-        vk_expr bigf e; statf st;
+    | Iteration  (While (first, st)) ->
+	(match first with
+	  WhileExp(e) ->
+            vk_expr bigf e;
+            statf (mk_st (ExprStatement (Some e)) [])
+	| WhileDecl(d) ->
+	    vk_decl bigf d);
+        statf st;
     | Iteration  (DoWhile (st, e)) -> statf st; vk_expr bigf e;
     | Iteration  (For (first, st)) ->
 	(match first with
@@ -587,7 +595,7 @@ and vk_decl = fun bigf d ->
   let f = bigf.kdecl in
   let k decl =
     match decl with
-    | DeclList (xs,ii) ->
+    | DeclList ((xs, has_ender),ii) ->
 	iif ii;
 	xs +> List.iter (fun (x,ii) ->
         iif ii;
@@ -712,7 +720,7 @@ and vk_struct_field = fun bigf field ->
 
       (* C++ *)
     | FunctionField def -> vk_def bigf def
-    | PublicLabel ii | ProtectedLabel ii | PrivateLabel ii -> iif ii
+    | AccSpec ii -> iif ii
     | ConstructDestructField cd -> vk_constr_destr bigf cd
   in
   f (k, bigf) field
@@ -952,7 +960,7 @@ and vk_cpp_directive bigf directive =
         vk_define_val bigf defval
     | Pragma ((id,rest),ii) ->
 	vk_name bigf id;
-	rest +> List.iter (fun (str,ii) -> iif ii);
+	vk_info bigf rest;
 	iif ii
     | OtherDirective (ii) ->
         iif ii
@@ -1064,11 +1072,13 @@ and vk_node = fun bigf node ->
 
     | F.IfHeader (_, (e,ii))
     | F.SwitchHeader (_, (e,ii))
-    | F.WhileHeader (_, (e,ii))
+    | F.WhileHeader (_, (WhileExp (e), ii))
     | F.DoWhileTail (e,ii) ->
         iif ii;
         vk_expr bigf e
-
+    | F.WhileHeader (_, (WhileDecl (d), ii)) ->
+        iif ii;
+        d +> (vk_decl bigf);
     | F.TryHeader (_,info) ->
         infof info
     | F.CatchHeader (param,ii) ->
@@ -1409,11 +1419,11 @@ let rec vk_expr_s = fun bigf expr ->
               vk_argument_s bigf e, iif ii)) init)
       | Delete (box,e) -> Delete (box,vk_expr_s bigf e)
     | TemplateInst(nm,args) ->
-	TemplateInst(exprf nm,
+      TemplateInst(exprf nm,
 		     List.map
 		       (fun (e,ii) -> vk_argument_s bigf e, iif ii)
 		       args)
-
+      | TupleExpr(init) -> TupleExpr(vk_ini_s bigf init)
       | Defined name  -> Defined (vk_name_s bigf name)
 
     in
@@ -1501,8 +1511,12 @@ and vk_statement_s = fun bigf st ->
                   ((vk_param_s bigf param, statf st), vk_ii_s bigf ii))))
       | Selection (Switch (e, st))   ->
           Selection  (Switch ((vk_expr_s bigf) e, statf st))
-      | Iteration (While (e, st))    ->
-          Iteration  (While ((vk_expr_s bigf) e, statf st))
+      | Iteration (While (first, st))    ->
+	(match first with
+	  WhileExp(e) ->
+            Iteration  (While (WhileExp ((vk_expr_s bigf) e), statf st))
+	| WhileDecl(d) ->
+            Iteration  (While (WhileDecl((vk_decl_s bigf) d), statf st)))
       | Iteration (DoWhile (st, e))  ->
           Iteration  (DoWhile (statf st, (vk_expr_s bigf) e))
       | Iteration (For (first, st)) ->
@@ -1721,8 +1735,8 @@ and vk_decl_s = fun bigf d ->
   let iif ii = vk_ii_s bigf ii in
   let rec k decl =
     match decl with
-    | DeclList (xs, ii) ->
-        DeclList (List.map (fun (x,ii) -> (vk_onedecl_s bigf x, iif ii)) xs,
+    | DeclList ((xs,has_ender), ii) ->
+        DeclList ((List.map (fun (x,ii) -> (vk_onedecl_s bigf x, iif ii)) xs, has_ender),
 		  iif ii)
     | MacroDecl ((stob, preattrs, s, args, attrs, ptvg),ii) ->
         MacroDecl
@@ -1751,7 +1765,8 @@ and vk_onedecl_opt_s process_type bigf {v_namei = var;
             v_storage = sto;
             v_local= local;
             v_attr = attrs;
-            v_endattr = endattrs} =
+            v_endattr = endattrs;
+            } =
   let iif ii = vk_ii_s bigf ii in
     {v_namei =
       (var +> map_option (fun (name, iniopt) ->
@@ -1872,9 +1887,7 @@ and vk_struct_field_s = fun bigf field ->
   | IfdefStruct ifdef ->
       IfdefStruct (vk_ifdef_directive_s bigf ifdef)
   | FunctionField def -> FunctionField (vk_def_s bigf def)
-  | PublicLabel ii -> PublicLabel(iif ii)
-  | ProtectedLabel ii -> ProtectedLabel(iif ii)
-  | PrivateLabel ii -> PrivateLabel(iif ii)
+  | AccSpec ii -> AccSpec (iif ii)
   | ConstructDestructField cd ->
       ConstructDestructField(vk_constr_destr_s bigf cd)
 
@@ -2055,9 +2068,7 @@ and vk_cpp_directive_s = fun bigf top ->
         Define ((s, iif ii),
                (vk_define_kind_s bigf defkind, vk_define_val_s bigf defval))
     | Pragma((id,rest),ii) ->
-	Pragma((vk_name_s bigf id,
-		rest +> List.map (fun (str,ii) -> (str,iif ii))),
-	       iif ii)
+	Pragma((vk_name_s bigf id, vk_info_s bigf rest),iif ii)
     | OtherDirective (ii) -> OtherDirective (iif ii)
     | UsingTypename((name,def),ii) ->
 	UsingTypename((vk_name_s bigf name,vk_type_s bigf def),iif ii)
@@ -2196,8 +2207,10 @@ and vk_node_s = fun bigf node ->
         F.TryHeader (st, infof info)
     | F.CatchHeader (param,ii) ->
         F.CatchHeader (vk_param_s bigf param, iif ii)
-    | F.WhileHeader (st, (e,ii))  ->
-        F.WhileHeader (st, (vk_expr_s bigf e, iif ii))
+    | F.WhileHeader (st, ((WhileExp e),ii))  ->
+        F.WhileHeader (st, (WhileExp (vk_expr_s bigf e), iif ii))
+    | F.WhileHeader (st, ((WhileDecl d),ii))  ->
+        F.WhileHeader (st, (WhileDecl (vk_decl_s bigf d), iif ii))
     | F.DoWhileTail (e,ii)  ->
         F.DoWhileTail (vk_expr_s bigf e, iif ii)
 
