@@ -52,16 +52,6 @@ let get_option fn = function
 
 let anyopt l fn = List.exists (function w -> fn(Ast0.unwrap w)) l
 
-let allopt l fn =
-  let rec loop = function
-      [] -> []
-    | x::xs ->
-	match fn (Ast0.unwrap x) with
-	  Some x -> x :: (loop xs)
-	| None -> [] in
-  let res = loop l in
-  if List.length res = List.length l then Some res else None
-
 (* --------------------------------------------------------------------- *)
 (* --------------------------------------------------------------------- *)
 (* Mcode *)
@@ -306,6 +296,13 @@ let rec top_expression opt_allowed tgt expr =
       let tn = expression arity tn in
       let (lb,args,rb) = arg_list tgt (lb,args,rb) in
       make_exp expr tgt arity (Ast0.TemplateInst(tn,lb,args,rb))
+  | Ast0.TupleExpr(init) ->
+      let new_init = initialiser tgt init in
+      Ast0.rewrap expr
+	(match Ast0.unwrap new_init with
+	  Ast0.OptIni(init) ->
+	    Ast0.OptExp(Ast0.rewrap expr (Ast0.TupleExpr(init)))
+	| _ -> Ast0.TupleExpr(init))
   | Ast0.TypeExp(ty) -> Ast0.rewrap expr (Ast0.TypeExp(typeC tgt ty))
   | Ast0.MetaErr(name,constraints,pure)  ->
       let arity = exp_same (mcode2line name) [mcode2arity name] in
@@ -627,8 +624,9 @@ and declaration tgt decl =
   | Ast0.Init(al,stg,ty,id,endattr,eq,exp,sem) ->
       let arity =
 	all_same true tgt (mcode2line eq)
-	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
-	   (List.map mcode2arity [eq;sem])) in
+	  ((List.map mcode2arity (Common.opt_to_list stg)) @
+	   (mcode2arity eq) ::
+	   (List.map mcode2arity (Common.opt_to_list sem))) in
       let al = get_option (alignas tgt) al in
       let stg = get_option mcode stg in
       let ty = typeC arity ty in
@@ -641,7 +639,7 @@ and declaration tgt decl =
   | Ast0.UnInit(al,stg,ty,id,endattr,sem) ->
       let arity =
 	all_same true tgt (mcode2line sem)
-	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
+	  ((List.map mcode2arity (Common.opt_to_list stg)) @
 	   [mcode2arity sem]) in
       let al = get_option (alignas tgt) al in
       let stg = get_option mcode stg in
@@ -800,6 +798,11 @@ and field tgt decl =
       then failwith "unexpected code");
       let res = Ast0.ConjField(starter,decls,mids,ender) in
       Ast0.rewrap decl res
+  | Ast0.AccSpec(name,dd) ->
+      let arity = all_same true tgt (mcode2line name) [mcode2arity name; mcode2arity dd] in
+      let name = mcode name in
+      let dd = mcode dd in
+      make_field decl tgt arity (Ast0.AccSpec(name,dd))
   | Ast0.Fdots(dots,whencode) ->
       let arity = all_same true tgt (mcode2line dots) [mcode2arity dots] in
       let dots = mcode dots in
@@ -1044,16 +1047,16 @@ and statement tgt stm =
       let branch2 = statement arity branch2 in
       make_rule_elem stm tgt arity
 	(Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,aft))
-  | Ast0.While(wh,lp,exp,rp,body,aft) ->
+  | Ast0.While(wh,lp,cond,rp,body,aft) ->
       let arity =
 	stm_same (mcode2line wh)
 	  (List.map mcode2arity [wh;lp;rp]) in
       let wh = mcode wh in
       let lp = mcode lp in
-      let exp = expression arity exp in
+      let cond = whileinfo arity cond in
       let rp = mcode rp in
       let body = statement arity body in
-      make_rule_elem stm tgt arity (Ast0.While(wh,lp,exp,rp,body,aft))
+      make_rule_elem stm tgt arity (Ast0.While(wh,lp,cond,rp,body,aft))
   | Ast0.Do(d,body,wh,lp,exp,rp,sem) ->
       let arity =
 	stm_same (mcode2line wh) (List.map mcode2arity [d;wh;lp;rp;sem]) in
@@ -1334,6 +1337,11 @@ and statement tgt stm =
   | Ast0.OptStm(_) | Ast0.AsStmt _ ->
       failwith "unexpected code"
 
+and whileinfo tgt cond =
+  match cond with
+    Ast0.WhileExp(e) -> Ast0.WhileExp(expression tgt e)
+  | Ast0.WhileDecl(bef,d) -> Ast0.WhileDecl(bef,declaration tgt d)
+
 and make_pragma =
   make_opt
     (function x -> failwith "opt not allowed for pragma")
