@@ -42,13 +42,11 @@ type info = { pos_info : position_info;
 (* adjacency index is incremented when we skip over dots or nest delimiters
 it is used in deciding how much to remove, when two adjacent code tokens are
 removed. *)
-type adjacency = int
-
-type fake_mcode = info * mcodekind * adjacency
+type fake_mcode = info * mcodekind * Ast.adj
 
 type 'a mcode =
     'a * arity * info * mcodekind * anything list ref (* pos, - only *) *
-      adjacency (* adjacency_index *)
+      Ast.adj (* adjacency_index *)
 (* int ref is an index *)
 and 'a wrap =
     { node : 'a;
@@ -119,15 +117,25 @@ and base_expression =
   | Nested         of expression * binaryOp * expression
   | Paren          of string mcode (* ( *) * expression *
                       string mcode (* ) *)
-  | ArrayAccess    of expression * string mcode (* [ *) * expression *
+  | ArrayAccess    of expression * string mcode (* [ *) * expression dots *
 	              string mcode (* ] *)
   | RecordAccess   of expression * string mcode (* . *) * ident
   | RecordPtAccess of expression * string mcode (* -> *) * ident
-  | Cast           of string mcode (* ( *) * typeC * attr list *
+  | QualifiedAccess of typeC option * string mcode (* :: *) * ident
+  | Cast           of string mcode (* ( *) * typeC *
                       string mcode (* ) *) * expression
   | SizeOfExpr     of string mcode (* sizeof *) * expression
   | SizeOfType     of string mcode (* sizeof *) * string mcode (* ( *) *
-                      typeC * string mcode (* ) *)
+                   typeC * string mcode (* ) *)
+  | Delete         of string mcode (* delete *) * expression
+  | DeleteArr      of string mcode (* delete *) * string mcode (* [ *) * string mcode (* ] *) *
+	           expression
+  | New            of string mcode (* new *) * arguments option *
+	           string mcode (* ( *) option *  typeC *
+	           string mcode (* ) *) option * arguments option (* argument list *)
+  | TemplateInst of expression (* name *) * string mcode (* < *) *
+        expression dots * string mcode (* > *)
+  | TupleExpr      of initialiser
   | TypeExp        of typeC (* type name used as an expression, only in args *)
   | Constructor    of string mcode (* ( *) * typeC * string mcode (* ) *) *
 	              initialiser
@@ -155,6 +163,8 @@ and base_expression =
 
 and expression = base_expression wrap
 
+and arguments = string mcode (* ( *) * expression dots * string mcode (* ) *)
+
 and constraints = expression Ast.generic_constraints
 
 and listlen =
@@ -196,7 +206,7 @@ and binaryOp = base_binaryOp wrap
 (* Types *)
 
 and base_typeC =
-    ConstVol        of Ast.const_vol mcode * typeC
+    ConstVol        of cvattr list * typeC * cvattr list
   | BaseType        of Ast.baseType * string mcode list
   | Signed          of Ast.sign mcode * typeC option
   | Pointer         of typeC * string mcode (* * *)
@@ -209,18 +219,21 @@ and base_typeC =
 	               expression *
 	               string mcode option (* , *) * expression option *
 	               string mcode (* ) *) (* IBM C only *)
-  | EnumName        of string mcode (*enum*) * ident option (* name *)
-  | EnumDef  of typeC (* either StructUnionName or metavar *) *
-	string mcode (* { *) * enum_decl dots * string mcode (* } *)
+  | EnumName        of string mcode (*enum*) * Ast.structUnion mcode option (* struct/class/None *) * ident option (* name *)
+  | EnumDef         of typeC (* EnumName *) * enum_base option (* base type *) *
+	               string mcode (* { *) * enum_decl dots * string mcode (* } *)
   | StructUnionName of Ast.structUnion mcode * ident option (* name *)
   | StructUnionDef  of typeC (* either StructUnionName or metavar *) *
 	string mcode (* { *) * field dots * string mcode (* } *)
-  | TypeOfExpr      of string mcode (* sizeof *) * string mcode (* ( *) *
+  | TypeOfExpr      of string mcode (* typeof *) * string mcode (* ( *) *
                        expression * string mcode (* ) *)
-  | TypeOfType      of string mcode (* sizeof *) * string mcode (* ( *) *
+  | TypeOfType      of string mcode (* typeof *) * string mcode (* ( *) *
                        typeC * string mcode (* ) *)
   | TypeName        of string mcode
+  | QualifiedType   of typeC option * string mcode (* :: *) * ident
   | AutoType        of string mcode (* auto *) (* c++ >= 11 *)
+  | TemplateType    of typeC (* name *) * string mcode (* < *) *
+                       expression dots * string mcode (* > *)
   | MetaType        of Ast.meta_name mcode * constraints * pure
   | AsType          of typeC * typeC (* as type, always metavar *)
   | DisjType        of string mcode * typeC list *
@@ -231,6 +244,8 @@ and base_typeC =
 
 and typeC = base_typeC wrap
 
+and enum_base = string mcode (* : *) * typeC
+
 (* --------------------------------------------------------------------- *)
 (* Variable declaration *)
 (* Even if the Cocci program specifies a list of declarations, they are
@@ -242,24 +257,25 @@ and base_declaration =
        they don't match the same thing at all.  Consider whether there
        should be a separate type for fields, as in the C AST *)
   | AsDecl        of declaration * declaration
-  | Init of Ast.storage mcode option * typeC * ident * attr list *
-	string mcode (*=*) * initialiser * string mcode (*;*)
-  | UnInit of Ast.storage mcode option * typeC * ident * attr list *
-	string mcode (* ; *)
+  | Init of alignas option * Ast.storage mcode option * typeC * ident *
+        attr list * string mcode (*=*) * initialiser * string mcode option (*;*)
+  | UnInit of alignas option * Ast.storage mcode option * typeC * ident *
+	attr list * string mcode (* ; *)
   | FunProto of
 	fninfo list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
         (string mcode (* , *) * string mcode (* ...... *) ) option *
 	string mcode (* ) *) *
 	string mcode (* ; *)
-  | TyDecl of typeC * attr list * string mcode (* ; *)
-  | MacroDecl of Ast.storage mcode option *
+  | TyDecl of typeC * string mcode (* ; *)
+  | MacroDecl of Ast.storage mcode option * attr list *
 	ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) *
         attr list * string mcode (* ; *)
-  | MacroDeclInit of Ast.storage mcode option *
+  | MacroDeclInit of Ast.storage mcode option * attr list *
 	ident (* name *) * string mcode (* ( *) *
-        expression dots * string mcode (* ) *) * string mcode (*=*) *
+        expression dots * string mcode (* ) *) *
+	attr list * string mcode (*=*) *
 	initialiser * string mcode (* ; *)
   | Typedef of string mcode (* typedef *) * typeC * typeC * string mcode (*;*)
   | DisjDecl   of string mcode * declaration list *
@@ -270,6 +286,9 @@ and base_declaration =
 
 and declaration = base_declaration wrap
 
+and alignas = Align of string mcode (* alignas *) * string mcode (* ( *)
+					* expression * string mcode (* ) *)
+
 (* --------------------------------------------------------------------- *)
 (* Field declaration *)
 
@@ -277,11 +296,16 @@ and base_field =
   | MetaField  of Ast_cocci.meta_name mcode * constraints *
 	pure (* structure fields *)
   | MetaFieldList of Ast_cocci.meta_name mcode * listlen * constraints * pure
-  | Field     of typeC * ident option * bitfield option * string mcode (* ; *)
+  | Field     of typeC * ident option * bitfield option * attr list * string mcode (* ; *)
+  | MacroDeclField of ident (* name *) * string mcode (* ( *) *
+        expression dots * string mcode (* ) *) *
+        attr list * string mcode (* ; *)
+  | CppField    of directive
   | DisjField   of string mcode * field list * string mcode list *
 	          string mcode
   | ConjField   of string mcode * field list * string mcode list *
 	          string mcode
+  | AccSpec of string mcode * string mcode   (* C++ *)
   | Fdots      of string mcode (* ... *) * (string mcode * string mcode *
                   field) option (* whencode *)
   | OptField    of field
@@ -335,8 +359,7 @@ and initialiser_list = initialiser dots
 (* Parameter *)
 
 and base_parameterTypeDef =
-    VoidParam     of typeC * attr list
-  | Param         of typeC * ident option * attr list
+    Param         of typeC * ident option * attr list
   | MetaParam     of Ast.meta_name mcode * constraints * pure
   | MetaParamList of Ast.meta_name mcode * listlen * constraints * pure
   | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
@@ -353,6 +376,7 @@ and parameter_list = parameterTypeDef dots
 
 and base_define_param =
     DParam        of ident
+  | DParamEll     of ident * string mcode (* ... *)
   | MetaDParamList of Ast.meta_name mcode * listlen * constraints * pure
   | DPComma       of string mcode
   | DPdots        of string mcode (* ... *)
@@ -383,15 +407,14 @@ and base_statement =
 	             statement * string mcode (* else *) * statement *
 	             fake_mcode (* after info *)
   | While         of string mcode (* while *) * string mcode (* ( *) *
-	             expression * string mcode (* ) *) *
+	             whileinfo * string mcode (* ) *) *
 	             statement * fake_mcode (* after info *)
   | Do            of string mcode (* do *) * statement *
                      string mcode (* while *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
                      string mcode (* ; *)
   | For           of string mcode (* for *) * string mcode (* ( *) * forinfo *
-	             expression option * string mcode (*;*) *
-                     expression option * string mcode (* ) *) * statement *
+	             string mcode (* ) *) * statement *
 	             fake_mcode (* after info *)
   | Iterator      of ident (* name *) * string mcode (* ( *) *
 	             expression dots * string mcode (* ) *) *
@@ -418,6 +441,7 @@ and base_statement =
   | Ty            of typeC (* only at top level *)
   | TopId         of ident (* only at top level *)
   | TopInit       of initialiser (* only at top level *)
+  | CppTop        of directive
   | Disj          of string mcode * statement dots list *
 	             string mcode list (* the |s *)  * string mcode
   | Conj          of string mcode * statement dots list *
@@ -430,42 +454,77 @@ and base_statement =
 	fninfo list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
 	(string mcode (* , *) * string mcode (* ...... *) ) option *
-	string mcode (* ) *) *
+	string mcode (* ) *) * attr list *
 	string mcode (* { *) * statement dots *
 	string mcode (* } *) *
 	(info * mcodekind) (* after the function decl *)
-  | Include of string mcode (* #include *) * Ast.inc_file mcode (* file *)
-  | MetaInclude of string mcode (* #include *) * expression (* file *)
+  | TemplateDefinition of
+	string mcode (* template *) * string mcode (* < *) *
+        template_parameter_list * string mcode (* > *) *
+        statement
+
   | Undef of string mcode (* #define *) * ident (* name *)
   | Define of string mcode (* #define *) * ident (* name *) *
 	define_parameters (*params*) * statement dots
-  | Pragma of string mcode (* #pragma *) * ident * pragmainfo
   | OptStm   of statement
 
-and base_pragmainfo =
-    PragmaString of string mcode
-  | PragmaDots of string mcode
-
-and pragmainfo = base_pragmainfo wrap
+and base_templateParameterTypeDef =
+    TypenameOrClassParam of string mcode (* typename|class *) * ident (* name *) *
+	(string mcode (* = *) * typeC) option
+  | VarNameParam of typeC * ident (* name *) * (string mcode (* = *) * initialiser) option
+  | TPComma of string mcode
+  | TPDots of string mcode (* ... *)
+  (* Note: TemplateParam not supported yet. *)
+and templateParameterTypeDef = base_templateParameterTypeDef wrap
+and template_parameter_list = templateParameterTypeDef dots
 
 and base_forinfo =
-    ForExp of expression option * string mcode (*;*)
-  | ForDecl of (info * mcodekind) (* before the decl *) * declaration
+    ForExp of expression option * string mcode (*;*) *
+	expression option * string mcode (*;*) *
+        expression option
+  | ForDecl of (info * mcodekind) (* before the decl *) * declaration *
+	expression option * string mcode (*;*) *
+        expression option
+  | ForRange of (info * mcodekind) (* before the decl *) * declaration *
+	initialiser
+
 
 and forinfo = base_forinfo wrap
 
+and whileinfo =
+    WhileExp  of expression
+  | WhileDecl of (info * mcodekind) (* before the decl *) * declaration
+
 and fninfo =
     FStorage of Ast.storage mcode
   | FType of typeC
   | FInline of string mcode
-  | FAttr of attr
+
+and cvattr =
+    CV of Ast.const_vol mcode
+  | Attr of attr
 
 and base_attr =
-    Attribute of string mcode
-  | MetaAttribute of Ast.meta_name mcode * constraints * pure
+    Attribute of attr_arg
+  | GccAttribute of string mcode (* __attribute__ *) *
+                    string mcode (* ( *) * string mcode (* ( *) *
+                    expression dots * string mcode (* ) *) * string mcode (* ) *)
+  | CxxAttribute of string mcode (* [[ *) *
+                    expression dots * string mcode (* ] *) * string mcode (* ] *)
+  | CxxAttributeUsing of string mcode (* [[ *) * string mcode (* using *) *
+                    ident * string mcode (* : *) *
+                    expression dots * string mcode (* ] *) * string mcode (* ] *)
 
 and attr = base_attr wrap
 
+and base_attr_arg =
+    MacroAttr of string mcode
+  | MacroAttrArgs of string mcode * string mcode (* ( *) *
+                      expression dots * string mcode (* ) *)
+  | MetaAttr of Ast.meta_name mcode * constraints * pure
+
+and attr_arg = base_attr_arg wrap
+
 and ('a,'b) whencode =
     WhenNot of string mcode (* when *) * string mcode (* != *) * 'a
   | WhenAlways of string mcode (* when *) * string mcode (* = *) * 'b
@@ -476,6 +535,26 @@ and ('a,'b) whencode =
 
 and statement = base_statement wrap
 
+and base_directive = 
+  | Include of string mcode (* #include *) * Ast.inc_file mcode (* file *)
+  | MetaInclude of string mcode (* #include *) * expression (* file *)
+  | Pragma of string mcode (* #pragma *) * ident * pragmainfo
+  | UsingNamespace of string mcode (*using*) * string mcode (*namespace*) *
+      ident (*name*) * string mcode (*;*)
+  | UsingTypename of string mcode (*using*) * ident (*name*) *
+      string mcode (*=*) * string mcode option (*typename*) *
+      typeC (*full_type*) * string mcode (*;*)
+  | UsingMember of string mcode (*using*) * ident (*name*) * string mcode (*;*)
+
+and directive = base_directive wrap
+
+and base_pragmainfo =
+    PragmaString of string mcode
+  | PragmaDots of string mcode
+  | MetaPragmaInfo of Ast_cocci.meta_name mcode * constraints * pure
+
+and pragmainfo = base_pragmainfo wrap
+
 and base_case_line =
     Default of string mcode (* default *) * string mcode (*:*) * statement dots
   | Case of string mcode (* case *) * expression * string mcode (*:*) *
@@ -556,6 +635,7 @@ and anything =
     DotsExprTag of expression dots
   | DotsInitTag of initialiser dots
   | DotsParamTag of parameterTypeDef dots
+  | DotsTemplateParamTag of templateParameterTypeDef dots
   | DotsStmtTag of statement dots
   | DotsDeclTag of declaration dots
   | DotsFieldTag of field dots
@@ -570,6 +650,7 @@ and anything =
   | TestExprTag of expression (* for isos *)
   | TypeCTag of typeC
   | ParamTag of parameterTypeDef
+  | TemplateParamTag of templateParameterTypeDef
   | InitTag of initialiser
   | DeclTag of declaration
   | FieldTag of field
@@ -579,6 +660,7 @@ and anything =
   | CaseLineTag of case_line
   | StringFragmentTag of string_fragment
   | AttributeTag of attr
+  | AttrArgTag of attr_arg
   | TopTag of top_level
   | IsoWhenTag of Ast.when_modifier
   | IsoWhenTTag of expression
@@ -590,6 +672,7 @@ and anything =
 
 let dotsExpr x = DotsExprTag x
 let dotsParam x = DotsParamTag x
+let dotsTemplateParam x = DotsTemplateParamTag x
 let dotsInit x = DotsInitTag x
 let dotsStmt x = DotsStmtTag x
 let dotsDecl x = DotsDeclTag x
@@ -603,6 +686,7 @@ let assignOp x = AssignOpTag x
 let binaryOp x = BinaryOpTag x
 let typeC x = TypeCTag x
 let param x = ParamTag x
+let template_param x = TemplateParamTag x
 let ini x = InitTag x
 let decl x = DeclTag x
 let field x = FieldTag x
@@ -611,6 +695,7 @@ let forinfo x = ForInfoTag x
 let case_line x = CaseLineTag x
 let string_fragment x = StringFragmentTag x
 let attr x = AttributeTag x
+let attr_arg x = AttrArgTag x
 let top x = TopTag x
 let enum_decl x = EnumDeclTag x
 
@@ -677,7 +762,6 @@ let set_index x i   = x.index := i
 let get_mcodekind x = !(x.mcodekind)
 let get_mcode_mcodekind (_,_,_,mcodekind,_,_) = mcodekind
 let get_mcode_line (_,_,info,_,_,_) = info.pos_info.line_start
-let get_mcode_logline (_,_,info,_,_,_) = info.pos_info.logical_start
 let get_mcodekind_ref x = x.mcodekind
 let set_mcodekind x mk  = x.mcodekind := mk
 let set_type x t        = x.exp_ty := t
@@ -752,11 +836,11 @@ let meta_names_of_expression expression =
 
 let rec meta_names_of_typeC ty =
   match unwrap ty with
-    ConstVol (_, ty)
+    ConstVol (_, ty, _)
   | Signed (_, Some ty)
   | Pointer (ty, _)
   | Array (ty, _, _, _) -> meta_names_of_typeC ty
-  | EnumName (_, Some ident)
+  | EnumName (_, _, Some ident)
   | StructUnionName(_, Some ident) -> meta_names_of_ident ident
   | MetaType (tyname, _, _) -> [unwrap_mcode tyname]
   | Decimal (_, _, e1, _, e2, _) ->
@@ -787,10 +871,12 @@ let fresh_index _ = let cur = !index_counter in index_counter := cur + 1; cur
 (* this function is a rather minimal attempt.  the problem is that information
 has been lost.  but since it is only used for metavariable types in the isos,
 perhaps it doesn't matter *)
-let make_mcode x = (x,NONE,default_info(),context_befaft(),ref [],-1)
-let make_mcode_info x info = (x,NONE,info,context_befaft(),ref [],-1)
+let make_mcode x =
+  (x,NONE,default_info(),context_befaft(),ref [],{Ast.counter = -1; Ast.ender = false})
+let make_mcode_info x info =
+  (x,NONE,info,context_befaft(),ref [],{Ast.counter = -1; Ast.ender = false})
 and make_minus_mcode x =
-  (x,NONE,default_info(),minus_befaft(),ref [],-1)
+  (x,NONE,default_info(),minus_befaft(),ref [],{Ast.counter = -1; Ast.ender = false})
 
 (* --------------------------------------------------------------------- *)
 
