@@ -44,7 +44,10 @@ val field : Ast0_cocci.field -> Ast_cocci.field
 val enum_decl : Ast0_cocci.enum_decl -> Ast_cocci.enum_decl
 val parameterTypeDef :
     Ast0_cocci.parameterTypeDef -> Ast_cocci.parameterTypeDef
+val templateParameterTypeDef :
+    Ast0_cocci.templateParameterTypeDef -> Ast_cocci.templateParameterTypeDef
 val parameter_list : Ast0_cocci.parameter_list -> Ast_cocci.parameter_list
+val template_parameter_list : Ast0_cocci.template_parameter_list -> Ast_cocci.template_parameter_list
 val top_level : Ast0_cocci.top_level -> Ast_cocci.top_level
 val convert_info : Ast0_cocci.info -> Ast_cocci.info
 val constraints : Ast0_cocci.constraints -> Ast_cocci.constraints
