@@ -196,6 +196,7 @@ and base_expression =
                    string mcode (* ) *) option * arguments option
   | TemplateInst   of expression (* name *) * string mcode (* < *) *
                       expression dots * string mcode (* > *)
+  | TupleExpr      of initialiser
   | TypeExp        of fullType
 
   | Paren          of string mcode (* ( *) * expression *
@@ -391,7 +392,7 @@ and const_vol = Const | Volatile
 
 and base_declaration =
   | Init of alignas option * storage mcode option * fullType * ident *
-        attr list * string mcode (*=*) * initialiser * string mcode (*;*)
+        attr list * string mcode (*=*) * initialiser * string mcode option (*;*)
   | UnInit of alignas option * storage mcode option * fullType * ident *
 	attr list * string mcode (* ; *)
   | FunProto of
@@ -440,6 +441,7 @@ and base_field =
   | MetaField of meta_name mcode * constraints * keep_binding * inherited
   | MetaFieldList of meta_name mcode * listlen * constraints * keep_binding *
 	inherited
+  | AccSpec of string mcode * string mcode   (* C++ *)
 
 and bitfield = string mcode (* : *) * expression
 
@@ -572,7 +574,7 @@ and base_rule_elem =
 	             expression * string mcode (* ) *)
   | Else          of string mcode (* else *)
   | WhileHeader   of string mcode (* while *) * string mcode (* ( *) *
-	             expression * string mcode (* ) *)
+	             whileinfo * string mcode (* ) *)
   | DoHeader      of string mcode (* do *)
   | WhileTail     of string mcode (* while *) * string mcode (* ( *) *
 	             expression * string mcode (* ) *) *
@@ -641,6 +643,10 @@ and forinfo =
         expression option
   | ForRange of annotated_decl * initialiser
 
+and whileinfo =
+    WhileExp  of expression
+  | WhileDecl of annotated_decl
+
 and fninfo =
     FStorage of storage mcode
   | FType of fullType
@@ -889,12 +895,10 @@ val unwrap_mcode : 'a mcode -> 'a
 val get_mcodekind : 'a mcode -> mcodekind
 val get_line : 'a wrap -> line
 val get_mcode_line : 'a mcode -> line
-val get_mcode_col : 'a mcode -> int
 val get_fvs : 'a wrap -> meta_name list
 val get_wcfvs : ('a wrap,'b wrap) whencode list -> meta_name list
 val set_fvs : meta_name list -> 'a wrap -> 'a wrap
 val get_mfvs : 'a wrap -> meta_name list
-val set_mfvs : meta_name list -> 'a wrap -> 'a wrap
 val get_minus_nc_fvs : 'a wrap -> meta_name list
 val get_fresh : 'a wrap -> (meta_name * seed) list
 val get_inherited : 'a wrap -> meta_name list
@@ -917,8 +921,6 @@ val drop_pos : 'a mcode -> 'a mcode
 
 val get_meta_name : metavar -> meta_name
 
-val tag2c : anything -> string
-
 val no_info : info
 
 val make_meta_rule_elem :
@@ -926,10 +928,15 @@ val make_meta_rule_elem :
       (meta_name list * (meta_name * seed) list * meta_name list) ->
       rule_elem
 
-val make_meta_decl :
+val make_meta_id :
     string -> mcodekind -> constraints ->
       (meta_name list * (meta_name * seed) list * meta_name list) ->
-      declaration
+      ident
+
+val make_meta_type :
+    string -> mcodekind -> constraints ->
+      (meta_name list * (meta_name * seed) list * meta_name list) ->
+      typeC
 
 val make_term : 'a -> 'a wrap
 val make_inherited_term : 'a -> meta_name list (* inherited vars *) ->
@@ -947,7 +954,6 @@ val string_of_sign : sign -> string
 val string_of_baseType : baseType -> string
 val string_of_const_vol : const_vol -> string
 val string_of_structUnion : structUnion -> string
-val string_of_typeC : typeC -> string
 val string_of_fullType : fullType -> string
 
 val typeC_of_fullType_opt : fullType -> typeC option
@@ -1052,10 +1058,6 @@ val cstr_meta_names: 'expression generic_constraints -> meta_name list
 
 val cstr_pos_meta_names: 'expression generic_constraints -> meta_name list
 
-val filter_merge_variables:
-    (script_meta_name * meta_name * metavar * mvinit) list ->
-      (string * string) list
-
 val prepare_merge_variables:
     ('a -> ('b * (script_meta_name * meta_name * metavar * mvinit) list) option)
     -> 'a list -> ('b * (int * string array)) list * string array
