@@ -220,6 +220,8 @@ and expression context old_metas table minus e =
   | Ast0.TemplateInst(name,lab,args,rab) ->
       expression ID old_metas table minus name;
       dots (expression ID old_metas table minus) args
+  | Ast0.TupleExpr(init) ->
+      initialiser old_metas table minus init
   | Ast0.TypeExp(ty) -> typeC old_metas table minus ty
   | Ast0.Constructor(lp,ty,rp,init) ->
       typeC old_metas table minus ty; initialiser old_metas table minus init
@@ -404,6 +406,7 @@ and field context old_metas table minus d =
   | Ast0.DisjField(_,decls,_,_)
   | Ast0.ConjField(_,decls,_,_) ->
       List.iter (field ID old_metas table minus) decls
+  | Ast0.AccSpec(_) -> ()
   | Ast0.OptField(_) ->
       failwith "unexpected code"
   | Ast0.Fdots(_,Some (_,_,x)) -> field ID old_metas table minus x
@@ -515,8 +518,8 @@ and statement old_metas table minus s =
       expression ID old_metas table minus exp;
       statement old_metas table minus branch1;
       statement old_metas table minus branch2
-  | Ast0.While(wh,lp,exp,rp,body,_) ->
-      expression ID old_metas table minus exp;
+  | Ast0.While(wh,lp,cond,rp,body,_) ->
+      whileinfo old_metas table minus cond;
       statement old_metas table minus body
   | Ast0.Do(d,body,wh,lp,exp,rp,sem) ->
       statement old_metas table minus body;
@@ -559,6 +562,7 @@ and statement old_metas table minus s =
   | Ast0.Ty(ty) -> typeC old_metas table minus ty
   | Ast0.TopId(id) -> ident ID old_metas table minus id
   | Ast0.TopInit(init) -> initialiser old_metas table minus init
+  | Ast0.CppTop(di) -> directive old_metas table minus di
   | Ast0.Disj(_,rule_elem_dots_list,_,_)
   | Ast0.Conj(_,rule_elem_dots_list,_,_) ->
       List.iter (dots (statement old_metas table minus)) rule_elem_dots_list
@@ -589,6 +593,11 @@ and statement old_metas table minus s =
   | Ast0.Goto(_,i,_) -> ident ID old_metas table minus i
   | _ -> () (* no metavariable subterms *)
 
+and whileinfo old_metas table minus c =
+  match c with
+    Ast0.WhileExp(e) -> expression ID old_metas table minus e
+  | Ast0.WhileDecl(_,d) -> declaration ID old_metas table minus d
+
 and pragmainfo old_metas table minus pi =
   match Ast0.unwrap pi with
     Ast0.PragmaString(s) -> ()
