@@ -479,6 +479,9 @@ let rec expression e =
       let args = dots is_exp_dots (Some(promote_mcode lp)) expression args in
       let rp = normal_mcode rp in
       mkres e (Ast0.TemplateInst(tn,lp,args,rp)) tn (promote_mcode rp)
+  | Ast0.TupleExpr(init) ->
+      let init = initialiser init in
+      mkres e (Ast0.TupleExpr(init)) init init
   | Ast0.TypeExp(ty) ->
       let ty = typeC ty in mkres e (Ast0.TypeExp(ty)) ty ty
   | Ast0.Constructor(lp,ty,rp,init) ->
@@ -756,7 +759,7 @@ and alignas (Ast0.Align(al,lp,expr,rp)) =
       let lp = normal_mcode lp in
       let expr = expression expr in
       let rp = normal_mcode rp in
-      (Ast0.Align(al,lp,expr,rp), al)
+      Ast0.Align(al,lp,expr,rp)
 
 and declaration d =
   match Ast0.unwrap d with
@@ -765,39 +768,45 @@ and declaration d =
       let ln = promote_mcode name in
       mkres d (Ast0.MetaDecl(name,a,b)) ln ln
   | Ast0.Init(al,stg,ty,id,endattr,eq,exp,sem) ->
+      let al = get_option alignas al in
+      let stg = get_option normal_mcode stg in
       let ty = typeC ty in
       let id = ident id in
       let endattr = List.map attribute endattr in
       let eq = normal_mcode eq in
       let exp = initialiser exp in
-      let sem = normal_mcode sem in
-      (match (al,stg) with
-	(None,None) ->
-	  mkres d (Ast0.Init(al,stg,ty,id,endattr,eq,exp,sem)) ty (promote_mcode sem)
-      | (None,Some x)  ->
-	  let stg = Some (normal_mcode x) in
-	  mkres d (Ast0.Init(al,stg,ty,id,endattr,eq,exp,sem))
-	    (promote_mcode x) (promote_mcode sem)
-      | (Some al, _) ->
-          let (al,x) = alignas al in
-          mkres d (Ast0.Init(Some al,stg,ty,id,endattr,eq,exp,sem))
-	    (promote_mcode x) (promote_mcode sem))
+      let sem = get_option normal_mcode sem in
+      let re_init left right =
+	mkres d (Ast0.Init(al,stg,ty,id,endattr,eq,exp,sem)) left right in
+      (match (al,stg,sem) with
+	(None,None,None) ->
+	  re_init ty exp
+      | (None,None,Some sem) ->
+	  re_init ty (promote_mcode sem)
+      | (None,Some stg,None) ->
+	  re_init (promote_mcode stg) exp
+      | (None,Some stg,Some sem) ->
+	  re_init (promote_mcode stg) (promote_mcode sem)
+      | (Some (Ast0.Align(al,lp,expr,rp)),_,None) ->
+	  re_init (promote_mcode al) exp
+      | (Some (Ast0.Align(al,lp,expr,rp)),_,Some sem) ->
+	  re_init (promote_mcode al) (promote_mcode sem))
   | Ast0.UnInit(al,stg,ty,id,endattr,sem) ->
+      let al = get_option alignas al in
+      let stg = get_option normal_mcode stg in
       let ty = typeC ty in
       let id = ident id in
       let endattr = List.map attribute endattr in
       let sem = normal_mcode sem in
+      let re_init left right =
+	mkres d (Ast0.UnInit(al,stg,ty,id,endattr,sem)) left right in
       (match (al,stg) with
 	(None,None) ->
-	  mkres d (Ast0.UnInit(al,stg,ty,id,endattr,sem)) ty (promote_mcode sem)
-      | (None,Some x)  ->
-	  let stg = Some (normal_mcode x) in
-	  mkres d (Ast0.UnInit(al,stg,ty,id,endattr,sem))
-	    (promote_mcode x) (promote_mcode sem)
-      | (Some al, _) ->
-          let (al,x) = alignas al in
-          mkres d (Ast0.UnInit(Some al,stg,ty,id,endattr,sem))
-	    (promote_mcode x) (promote_mcode sem))
+	  re_init ty (promote_mcode sem)
+      | (None,Some stg)  ->
+	  re_init (promote_mcode stg) (promote_mcode sem)
+      | (Some (Ast0.Align(al,lp,expr,rp)), _) ->
+	  re_init (promote_mcode al) (promote_mcode sem))
 
   | Ast0.FunProto(fninfo,name,lp1,params,va1,rp1,sem) ->
       let fninfo =
@@ -933,6 +942,12 @@ and field d =
       do_disj d starter decls mids ender field
 	(fun starter decls mids ender ->
 	  Ast0.ConjField(starter,decls,mids,ender))
+  | Ast0.AccSpec(decl,dd) ->
+      let l = promote_mcode decl in
+      let r = promote_mcode dd in
+      let decl = normal_mcode decl in
+      let dd = normal_mcode dd in
+      mkres d (Ast0.AccSpec(decl,dd)) l r
   | Ast0.OptField(decl) ->
       let decl = field decl in
       mkres d (Ast0.OptField(field decl)) decl decl
@@ -1329,14 +1344,14 @@ let rec statement s =
 	  (Ast0.IfThenElse(iff,lp,exp,rp,branch1,els,branch2,
 	    (rightinfo,aft,adj)))
 	  (promote_mcode iff) right
-    | Ast0.While(wh,lp,exp,rp,body,(_,aft,adj)) ->
+    | Ast0.While(wh,lp,cond,rp,body,(_,aft,adj)) ->
 	let wh = normal_mcode wh in
 	let lp = normal_mcode lp in
-	let exp = expression exp in
+	let cond = whileinfo cond in
 	let rp = normal_mcode rp in
 	let body = statement body in
 	let (rightinfo,right,body) = promote_to_statement_end body aft in
-	mkres s (Ast0.While(wh,lp,exp,rp,body,(rightinfo,aft,adj)))
+	mkres s (Ast0.While(wh,lp,cond,rp,body,(rightinfo,aft,adj)))
 	  (promote_mcode wh) right
     | Ast0.Do(d,body,wh,lp,exp,rp,sem) ->
 	let d = normal_mcode d in
@@ -1608,6 +1623,13 @@ let rec statement s =
     | Ast0.DroppingBetweenDots s ->
 	Ast0.DroppingBetweenDots(statement s))
 
+and whileinfo = function
+      Ast0.WhileExp(e) -> Ast0.WhileExp(expression e)
+    | Ast0.WhileDecl((_,bef),decl) ->
+	let decl = declaration decl in
+	let (leftinfo,decl) = promote_to_statement_start decl bef in
+	Ast0.WhileDecl((leftinfo,bef),decl)
+
 and leftfninfo fninfo name bef = (* cases on what is leftmost *)
   match fninfo with
     [] ->
