@@ -270,6 +270,8 @@ let bind c1 c2 =
 let option_default = (*Bind(Neutral,[],[],[],[],[])*)
   Recursor(Neutral,[],[],[])
 
+let get_option f = Common.default option_default f
+
 let contains_added_strings info =
   let unsafe l =
     List.exists
@@ -434,8 +436,8 @@ let classify is_minus all_marked table code =
 	   using rebuilder, which just visits the subterms, rather than
 	   reordering their components. *)
       |	Ast0.Init(al,stg,ty,id,endattr,eq,ini,sem) ->
-	  bind (match al with Some al -> alignas r al | _ -> option_default)
-	    (bind (match stg with Some stg -> mcode stg | _ -> option_default)
+	  bind (get_option (alignas r) al)
+	    (bind (get_option mcode stg)
 	      (bind (r.VT0.combiner_rec_typeC ty)
                  (bind (r.VT0.combiner_rec_ident id)
                     (bind
@@ -444,10 +446,10 @@ let classify is_minus all_marked table code =
                           option_default)
                        (bind (mcode eq)
                           (bind (r.VT0.combiner_rec_initialiser ini)
-                             (mcode sem)))))))
+                             (get_option mcode sem)))))))
       | Ast0.UnInit(al,stg,ty,id,endattr,sem) ->
-          bind (match al with Some al -> alignas r al | _ -> option_default)
-	    (bind (match stg with Some stg -> mcode stg | _ -> option_default)
+          bind (get_option (alignas r) al)
+	    (bind (get_option mcode stg)
 	      (bind (r.VT0.combiner_rec_typeC ty)
                  (bind (r.VT0.combiner_rec_ident id)
                     (bind
@@ -603,20 +605,6 @@ the same context children *)
 let equal_mcode (_,_,info1,_,_,_) (_,_,info2,_,_,_) =
   info1.Ast0.pos_info.Ast0.offset = info2.Ast0.pos_info.Ast0.offset
 
-let assignOp_equal_mcode op1 op2 =
-  match (Ast0.unwrap op1, Ast0.unwrap op2) with
-    Ast0.SimpleAssign op1', Ast0.SimpleAssign op2' -> equal_mcode op1' op2'
-  | Ast0.OpAssign op1', Ast0.OpAssign op2' -> equal_mcode op1' op2'
-  | Ast0.MetaAssign(mv1,_,_), Ast0.MetaAssign(mv2,_,_) -> equal_mcode mv1 mv2
-  | _ -> false
-
-let binaryOp_equal_mcode op1 op2 =
-  match (Ast0.unwrap op1, Ast0.unwrap op2) with
-    Ast0.Arith op1', Ast0.Arith op2' -> equal_mcode op1' op2'
-  | Ast0.Logical op1', Ast0.Logical op2' -> equal_mcode op1' op2'
-  | Ast0.MetaBinary(mv1,_,_), Ast0.MetaBinary(mv2,_,_) -> equal_mcode mv1 mv2
-  | _ -> false
-
 let equal_option e1 e2 =
   match (e1,e2) with
     (Some x, Some y) -> equal_mcode x y
@@ -727,6 +715,7 @@ let rec equal_expression e1 e2 =
   | (Ast0.TypeExp(_),Ast0.TypeExp(_)) -> true
   | (Ast0.Constructor(lp1,_,rp1,_),Ast0.Constructor(lp2,_,rp2,_)) ->
       equal_mcode lp1 lp2 && equal_mcode rp1 rp2
+  | (Ast0.TupleExpr(_),Ast0.TupleExpr(_)) -> true
   | (Ast0.MetaErr(name1,_,_),Ast0.MetaErr(name2,_,_))
   | (Ast0.MetaExpr(name1,_,_,_,_,_),Ast0.MetaExpr(name2,_,_,_,_,_))
   | (Ast0.MetaExprList(name1,_,_,_),Ast0.MetaExprList(name2,_,_,_)) ->
@@ -854,7 +843,7 @@ let equal_declaration d1 d2 =
       equal_option stg1 stg2 &&
       (List.length endattr1) = (List.length endattr2) &&
       List.for_all2 equal_attribute endattr1 endattr2 &&
-      equal_mcode eq1 eq2 && equal_mcode sem1 sem2
+      equal_mcode eq1 eq2 && equal_option sem1 sem2
   | (Ast0.UnInit(_,stg1,_,_,endattr1,sem1),Ast0.UnInit(_,stg2,_,_,endattr2,sem2)) ->
       equal_option stg1 stg2 &&
       (List.length endattr1) = (List.length endattr2) &&
@@ -936,6 +925,8 @@ let equal_field d1 d2 =
       equal_mcode lp1 lp2 && equal_mcode lp1 lp2 &&
       equal_mcode sem1 sem2
   | (Ast0.CppField di1,Ast0.CppField di2) -> equal_directive di1 di2
+  | (Ast0.AccSpec(decl1,dd1), Ast0.AccSpec(decl2,dd2)) ->
+      equal_mcode decl1 decl2 && equal_mcode dd1 dd2
        
   | (Ast0.Fdots(dots1,_),Ast0.Fdots(dots2,_)) -> equal_mcode dots1 dots2
   | (Ast0.OptField(_),Ast0.OptField(_)) -> true
