@@ -110,6 +110,10 @@ val add_binaryOp_meta:
     (Ast_cocci.meta_name ->
       Ast0_cocci.constraints -> Ast0_cocci.pure -> unit) ref
 
+val add_pragmainfo_meta:
+    (Ast_cocci.meta_name ->
+      Ast0_cocci.constraints -> Ast0_cocci.pure -> unit) ref
+
 val add_type_name: (string -> unit) ref
 val add_attribute: (string -> unit) ref
 val add_attribute_meta: cstr_meta_type ref
@@ -141,9 +145,11 @@ val fresh_id_scripts:
 (** The list of all fresh id scripts. *)
 
 (* ---------------------------------------------------------------------- *)
-(* Names of some special tokens.  Make these acessible to the C parser *)
+(* Names of some special tokens.  Make these accessible to the C parser *)
 
-val type_names : string list ref
-val attr_names : string list ref
-val declarer_names : string list ref
-val iterator_names : string list ref
+type nametypes = Type | Attr | AttrArgs | Declarer | Iterator | CommaInit | Ambiguous
+val add_special_name : string -> nametypes -> unit
+val get_special_name : string -> nametypes
+val clear_special_names : unit -> unit
+val set_define_params : string list -> unit
+val clear_define_params : unit -> unit
