@@ -42,11 +42,6 @@ let rec disjmult f = function
       let rest = disjmult f xs in
       disjmult2 cur rest (function cur -> function rest -> cur :: rest)
 
-let disjmult_two fstart frest (start,rest) =
-  let cur = fstart start in
-  let rest = disjmult frest rest in
-  disjmult2 cur rest (function cur -> function rest -> (cur,rest))
-
 let disjtwoelems fstart frest (start,rest) =
   let cur = fstart start in
   let rest = frest rest in
@@ -279,6 +274,10 @@ and disjexp e =
       disjmult2 (disjexp tn) (disjdots disjexp args)
 	(fun tn args ->
 	  Ast.rewrap e (Ast.TemplateInst(tn,lp,args,rp)))
+  | Ast.TupleExpr(init) ->
+      let init = disjini init in
+      List.map (function init ->
+	   Ast.rewrap e (Ast.TupleExpr(init))) init
   | Ast.TypeExp(ty) ->
       let ty = disjty ty in
       List.map (function ty -> Ast.rewrap e (Ast.TypeExp(ty))) ty
@@ -461,6 +460,7 @@ and disjfield d =
   | Ast.CppField di ->
       let di = disjdirective di in
       List.map (fun di -> Ast.rewrap d (Ast.CppField di)) di
+  | Ast.AccSpec(_) -> [d]
 
 and disjdirective di =
   match Ast.unwrap di with
@@ -520,9 +520,12 @@ let rec disj_rule_elem r k re =
       orify_rule_elem re exp
 	(function exp -> Ast.rewrap re (Ast.IfHeader(iff,lp,exp,rp)))
   | Ast.Else(els) -> re
-  | Ast.WhileHeader(whl,lp,exp,rp) ->
+  | Ast.WhileHeader(whl,lp,Ast.WhileExp(exp),rp) ->
       orify_rule_elem re exp
-	(function exp -> Ast.rewrap re (Ast.WhileHeader(whl,lp,exp,rp)))
+	(function exp -> Ast.rewrap re (Ast.WhileHeader(whl,lp,Ast.WhileExp(exp),rp)))
+  | Ast.WhileHeader(whl,lp,Ast.WhileDecl(decl),rp) ->
+      generic_orify_rule_elem anndisjdecl re decl
+	(function decl -> Ast.rewrap re (Ast.WhileHeader(whl,lp,Ast.WhileDecl(decl),rp)))
   | Ast.DoHeader(d) -> re
   | Ast.WhileTail(whl,lp,exp,rp,sem) ->
       orify_rule_elem re exp
