@@ -137,10 +137,10 @@ let collect_refs include_constraints =
 	  bind (constraints cstr) [metaid name]
       | _ -> option_default) in
 
-  let astfvattribute recursor k a =
+  let astfvattr_arg recursor k a =
     bind (k a)
       (match Ast.unwrap a with
-	Ast.MetaAttribute(name,cstr,_,_) ->
+	Ast.MetaAttr(name,cstr,_,_) ->
 	  bind (constraints cstr) [metaid name]
       | _ -> option_default) in
 
@@ -162,6 +162,15 @@ let collect_refs include_constraints =
   let astfvbinaryop recursor k bop =
     bind (k bop) (collect_binary_names bop) in
 
+  let rec collect_pragma_info_names pm =
+    match Ast.unwrap pm with
+      Ast.MetaPragmaInfo(name,cstr,_,_) ->
+	bind (constraints cstr) [metaid name]
+    | _ -> option_default in
+
+  let astfvpragmainfo recursor k pm =
+    bind (k pm) (collect_pragma_info_names pm) in
+
   let astfvdecls recursor k d =
     match Ast.unwrap d with
       Ast.MetaDecl(name,cstr,_,_) ->
@@ -199,7 +208,8 @@ let collect_refs include_constraints =
   let astfvinit recursor k ty =
     bind (k ty)
       (match Ast.unwrap ty with
-	Ast.MetaInit(name,cstr,_,_) -> [metaid name]
+	Ast.MetaInit(name,cstr,_,_) ->
+	  bind (constraints cstr) [metaid name]
       | Ast.MetaInitList(name,len,cstr,_,_) ->
 	  bind (constraints cstr) (bind (listlen len) [metaid name])
       | _ -> option_default) in
@@ -253,15 +263,15 @@ let collect_refs include_constraints =
 	   (Ast.get_pos_var mc))
     (* else option_default *) in
 
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    astfvident astfvexpr astfvfrag astfvfmt astfvassignop astfvbinaryop
-    astfvfullType astfvtypeC astfvinit astfvparam astfvdefine_param
-    astfvdecls donothing astfvfields astafvfields donothing
-    astfvrule_elem astfvstatement donothing astfvattribute
-    donothing donothing_a
+  V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing}
+    ~ident:astfvident ~expr:astfvexpr ~string_fragment:astfvfrag ~fmt:astfvfmt
+    ~assignOp:astfvassignop ~binaryOp:astfvbinaryop
+    ~pragma_info:astfvpragmainfo
+    ~ft:astfvfullType ~ty:astfvtypeC ~init:astfvinit ~param:astfvparam
+    ~define_param:astfvdefine_param
+    ~decl:astfvdecls ~field:astfvfields ~annotated_field:astafvfields
+    ~rule:astfvrule_elem ~stmt:astfvstatement ~attr_arg:astfvattr_arg
+    donothing_a
 
 let collect_all_refs = collect_refs true
 let collect_non_constraint_refs = collect_refs false
@@ -304,14 +314,8 @@ let collect_pos_positions =
 	List.fold_left Common.inter_set (List.hd subres) (List.tl subres)
     | _ -> k s in
 
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing
-    cprule_elem cpstmt donothing donothing donothing donothing
+  V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing}
+    ~rule:cprule_elem ~stmt:cpstmt donothing
 
 (* ---------------------------------------------------------------- *)
 
@@ -377,10 +381,10 @@ let collect_saved =
 	Ast.MetaFormat(name,_,Ast.Saved,_) -> [metaid name]
       | _ -> option_default) in
 
-  let astfvattribute recursor k a =
+  let astfvattr_arg recursor k a =
     bind (k a)
       (match Ast.unwrap a with
-	Ast.MetaAttribute(name,_,Ast.Saved,_) -> [metaid name]
+	Ast.MetaAttr(name,_,Ast.Saved,_) -> [metaid name]
       | _ -> option_default) in
 
   let astfvassign recursor k aop =
@@ -395,6 +399,12 @@ let collect_saved =
 	Ast.MetaBinary(name,_,Ast.Saved,_) -> [metaid name]
       | _ -> option_default) in
 
+  let astfvpragmainfo recursor k v =
+    bind (k v)
+      (match Ast.unwrap v with
+	Ast.MetaPragmaInfo(name,_,Ast.Saved,_) -> [metaid name]
+      | _ -> option_default) in
+
   let astfvtypeC recursor k ty =
     bind (k ty)
       (match Ast.unwrap ty with
@@ -486,14 +496,11 @@ let collect_saved =
 	  | _ -> acc)
       option_default (Ast.get_pos_var e) in
 
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    astfvident astfvexpr astfvfrag astfvfmt astfvassign astfvbinary donothing
-    astfvtypeC astfvinit astfvparam astfvdefine_param astfvdecls donothing
-    astfvfields donothing donothing astfvrule_elem donothing donothing
-    astfvattribute donothing donothing
+  V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing}
+    ~ident:astfvident ~expr:astfvexpr ~string_fragment:astfvfrag ~fmt:astfvfmt ~assignOp:astfvassign
+    ~binaryOp:astfvbinary ~pragma_info:astfvpragmainfo ~ty:astfvtypeC ~init:astfvinit ~param:astfvparam
+    ~define_param:astfvdefine_param ~decl:astfvdecls ~field:astfvfields ~rule:astfvrule_elem
+    ~attr_arg:astfvattr_arg donothing
 
 (* ---------------------------------------------------------------- *)
 
@@ -582,7 +589,7 @@ let collect_in_plus_term =
 
   let astfvrule_elem recursor k re =
     match Ast.unwrap re with
-      Ast.FunHeader(bef,_,fi,nm,_,params,_,_) ->
+      Ast.FunHeader(bef,_,fi,nm,_,params,_,_,_) ->
 	bind (cip_mcodekind recursor bef) (k re)
 	  (* no clue why this code is here *) (*
 	let fi_metas =
@@ -598,7 +605,7 @@ let collect_in_plus_term =
 	    (List.map
 	       (function p ->
 	          match Ast.unwrap p with
-	            Ast.VoidParam(t) | Ast.Param(t,_) ->
+	            Ast.Param(t,_) ->
 	              collect_all_refs.V.combiner_fullType t
 		  | _ -> [])
 	       (Ast.unwrap params)) in
@@ -608,7 +615,8 @@ let collect_in_plus_term =
 		(bind (cip_mcodekind recursor bef) (k re)))) *)
     | Ast.Decl decl ->
 	bind (cip_mcodekind recursor (annotated_decl decl)) (k re)
-    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
+    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl,_,_,_),rp)
+    | Ast.ForHeader(fr,lp,Ast.ForRange(decl,_),rp) ->
 	bind (cip_mcodekind recursor (annotated_decl decl)) (k re)
     | _ -> k re in
 
@@ -620,13 +628,8 @@ let collect_in_plus_term =
 	bind (k s) (cip_mcodekind recursor aft)
     | _ -> k s in
 
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing astfvrule_elem
-    astfvstatement donothing donothing donothing donothing
+  V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing}
+    ~rule:astfvrule_elem ~stmt:astfvstatement donothing
 
 let collect_in_plus metavars minirules =
   nub
@@ -726,9 +729,9 @@ let classify_variables metavar_decls minirules used_after =
             Ast.Ident ident -> Ast.rewrap e (Ast.Ident (classify_ident ident))
           | _ -> e in
         Ast.Decimal (s0, s1, e2mv e1, s2, Common.map_option e2mv e2, s3));
-      enumName = Some (fun s0 ident ->
+      enumName = Some (fun s0 s1 ident ->
         let ident' = Common.map_option classify_ident ident in
-        Ast.EnumName (s0, ident'));
+        Ast.EnumName (s0, s1, ident'));
       structUnionName = Some (fun su ident ->
         let ident' = Common.map_option classify_ident ident in
         Ast.StructUnionName (su, ident'));
@@ -793,12 +796,12 @@ let classify_variables metavar_decls minirules used_after =
 	Ast.rewrap ft (Ast.MetaFormat(name,constraints,unitary,inherited))
     | _ -> ft in
 
-  let attribute r k a =
+  let attr_arg r k a =
     let a = k a in
     match Ast.unwrap a with
-      Ast.MetaAttribute(name,constraints,_,_) ->
+      Ast.MetaAttr(name,constraints,_,_) ->
 	let (unitary,inherited) = classify name in
-	Ast.rewrap a (Ast.MetaAttribute(name,constraints,unitary,inherited))
+	Ast.rewrap a (Ast.MetaAttr(name,constraints,unitary,inherited))
     | _ -> a in
 
   let assignop r k ft =
@@ -817,6 +820,14 @@ let classify_variables metavar_decls minirules used_after =
 	Ast.rewrap ft (Ast.MetaBinary(name,constraints,unitary,inherited))
     | _ -> ft in
 
+  let pragmainfo r k ft =
+    let ft = k ft in
+    match Ast.unwrap ft with
+      Ast.MetaPragmaInfo(name,constraints,_,_) ->
+	let (unitary,inherited) = classify name in
+	Ast.rewrap ft (Ast.MetaPragmaInfo(name,constraints,unitary,inherited))
+    | _ -> ft in
+
   let typeC r k e =
     let e = k e in
     match Ast.unwrap e with
@@ -920,14 +931,13 @@ let classify_variables metavar_decls minirules used_after =
 	Ast.rewrap e (Ast.MetaStmtList(name,lenname,cstr,unitary,inherited))
     | _ -> e in
 
-  let fn = V.rebuilder
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      mcode mcode
-      donothing donothing donothing donothing donothing donothing donothing
-      ident expression string_fragment string_format assignop binaryop
-      donothing typeC
-      init param define_param decl donothing field donothing donothing
-      rule_elem donothing donothing attribute donothing donothing in
+  let fn =
+    V.rebuilder {V.rmcode=mcode} {V.rdonothing=donothing}
+      ~ident:ident ~expr:expression ~string_fragment:string_fragment ~fmt:string_format
+      ~assignOp:assignop ~binaryOp:binaryop
+      ~pragma_info:pragmainfo ~ty:typeC
+      ~init:init ~param:param ~define_param:define_param ~decl:decl ~field:field
+      ~rule:rule_elem ~attr_arg:attr_arg donothing in
 
   List.map fn.V.rebuilder_top_level minirules
 
@@ -995,11 +1005,10 @@ let astfvs metavars bound =
     | Ast.Ty _ -> "Ty"
     | Ast.TopId _ -> "TopId"
     | Ast.TopInit _ -> "TopInit"
-    | Ast.Include _ -> "Include"
-    | Ast.MetaInclude _ -> "MetaInclude"
+    | Ast.CppTop _ -> "CppTop"
     | Ast.Undef _ -> "Undef"
     | Ast.DefineHeader _ -> "DefineHeader"
-    | Ast.Pragma _ -> "Pragma"
+    | Ast.TemplateDefinitionHeader(_,_,_,_) -> "TemplateDefinitionHeader"
     | Ast.Case _ -> "Case"
     | Ast.Default _ -> "Default"
     | Ast.AsRe _ -> "AsRe"
@@ -1106,14 +1115,9 @@ let astfvs metavars bound =
   let mcode x = x in
   let donothing r k e = k e in
 
-  V.rebuilder
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode
-    donothing donothing astfvstatement_dots donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing
-    astfvrule_elem astfvstatement astfvcase_line donothing astfvtoplevel
+  V.rebuilder {V.rmcode=mcode} {V.rdonothing=donothing}
+    ~dotsstmt:astfvstatement_dots
+    ~stmt:astfvstatement ~rule:astfvrule_elem ~case:astfvcase_line ~top:astfvtoplevel
     donothing
 
 (*
@@ -1192,14 +1196,7 @@ let get_neg_pos_list (_,rule) used_after_list =
 	  | _ -> (pos_vars,neg_vars,all_vars)))
       option_default (Ast.get_pos_var mc) in
   let v =
-    V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing in
+    V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing} donothing in
   match rule with
     Ast.CocciRule(rule_name,_,minirules,_,_) ->
       List.map
