@@ -50,7 +50,7 @@ let collect_function (stm : Ast0.statement) =
 	     (Ast0.Decl((new_bef_info,Ast0.context_befaft()),
 			Ast0.copywrap stm
 			  (Ast0.FunProto
-                             (fninfo,[],name,lp,params,None,rp,make_semi aft_info))))))
+                             (fninfo,name,lp,params,None,rp,make_semi aft_info))))))
 	(get_name name)
   | _ -> []
 
@@ -61,14 +61,7 @@ let drop_positions =
   let mcode (term,arity,info,mc,_,adj) =
     (term,arity,info,mc,ref [],adj) in
   let donothing r k e = k e in
-  let res =
-    V0.flat_rebuilder
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing in
+  let res = V0.rebuilder {V0.rmcode=mcode} {V0.rdonothing=donothing} () in
   res.VT0.rebuilder_rec_statement
 
 let get_all_functions rule =
@@ -133,7 +126,7 @@ and strip =
       Ast0.mcodekind = ref  (Ast0.PLUS Ast.ONE)} in
   let mcode (mc,_,_,_,_,_) =
     (mc,Ast0.NONE,Ast0.default_info(),Ast0.PLUS Ast.ONE,
-     ref [],-1) in
+     ref [],{Ast.counter = -1; Ast.ender = false}) in
 
   (* need a case for everything that has an unvisited component and can be in
      a function prototype. *)
@@ -175,14 +168,8 @@ and strip =
 	     Ast0.MetaAttr(nm,cstr,Ast0.Pure)
 	 | e -> e)) in
 
-  V0.flat_rebuilder
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing
-    ident donothing donothing donothing typeC donothing param
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing attr_arg donothing
+  V0.rebuilder {V0.rmcode=mcode} {V0.rdonothing=donothing}
+    ~ident:ident ~ty:typeC ~param:param ~attr_arg:attr_arg ()
 
 and changed_proto = function
     (mname,mdef,mproto,None) -> true
@@ -200,13 +187,7 @@ let collect_ident_strings id =
   let mcode (_,_,info,_,_,_) =
     info.Ast0.strings_before @ info.Ast0.strings_after in
   let v =
-    V0.flat_combiner bind option_default
-      mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
-      mcode mcode
-      donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing donothing donothing donothing in
+    V0.combiner bind option_default {V0.cmcode=mcode} {V0.cdonothing=donothing} () in
       v.VT0.combiner_rec_ident id
 
 let right_attach_mcode strings (x,ar,info,mc,pos,adj) =
@@ -224,10 +205,42 @@ let rec right_attach_ident strings id =
     | Ast0.AsIdent(id,asid) -> Ast0.AsIdent(right_attach_ident strings id,asid)
     | _ -> failwith "disj, opt, and funcs not supported")
 
+let right_attach_attr_args strings attr_args =
+  Ast0.rewrap attr_args
+    (match Ast0.unwrap attr_args with
+      Ast0.MacroAttr(arg) ->
+	Ast0.MacroAttr(right_attach_mcode strings arg)
+    | Ast0.MetaAttr(name,x,y) ->
+	Ast0.MetaAttr(right_attach_mcode strings name,x,y)
+    | Ast0.MacroAttrArgs(attr,lp,args,rp) ->
+	Ast0.MacroAttrArgs(attr,lp,args,right_attach_mcode strings rp))
+
+let right_attach_attr strings attr =
+  Ast0.rewrap attr
+    (match Ast0.unwrap attr with
+      Ast0.Attribute(arg) -> Ast0.Attribute(right_attach_attr_args strings arg)
+    | Ast0.GccAttribute(attr,lp1,lp2,args,rp2,rp1) ->
+	Ast0.GccAttribute(attr,lp1,lp2,args,rp2,right_attach_mcode strings rp1)
+    | Ast0.CxxAttribute(lb1,args,rb1,rb2) ->
+	Ast0.CxxAttribute(lb1,args,rb1,right_attach_mcode strings rb2)
+    | Ast0.CxxAttributeUsing(lb1,usng,atnm,dotdot,args,rb1,rb2) ->
+	Ast0.CxxAttributeUsing(lb1,usng,atnm,dotdot,args,rb1,right_attach_mcode strings rb2))
+
 let rec attach_right strings ty =
   Ast0.rewrap ty
     (match Ast0.unwrap ty with
-      Ast0.ConstVol(cv,ty) -> Ast0.ConstVol(cv,attach_right strings ty)
+      Ast0.ConstVol(cvbefore,ty,[]) ->
+	Ast0.ConstVol(cvbefore,attach_right strings ty,[])
+    | Ast0.ConstVol(cvbefore,ty,cvafter) ->
+	(match List.rev cvafter with
+	  cvattr::rest ->
+	    let cvattr =
+	      match cvattr with
+		Ast0.CV cv -> Ast0.CV (right_attach_mcode strings cv)
+	      | Ast0.Attr attr -> Ast0.Attr (right_attach_attr strings attr) in
+	    Ast0.ConstVol(cvbefore,attach_right strings ty,
+			  List.rev(cvattr::rest))
+	| _ -> failwith "not possible")
     | Ast0.BaseType(bt,sl) ->
 	let slhd = right_attach_mcode strings (List.hd(List.rev sl)) in
 	Ast0.BaseType(bt,List.rev (slhd :: (List.tl (List.rev sl))))
@@ -243,12 +256,12 @@ let rec attach_right strings ty =
 	Ast0.Array(ty,lb,e,right_attach_mcode strings rb)
     | Ast0.Decimal(dec,lp,e1,comma,e2,rp) ->
 	Ast0.Decimal(dec,lp,e1,comma,e2,right_attach_mcode strings rp)
-    | Ast0.EnumName(enum,None) ->
-	Ast0.EnumName(right_attach_mcode strings enum, None)
-    | Ast0.EnumName(enum,Some id) ->
-	Ast0.EnumName(enum,Some (right_attach_ident strings id))
-    | Ast0.EnumDef(ty,lb,es,rb) ->
-	Ast0.EnumDef(ty,lb,es,right_attach_mcode strings rb)
+    | Ast0.EnumName(enum,key,None) ->
+	Ast0.EnumName(right_attach_mcode strings enum, key, None)
+    | Ast0.EnumName(enum,key,Some id) ->
+	Ast0.EnumName(enum, key, Some (right_attach_ident strings id))
+    | Ast0.EnumDef(ty,base,lb,es,rb) ->
+	Ast0.EnumDef(ty,base,lb,es,right_attach_mcode strings rb)
     | Ast0.StructUnionName(su,None) ->
 	Ast0.StructUnionName(right_attach_mcode strings su, None)
     | Ast0.StructUnionName(su,Some id) ->
@@ -269,10 +282,10 @@ let rec attach_right strings ty =
 let rec drop_param_name p =
   Ast0.rewrap p
     (match Ast0.unwrap p with
-      Ast0.Param(p,midattr,Some id,attr) ->
+      Ast0.Param(p,Some id,attr) ->
 	let strings = collect_ident_strings id in
 	let p = attach_right strings p in
-        Ast0.Param(p,[],None,midattr@attr)
+        Ast0.Param(p,None,attr)
     | Ast0.OptParam(p) -> Ast0.OptParam(drop_param_name p)
     | p -> p)
 
@@ -281,7 +294,7 @@ let drop_names dec =
   match Ast0.unwrap dec with
     Ast0.Decl(info,proto) ->
       (match Ast0.unwrap proto with
-	Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem) ->
+	Ast0.FunProto(fninfo,name,lp,params,va,rp,sem) ->
 	  let params =
 	    Ast0.rewrap params
 	      (List.map drop_param_name (Ast0.unwrap params)) in
@@ -289,7 +302,7 @@ let drop_names dec =
 	    (Ast0.Decl
 	       (info,
 		Ast0.rewrap proto
-		  (Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem))))
+		  (Ast0.FunProto(fninfo,name,lp,params,va,rp,sem))))
       |	_ -> failwith "unexpected declaration")
   | _ -> failwith "unexpected term"
 
@@ -305,7 +318,7 @@ let new_iname name index =
 
 let rec rename_param old_name all param index =
   match Ast0.unwrap param with
-    Ast0.Param(ty,midattr,Some id,attr) when all ->
+    Ast0.Param(ty,Some id,attr) when all ->
       (match Ast0.unwrap id with
 	Ast0.MetaId
 	  (((_,name),arity,info,mcodekind,pos,adj),constraints,seed,pure) ->
@@ -316,7 +329,7 @@ let rec rename_param old_name all param index =
 		 ((nm,arity,info,mcodekind,pos,adj),constraints,seed,
 		  Ast0.Pure)) in
 	  ([Ast.MetaIdDecl(Ast.NONE,nm)],
-	   Ast0.rewrap param (Ast0.Param(ty,midattr,Some new_id,attr)))
+	   Ast0.rewrap param (Ast0.Param(ty,Some new_id,attr)))
       |	_ -> ([],param))
   | Ast0.Pdots(d) ->
       let nm = (old_name,new_iname "__P" index) in
@@ -354,7 +367,7 @@ let fresh_names old_name mdef dec =
   match Ast0.unwrap dec with
     Ast0.Decl(info,proto) ->
       (match Ast0.unwrap proto with
-	Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem) ->
+	Ast0.FunProto(fninfo,name,lp,params,va,rp,sem) ->
 	  let (metavars,newdec) =
 	    let (metavars,l) =
 	      let params = Ast0.unwrap params in
@@ -367,7 +380,7 @@ let fresh_names old_name mdef dec =
 		  (info,
 		   Ast0.rewrap proto
 		     (Ast0.FunProto
-			(fninfo,attr,name,lp,Ast0.rewrap params l,va,rp,sem))))) in
+			(fninfo,name,lp,Ast0.rewrap params l,va,rp,sem))))) in
 	  let (def_metavars,newdef) =
 	    match Ast0.unwrap mdef with
 	      Ast0.FunDecl(x,fninfo,name,lp,params,va,rp,attrs,lb,body,rb,y) ->
@@ -390,7 +403,7 @@ let no_names dec =
   match Ast0.unwrap dec with
     Ast0.Decl(info,proto) ->
       (match Ast0.unwrap proto with
-	Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem) ->
+	Ast0.FunProto(fninfo,name,lp,params,va,rp,sem) ->
 	  let sem =
 	    (* convert semicolon to minus, since we are dropping the whole
 	       thing *)
@@ -402,14 +415,15 @@ let no_names dec =
 	       (info,
 		Ast0.rewrap proto
 		  (Ast0.FunProto
-		     (fninfo,attr,name,lp,
+		     (fninfo,name,lp,
 		      Ast0.rewrap params
 			(let info = Ast0.get_info params in
 			let mcodekind =
 			  (* use the mcodekind of an atomic minused thing *)
 			  Ast0.get_mcode_mcodekind lp in
 			let pdots =
-			  ("...",Ast0.NONE,info,mcodekind,ref [],-1) in
+			  ("...",Ast0.NONE,info,mcodekind,ref [],
+			   {Ast.counter = -1; Ast.ender = false}) in
 			[Ast0.rewrap params (Ast0.Pdots(pdots))]),
 		      va,rp,sem))))
       |	_ -> dec)
