@@ -51,8 +51,24 @@ let get_minus_constants bind orbind =
 	     (List.map (function id -> ["."^id;"->"^id])
 		(r.V.combiner_ident fld)))
 	  (r.V.combiner_expression exp)
+    | Ast.QualifiedAccess(Some ty,_,fld) ->
+    bind
+	  (Common.union_all
+         (List.map (function id -> ["::"^id])
+		(r.V.combiner_ident fld)))
+	  (r.V.combiner_fullType ty)
+    | Ast.QualifiedACcess(None,_,fld) ->
+	  Common.union_all
+         (List.map (function id -> ["::"^id])
+	     (r.V.combiner_ident fld))
     | Ast.SizeOfExpr(sizeof,_) | Ast.SizeOfType(sizeof,_,_,_) ->
 	bind (k e) [Ast.unwrap_mcode sizeof]
+    | Ast.Delete(delete,_) | Ast.DeleteArr(delete,_,_,_) ->
+	bind (k e) [Ast.unwrap_mcode delete]
+    | Ast.NewPar(nw,_,_,_,_,_,_) ->
+	bind (k e) [Ast.unwrap_mcode new]
+    | Ast.NewNoPar(nw,_,_,_,_,_,_) ->
+	bind (k e) [Ast.unwrap_mcode new]
     | Ast.DisjExpr(exps) ->
 	disj_union_all (List.map r.V.combiner_expression exps)
     | Ast.Edots(_,_) | Ast.Ecircles(_,_) | Ast.Estars(_,_) -> []
@@ -181,7 +197,7 @@ let check_inherited nm =
     | _ -> k i in
 
   let rec type_collect res = function
-      TC.ConstVol(_,ty) | TC.Pointer(ty)
+      TC.ConstVol(_,ty,_) | TC.Pointer(ty)
     | TC.Array(ty) -> type_collect res ty
     | TC.MetaType(tyname,_,_) ->
 	inherited tyname
