@@ -369,7 +369,7 @@ let do_get_constants constants keywords env (neg_pos,_) =
 
   let rec cstr r k c =
     match c with
-      Ast.CstrFalse -> False
+      Ast.CstrFalse | Ast.CstrExit -> False
     | Ast.CstrTrue -> True
     | Ast.CstrAnd list ->
 	List.fold_left (fun accu c -> build_and accu (cstr r k c)) True list
@@ -409,7 +409,9 @@ let do_get_constants constants keywords env (neg_pos,_) =
         Ast.Id name -> add (constants (Ast.unwrap_mcode name))
       | Ast.MetaId(name,_,_,_) -> add (minherited name)
       | _ -> Common.id in
-    let enumOrStructUnionName _ ident res =
+    let structUnionName _ ident res =
+      Common.default res (fun ident' -> add_ident ident' res) ident in
+    let enumName _ _ ident res =
       Common.default res (fun ident' -> add_ident ident' res) ident in
     let pieces ty res =
       Ast.fullType_fold
@@ -419,8 +421,8 @@ let do_get_constants constants keywords env (neg_pos,_) =
 	  Some (fun tyname _ _ _ -> add (inherited (Ast.unwrap_mcode tyname)));
 	  typeName =
 	  Some(fun tyname -> add (constants (Ast.unwrap_mcode tyname)));
-	  enumName = Some enumOrStructUnionName;
-	  structUnionName = Some enumOrStructUnionName
+	  enumName = Some enumName;
+	  structUnionName = Some structUnionName
 	} ty res in
     let rec loop ty =
       match Ast.unwrap ty with
@@ -489,6 +491,9 @@ let do_get_constants constants keywords env (neg_pos,_) =
     | Ast.MetaExprList(name,_,_,_,_) -> minherited name
     | Ast.SizeOfExpr(sizeof,exp) -> bind (keywords "sizeof") (k e)
     | Ast.SizeOfType(sizeof,lp,ty,rp) -> bind (keywords "sizeof") (k e)
+    | Ast.Delete(delete,exp) -> bind (keywords "delete") (k e)
+    | Ast.DeleteArr(delete,lb,rb,exp) -> bind (keywords "delete") (k e)
+    | Ast.New(nw,pp_opt,lp_opt,ty,rp_opt,args_opt) -> bind (keywords "new") (k e)
     | Ast.NestExpr(starter,expr_dots,ender,wc,false) -> option_default
     | Ast.NestExpr(starter,expr_dots,ender,wc,true) ->
 	r.V.combiner_expression_dots expr_dots
@@ -511,6 +516,21 @@ let do_get_constants constants keywords env (neg_pos,_) =
       Ast.MetaFormat(name,_,_,_) -> bind (k ft) (minherited name)
     | _ -> k ft in
 
+  let binary r k ft =
+    match Ast.unwrap ft with
+      Ast.MetaBinary(name,_,_,_) -> bind (k ft) (minherited name)
+    | _ -> k ft in
+
+  let assign r k ft =
+    match Ast.unwrap ft with
+      Ast.MetaAssign(name,_,_,_) -> bind (k ft) (minherited name)
+    | _ -> k ft in
+
+  let pragma_info r k ft =
+    match Ast.unwrap ft with
+      Ast.MetaPragmaInfo(name,_,_,_) -> bind (k ft) (minherited name)
+    | _ -> k ft in
+
   let fullType r k ft =
     match Ast.unwrap ft with
       Ast.DisjType(decls) ->
@@ -539,7 +559,9 @@ let do_get_constants constants keywords env (neg_pos,_) =
     match Ast.unwrap a with
       Ast.Attribute(attr) -> k a
     | Ast.GccAttribute(attr_,_,_,arg,_,_) ->
-        bind (keywords (Ast.unwrap_mcode attr_)) (k a) in
+        bind (keywords (Ast.unwrap_mcode attr_)) (k a)
+    | Ast.CxxAttribute(_,arg,_,_) -> k a
+    | Ast.CxxAttributeUsing(_,_,_,_,arg,_,_) -> k a in
 
   let declaration r k d =
     match Ast.unwrap d with
@@ -549,9 +571,9 @@ let do_get_constants constants keywords env (neg_pos,_) =
 	disj_union_all (List.map r.V.combiner_declaration decls)
     | Ast.OptDecl(decl) -> option_default
     (* need things with explicit names too *)
-    | Ast.Init(_,_,midattr,_,endattr,_,_,_) | Ast.UnInit(_,_,midattr,_,endattr,_) ->
+    | Ast.Init(_,_,_,_,endattr,_,_,_) | Ast.UnInit(_,_,_,_,endattr,_) ->
 	List.fold_left bind (k d)
-	  (List.map r.V.combiner_attribute (midattr@endattr))
+	  (List.map r.V.combiner_attribute endattr)
     | _ -> k d in
 
   let field r k d =
@@ -601,7 +623,7 @@ let do_get_constants constants keywords env (neg_pos,_) =
 	bind (keywords "while") (k re)
     | Ast.WhileTail(whl,lp,exp,rp,sem) ->
 	bind (keywords "do") (k re)
-    | Ast.ForHeader(fr,lp,first,e2,sem2,e3,rp) ->
+    | Ast.ForHeader(fr,lp,first,rp) ->
 	bind (keywords "for") (k re)
     | Ast.SwitchHeader(switch,lp,exp,rp) ->
 	bind (keywords "switch") (k re)
@@ -613,8 +635,14 @@ let do_get_constants constants keywords env (neg_pos,_) =
 	bind (keywords "goto") (k re)
     | Ast.Default(def,colon) ->
 	bind (keywords "default") (k re)
-    | Ast.Include(inc,s) ->
-	bind (k re)
+    | Ast.DisjRuleElem(res) ->
+	disj_union_all (List.map r.V.combiner_rule_elem res)
+    | _ -> k re) in
+
+  let directive r k di =
+    match Ast.unwrap di with
+      Ast.Include(inc,s) ->
+	bind (k di)
 	  (match Ast.unwrap_mcode s with
 	    Ast.AnyInc -> True
 	  | Ast.Local l | Ast.NonLocal l ->
@@ -630,11 +658,7 @@ let do_get_constants constants keywords env (neg_pos,_) =
 	      (match strings with
 		[] -> True
 	      | x::xs -> List.fold_left bind x xs))
-    | Ast.Pragma(prg,id,body) ->
-	bind (keywords "pragma") (k re)
-    | Ast.DisjRuleElem(res) ->
-	disj_union_all (List.map r.V.combiner_rule_elem res)
-    | _ -> k re) in
+    | _ -> k di in
 
   let statement r k s =
     match Ast.unwrap s with
@@ -647,14 +671,13 @@ let do_get_constants constants keywords env (neg_pos,_) =
     | Ast.Dots(d,whn,_,_) -> option_default
     | _ -> k s in
 
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    ident expression string_fragment string_format donothing donothing
-    fullType typeC initialiser parameter define_parameter declaration donothing
-    field ann_field donothing rule_elem statement donothing attribute attr_arg
-    donothing donothing
+  V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing}
+    ~ident:ident ~expr:expression ~string_fragment:string_fragment
+    ~fmt:string_format ~assignOp:assign ~binaryOp:binary ~pragma_info:pragma_info
+    ~ft:fullType ~ty:typeC ~init:initialiser ~param:parameter
+    ~define_param:define_parameter ~decl:declaration ~field:field
+    ~annotated_field:ann_field ~rule:rule_elem ~stmt:statement ~attribute:attribute
+    ~attr_arg:attr_arg ~directive:directive donothing
 
 (* ------------------------------------------------------------------------ *)
 
@@ -704,7 +727,8 @@ let all_context =
       Ast.FunHeader(bef,_,_,_,_,_,_,_,_) -> bind (process_mcodekind bef) (k e)
     | Ast.Decl decl ->
 	bind (process_mcodekind (annotated_decl decl)) (k e)
-    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
+    | Ast.ForHeader(fr,lp,Ast.ForDecl(decl,_,_,_),rp)
+    | Ast.ForHeader(fr,lp,Ast.ForRange(decl,_),rp) ->
 	bind (process_mcodekind (annotated_decl decl)) (k e)
     | _ -> k e in
 
@@ -716,15 +740,8 @@ let all_context =
 	bind (k e) (end_info ei)
     | _ -> k e in
 
-  V.combiner bind option_default
-    mcode mcode mcode mcode mcode mcode mcode mcode mcode
-    mcode mcode mcode mcode mcode
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing donothing donothing donothing
-    donothing
-    initialiser donothing donothing donothing donothing donothing donothing
-    donothing rule_elem statement donothing donothing donothing donothing
-    donothing
+  V.combiner bind option_default {V.cmcode=mcode} {V.cdonothing=donothing}
+    ~init:initialiser ~rule:rule_elem ~stmt:statement donothing
 
 (* ------------------------------------------------------------------------ *)
 
@@ -867,5 +884,7 @@ let get_constants rules neg_pos_vars virt =
     | Flag.IdUtils ->
 	(grep,None,coccigrep,interpret_idutils res)
     | Flag.CocciGrep | Flag.GitGrep -> (grep,None,coccigrep,None)
+    | Flag.PatchDiff | Flag.PatchDiffRange _ ->
+	(None, None, None, None)
     end
   else (None,None,None,None)
