@@ -9,7 +9,7 @@ exception Failed
 let interpret dir query suffixes =
   let collect query =
     let cmd =
-      Printf.sprintf "cd %s; git grep -l -w %s -- %s" dir query suffixes in
+      Printf.sprintf "cd %s > /dev/null; git grep -l -w %s -- %s" dir query suffixes in
     let (res,code) = Common.cmd_to_list_and_status cmd in
     if code = Unix.WEXITED 0 || code = Unix.WEXITED 1
     then res
