@@ -1,7 +1,7 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
 
-val interpret : string -> Get_constants2.combine -> string list
+val interpret : string -> Get_constants2.combine option -> string list option
