@@ -5,7 +5,7 @@
  *)
 
 (* create an index for each constructor *)
-(* current max is 202, also unused: {9,15,161}
+(* current max is 203, also unused: {8,9,161,163,164}
 *)
 
 (* doesn't really work - requires that identical terms with no token
@@ -70,6 +70,7 @@ let expression e =
   | Ast0.DeleteArr(dlt,lb,rb,exp) -> [86]
   | Ast0.New(nw,pp_opt,lp_opt,ty,rp_opt,args) -> [57]
   | Ast0.TemplateInst(name,lab,args,rab) -> [202]
+  | Ast0.TupleExpr(args) -> [15]
   | Ast0.TypeExp(ty) -> [123] (* added after *)
   | Ast0.Constructor(lp,ty,rp,init) -> [155]
   | Ast0.MetaErr(name,_,_) -> [32]
@@ -143,6 +144,7 @@ let field d =
   | Ast0.CppField(_) -> [202]
   | Ast0.DisjField(_,decls,_,_) -> [189] (* added after *)
   | Ast0.ConjField(_,decls,_,_) -> [190] (* added after *)
+  | Ast0.AccSpec(_) -> [203]
   | Ast0.Fdots(dots,whencode) -> [133]
   | Ast0.OptField(decl) -> [191]
 
@@ -226,12 +228,6 @@ let forinfo fi =
   | Ast0.ForDecl (bef,decl,exp2,sem2,exp3) -> [159]
   | Ast0.ForRange (bef,decl,ini) -> [65]
 
-and pragmainfo pi =
-  match Ast0.unwrap pi with
-    Ast0.PragmaString(s) -> [163]
-  | Ast0.PragmaDots (dots) -> [164]
-  | Ast0.MetaPragmaInfo(_,_,_) -> [8]
-
 let case_line c =
   match Ast0.unwrap c with
     Ast0.Default(def,colon,code) -> [126]
