@@ -42,10 +42,6 @@ let make_info line logical_line logical_line_end offset col strbef straft
     Ast0.strings_before = strbef; Ast0.strings_after = straft;
     Ast0.isSymbolIdent = isSymbol; }
 
-let clt2info (_,line,logical_line,logical_line_end,offset,col,
-              strbef,straft,pos,ws) =
- make_info line logical_line logical_line_end offset col strbef straft false ws
-
 let drop_bef (arity,line,lline,llineend,offset,col,strbef,straft,pos,ws) =
   (arity,line,lline,llineend,offset,col,[],straft,pos,ws)
 
@@ -103,42 +99,21 @@ let id2mcode  (name, clt) = clt2mcode name clt
 let tok2mcode (name, clt) = clt2mcode name clt
 let sym2mcode (name, clt) = clt2mcode_ext name true clt
 
-let mkdots str (dot,whencode) =
-  match str with
-    "..." -> Ast0.wrap(Ast0.Dots(clt2mcode str dot, whencode))
-  | _ -> failwith "cannot happen"
-
 let mkedots str (dot,whencode) =
   match str with
     "..." -> Ast0.wrap(Ast0.Edots(clt2mcode str dot, whencode))
   | _ -> failwith "cannot happen"
 
-let mkdpdots str dot =
-  match str with
-    "..." -> Ast0.wrap(Ast0.DPdots(clt2mcode str dot))
-  | _ -> failwith "cannot happen"
-
 let mkidots str (dot,whencode) =
   match str with
     "..." -> Ast0.wrap(Ast0.Idots(clt2mcode str dot, whencode))
   | _ -> failwith "cannot happen"
 
-let mkfdots str (dot,whencode) =
-  match (str,whencode) with
-    ("...",None) -> Ast0.wrap(Ast0.Fdots(clt2mcode str dot, None))
-  | ("...",Some [w]) -> Ast0.wrap(Ast0.Fdots(clt2mcode str dot, Some w))
-  | _ -> failwith "cannot happen"
-
 let mkfdots_one str (dot,whencode) =
   match str with
     "..." -> Ast0.wrap(Ast0.Fdots(clt2mcode str dot, whencode))
   | _ -> failwith "cannot happen"
 
-let mkpdots str dot =
-  match str with
-    "..." -> Ast0.wrap(Ast0.Pdots(clt2mcode str dot))
-  | _ -> failwith "cannot happen"
-
 let mkenumdots str (dot,whencode) =
   match str with
     "..." -> Ast0.wrap(Ast0.EnumDots(clt2mcode str dot, whencode))
@@ -171,8 +146,6 @@ let make_cxx_attr_using lb1 usng atnm dotdot arg rb1 rb2 =
        (tok2mcode lb1,clt2mcode "using" usng, atnm,
 	clt2mcode ":" dotdot, arg, tok2mcode rb1, tok2mcode rb2))
 
-let top_dots l = Ast0.wrap l
-
 (* here the offset is that of the first in the sequence of *s, not that of
 each * individually *)
 let pointerify ty m =
@@ -293,6 +266,10 @@ let check_meta_tyopt type_irrelevant v =
       (match meta_lookup rule name v with
 	Ast.MetaAssignmentOperatorDecl(_,_) -> ()
       | _ -> fail name)
+  | Ast.MetaPragmaInfoDecl(Ast.NONE,(rule,name)) ->
+      (match meta_lookup rule name v with
+	Ast.MetaPragmaInfoDecl(_,_) -> ()
+      | _ -> fail name)
   | Ast.MetaConstDecl(Ast.NONE,(rule,name),ty) ->
       (match meta_lookup rule name v with
 	Ast.MetaConstDecl(_,_,ty1) when type_equal ty ty1 -> ()
@@ -458,19 +435,6 @@ let create_metadec_with_constraints ar ispure kindfn ids current_rule =
 	   kindfn ar rule ispure checker constraints)
        ids)
 
-let create_metadec_ty ar ispure kindfn ids current_rule =
-  List.concat
-    (List.map
-       (function ((rule,nm),constraints) ->
-	 let (rule,checker) =
-	   match rule with
-	     None -> ((current_rule,nm),function x -> [Common.Left x])
-	   | Some rule ->
-	       ((rule,nm),
-		function x -> check_meta x; [Common.Right x]) in
-	 kindfn ar rule ispure checker constraints)
-       ids)
-
 let create_len_metadec ar ispure kindfn lenid ids current_rule =
   let (lendec,lenname) =
     match lenid with
@@ -568,7 +532,12 @@ let forloop3 fr lp decl i2 rp s =
 
 let whileloop w lp e rp s =
   Ast0.wrap(Ast0.While(clt2mcode "while" w,clt2mcode "(" lp,
-		       e,clt2mcode ")" rp,s,make_fake_mcode()))
+		       Ast0.WhileExp e,clt2mcode ")" rp,s,make_fake_mcode()))
+
+let whileloop2 w lp d rp s =
+  let bef = (Ast0.default_info(),Ast0.context_befaft()) in
+  Ast0.wrap(Ast0.While(clt2mcode "while" w,clt2mcode "(" lp,
+		       Ast0.WhileDecl(bef,d),clt2mcode ")" rp,s,make_fake_mcode()))
 
 let doloop d s w lp e rp pv =
   Ast0.wrap(Ast0.Do(clt2mcode "do" d,s,clt2mcode "while" w,
