@@ -281,6 +281,8 @@ let rec expression e =
       mcode print_string_box lp;
       dots (function _ -> ()) expression args;
       close_box(); mcode print_string rp
+  | Ast.TupleExpr(init) ->
+      initialiser init
   | Ast.TypeExp(ty) -> fullType ty
   | Ast.Constructor(lp,ty,rp,init) ->
       mcode print_string_box lp; fullType ty; close_box();
@@ -632,7 +634,7 @@ and declaration d =
       print_named_type ty (fun _ -> ident id);
       print_attribute_list endattr;
       print_string " "; mcode print_string eq;
-      print_string " "; initialiser ini; mcode print_string sem
+      print_string " "; initialiser ini; print_option (mcode print_string) sem
   | Ast.UnInit(al,stg,ty,id,endattr,sem) ->
       print_option alignas al;
       print_option (mcode storage) stg;
@@ -713,6 +715,7 @@ and field d =
       print_attribute_list attr;
       mcode print_string sem
   | Ast.CppField(di) -> directive di
+  | Ast.AccSpec(decl,dd) -> mcode print_string decl; mcode print_string dd
 
 and annotated_field arity d =
   match Ast.unwrap d with
@@ -865,10 +868,10 @@ and rule_elem arity re =
       expression exp; close_box(); mcode print_string rp; print_string " "
   | Ast.Else(els) ->
       print_string arity; mcode print_string els; print_string " "
-  | Ast.WhileHeader(whl,lp,exp,rp) ->
+  | Ast.WhileHeader(whl,lp,cond,rp) ->
       print_string arity;
       mcode print_string whl; print_string " "; mcode print_string_box lp;
-      expression exp; close_box(); mcode print_string rp; print_string " "
+      whileinfo cond; close_box(); mcode print_string rp; print_string " "
   | Ast.DoHeader(d) ->
       print_string arity; mcode print_string d; print_string " "
   | Ast.WhileTail(whl,lp,exp,rp,sem) ->
@@ -1056,6 +1059,11 @@ and statement arity s =
       close_box(); force_newline()
   | Ast.OptStm(s) -> statement "?" s
 
+and whileinfo cond =
+  match cond with
+    Ast.WhileExp(e) -> expression e
+  | Ast.WhileDecl(d) -> annotated_decl "" d
+
 and directive di =
   match Ast.unwrap di with
     Ast.Include(inc,s) ->
