@@ -38,6 +38,19 @@ let size_type = Ast0.BaseType (Ast.SizeType, [])
 let ssize_type = Ast0.BaseType (Ast.SSizeType, [])
 let ptrdiff_type = Ast0.BaseType (Ast.PtrDiffType, [])
 
+let same_cvs cvattrs1 cvattrs2 =
+  let get_cvs =
+    List.fold_left
+      (fun prev ->
+	function
+	    Ast0.CV cv -> cv :: prev
+	  | Ast0.Attr attr -> prev)
+      [] in
+  let cvattrs1 = get_cvs cvattrs1 in
+  let cvattrs2 = get_cvs cvattrs2 in
+  List.sort compare(List.map Ast0.unwrap_mcode cvattrs1) =
+  List.sort compare(List.map Ast0.unwrap_mcode cvattrs2)
+
 let lub_type t1 t2 =
   match (t1,t2) with
     (None,None) -> None
@@ -48,8 +61,10 @@ let lub_type t1 t2 =
         match Ast0.unwrap ty1, Ast0.unwrap ty2 with
           (Ast0.BaseType (Ast.Unknown, _), ty)
         | (ty, Ast0.BaseType (Ast.Unknown, _)) -> Ast0.rewrap ty1 ty
-	| (Ast0.ConstVol(cv1,ty1),Ast0.ConstVol(cv2,ty2)) when cv1 = cv2 ->
-	    Ast0.rewrap ty1 (Ast0.ConstVol(cv1,loop ty1 ty2))
+	| (Ast0.ConstVol(cvbefore1,ty1,cvafter1),Ast0.ConstVol(cvbefore2,ty2,cvafter2))
+	  when same_cvs (cvbefore1@cvafter1) (cvbefore2@cvafter2) ->
+	    (* not sure what to do with attrs *)
+	    Ast0.rewrap ty1 (Ast0.ConstVol(cvbefore1,loop ty1 ty2,cvafter1))
         (* pad: in pointer arithmetic, as in ptr+1, the lub must be ptr *)
 	| (Ast0.Pointer(ty1, s),Ast0.Pointer(ty2, _)) ->
 	    Ast0.rewrap ty1 (Ast0.Pointer(loop ty1 ty2, s))
@@ -86,7 +101,7 @@ let lub_envs envs =
 let strip_cv =
   Common.map_option (function ty ->
     match Ast0.unwrap ty with
-    | Ast0.ConstVol(_,ty') -> ty'
+    | Ast0.ConstVol(_,ty',_) -> ty'
     | _ -> ty)
 
 (* types that might be integer types.  should char be allowed? *)
@@ -105,6 +120,7 @@ let rec is_int_type_unwrap = function
 and is_int_type ty = is_int_type_unwrap (Ast0.unwrap ty)
 
 let dummy = Ast0.make_mcode ""
+let dummy_star = Ast0.make_mcode "*"
 
 let num s = Ast0.Constant (Ast0.make_mcode (Ast.Int s))
 
@@ -145,7 +161,7 @@ let rec propagate_types env =
                      Ast0.rewrap e (
                        Ast0.Pointer(
                          Ast0.rewrap e char_type,
-                         dummy)))
+                         dummy_star)))
 	       | Ast.Char(_) -> Some (Ast0.rewrap e char_type)
 	       | Ast.Int(_) -> Some (Ast0.rewrap e int_type)
 	       | Ast.Float(_) ->  Some (Ast0.rewrap e float_type)
@@ -188,24 +204,24 @@ let rec propagate_types env =
 	    Ast0.get_type exp
 	| Ast0.Unary(exp,op) ->
 	    (match Ast0.unwrap_mcode op with
-		 Ast.GetRef ->
-                   Some (Ast0.rewrap e (Ast0.Pointer (
-                    (match Ast0.get_type exp with
-                       None -> Ast0.rewrap e unknown_type
-                     | Some t -> t),
-                     dummy)))
-	       | Ast.GetRefLabel ->
-                   Some (Ast0.rewrap e (Ast0.Pointer (
-                     Ast0.rewrap e void_type,
-                     dummy)))
-	       | Ast.DeRef ->
-                   (match Common.map_option Ast0.unwrap (Ast0.get_type exp) with
-                        Some (Ast0.Pointer(t, _)) -> Some t
-		      |	_ -> None)
-	       | Ast.UnPlus -> Ast0.get_type exp
-	       | Ast.UnMinus -> Ast0.get_type exp
-	       | Ast.Tilde -> Ast0.get_type exp
-               | Ast.Not -> Some (Ast0.rewrap e bool_type))
+	      Ast.GetRef ->
+		let exptype =
+		  match Ast0.get_type exp with
+		    None -> Ast0.rewrap e unknown_type
+		  | Some t -> t in
+                Some (Ast0.rewrap e (Ast0.Pointer (exptype,dummy_star)))
+	    | Ast.GetRefLabel ->
+                Some
+		  (Ast0.rewrap e
+		     (Ast0.Pointer (Ast0.rewrap e void_type,dummy_star)))
+	    | Ast.DeRef ->
+                (match Common.map_option Ast0.unwrap (Ast0.get_type exp) with
+                  Some (Ast0.Pointer(t, _)) -> Some t
+		| _ -> None)
+	    | Ast.UnPlus -> Ast0.get_type exp
+	    | Ast.UnMinus -> Ast0.get_type exp
+	    | Ast.Tilde _ -> Ast0.get_type exp
+            | Ast.Not _ -> Some (Ast0.rewrap e bool_type))
 	| Ast0.Nested(exp1,op,exp2) -> failwith "nested in type inf not possible"
 	| Ast0.Binary(exp1,op,exp2) ->
 	    let ty1 = Ast0.get_type exp1 in
@@ -231,12 +247,13 @@ let rec propagate_types env =
 	    (match Ast0.unwrap op with
 	      Ast0.Arith _ -> same_type ty1 ty2
 	    | Ast0.MetaBinary _ -> same_type ty1 ty2
-	    | Ast0.Logical(op') when (let op''=Ast0.unwrap_mcode op' in op''=Ast.AndLog || op''=Ast.OrLog) ->
-		Some (Ast0.wrap bool_type)
 	    | Ast0.Logical(op) ->
-		let ty = lub_type ty1 ty2 in
-		Ast0.set_type exp1 ty; Ast0.set_type exp2 ty;
-		Some (Ast0.wrap bool_type))
+		(match Ast0.unwrap_mcode op with
+		  Ast.AndLog _ | Ast.OrLog _ -> Some (Ast0.wrap bool_type)
+		| _ ->
+		    let ty = lub_type ty1 ty2 in
+		    Ast0.set_type exp1 ty; Ast0.set_type exp2 ty;
+		    Some (Ast0.wrap bool_type)))
 	| Ast0.Paren(lp,exp,rp) -> Ast0.get_type exp
 	| Ast0.ArrayAccess(exp1,lb,exp2,rb) ->
 	    (match strip_cv (Ast0.get_type exp2) with
@@ -282,7 +299,7 @@ let rec propagate_types env =
 		      | Some (Ast0.StructUnionName(_, _)) -> None
 		      | Some x ->
                           let ty =
-                            Ast0.wrap (Ast0.Pointer(t, Ast0.make_mcode "")) in
+                            Ast0.wrap (Ast0.Pointer(t, dummy_star)) in
                           err exp ty
 			    "non-structure pointer type in field ref"
 		      |	_ -> failwith "not possible")
@@ -291,9 +308,16 @@ let rec propagate_types env =
                | Some x ->
                    let ty = Ast0.wrap x in
                    err exp ty "non-structure pointer type in field ref")
-	| Ast0.Cast(lp,ty,attr,rp,exp) -> Some ty
+    | Ast0.QualifiedAccess(Some ty,coloncolon,field) -> Some ty
+    | Ast0.QualifiedAccess(None,coloncolon,field) -> None
+	| Ast0.Cast(lp,ty,rp,exp) -> Some ty
 	| Ast0.SizeOfExpr(szf,exp) -> Some (Ast0.wrap int_type)
 	| Ast0.SizeOfType(szf,lp,ty,rp) -> Some (Ast0.wrap int_type)
+	| Ast0.Delete(dlt,exp) -> Some (Ast0.wrap void_type)
+	| Ast0.DeleteArr(dlr,lb,rb,exp) -> Some (Ast0.wrap void_type)
+	| Ast0.New(nw,pp,lp_opt,ty,rp_opt,args) -> Some ty
+	| Ast0.TemplateInst(name,lab,args,rab) -> None
+	| Ast0.TupleExpr(args) -> None (* TODO: Make a proper return type for tuples *)
 	| Ast0.TypeExp(ty) -> None
 	| Ast0.Constructor(lp,ty,rp,init) -> Some ty
 	| Ast0.MetaErr(name,_,_) -> None
@@ -371,20 +395,34 @@ let rec propagate_types env =
 	    let _ = (propagate_types acc).VT0.combiner_rec_statement s in
 	    process_statement_list r acc ss)
 
+  and get_ty_type ty =
+    match Ast0.unwrap ty with
+      Ast0.TypeOfExpr(typeof,lp,exp,rp)
+      when Ast0.unwrap_mcode typeof = "decltype" ->
+	(match Ast0.unwrap exp with
+	  Ast0.Paren(lpp,exp,rpp) ->
+	    (match Ast0.get_type exp with
+	      Some t -> Ast0.rewrap ty (Ast0.Pointer(t, Ast0.make_mcode "&"))
+	    | None -> ty)
+	| _ -> ty)
+    | _ -> ty
+
   and process_decl env decl =
     match Ast0.unwrap decl with
       Ast0.MetaDecl(_,_,_) -> []
-    | Ast0.Init(_,ty,id,_,_,exp,_) ->
+    | Ast0.Init(_,_,ty,id,_,_,exp,_) ->
 	let _ = (propagate_types env).VT0.combiner_rec_initialiser exp in
-	List.map (function i -> (i,ty)) (strip id)
-    | Ast0.UnInit(_,ty,id,_,_) ->
-	List.map (function i -> (i,ty)) (strip id)
+	let _ = (propagate_types env).VT0.combiner_rec_typeC ty in
+	List.map (function i -> (i,get_ty_type ty)) (strip id)
+    | Ast0.UnInit(_,_,ty,id,_,_) ->
+	let _ = (propagate_types env).VT0.combiner_rec_typeC ty in
+	List.map (function i -> (i,get_ty_type ty)) (strip id)
     | Ast0.FunProto(fi,nm,lp,params,va,rp,sem) -> []
-    | Ast0.MacroDecl(_,_,_,_,_,_,_) -> []
-    | Ast0.MacroDeclInit(_,_,_,_,_,_,exp,_) ->
+    | Ast0.MacroDecl(_,_,_,_,_,_,_,_) -> []
+    | Ast0.MacroDeclInit(_,_,_,_,_,_,_,_,exp,_) ->
         let _ = (propagate_types env).VT0.combiner_rec_initialiser exp in
 	[]
-    | Ast0.TyDecl(ty,_,_) -> []
+    | Ast0.TyDecl(ty,_) -> []
               (* pad: should handle typedef one day and add a binding *)
     | Ast0.Typedef((a,_,_,_,_,_),b,c,(d,_,_,_,_,_)) ->
 	[]
@@ -417,7 +455,7 @@ let rec propagate_types env =
 
   let statement r k s =
     match Ast0.unwrap s with
-      Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
+      Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,_,lbrace,body,rbrace,_) ->
 	let rec get_binding p =
 	  match Ast0.unwrap p with
 	    Ast0.Param(ty,Some id,attr) ->
@@ -428,25 +466,38 @@ let rec propagate_types env =
 	let fenv = List.concat (List.map get_binding (Ast0.unwrap params)) in
 	(propagate_types (fenv@env)).VT0.combiner_rec_statement_dots body
     | Ast0.IfThen(_,_,exp,_,_,_) | Ast0.IfThenElse(_,_,exp,_,_,_,_,_)
-    | Ast0.While(_,_,exp,_,_,_) | Ast0.Do(_,_,_,_,exp,_,_) ->
+    | Ast0.While(_,_,Ast0.WhileExp(exp),_,_,_) | Ast0.Do(_,_,_,_,exp,_,_) ->
 	let _ = k s in
         post_bool exp;
         None
-    | Ast0.For(a,b,first,exp,c,d,e,f,g) ->
+    | Ast0.While(_,_,Ast0.WhileDecl(_,decl),_,s,_) ->
+	let newenv = (process_decl env decl)@env in
+        (propagate_types newenv).VT0.combiner_rec_statement s
+    | Ast0.For(a,b,first,e,f,g) ->
 	(match Ast0.unwrap first with
-	  Ast0.ForExp _ ->
-	    (match exp with
+	  Ast0.ForExp(e1,sem1,e2,sem2,e3) ->
+	    (match e2 with
 	      Some exp ->
 		let _ = k s in
 		post_bool exp;
 		None
 	    | None -> k s)
-	| Ast0.ForDecl (_,decl) ->
+	| Ast0.ForDecl (_,decl,e2,sem2,e3) ->
 	(* not super elegant..., reuses a ; (d) *)
 	    let newenv = (process_decl env decl)@env in
-	    let dummy = Ast0.rewrap first (Ast0.ForExp (None,c)) in
+	    let dummy =
+	      Ast0.rewrap first (Ast0.ForExp (None,sem2,e2,sem2,e3)) in
 	    (propagate_types newenv).VT0.combiner_rec_statement
-	      (Ast0.rewrap s (Ast0.For(a,b,dummy,exp,c,d,e,f,g))))
+	      (Ast0.rewrap s (Ast0.For(a,b,dummy,e,f,g)))
+	| Ast0.ForRange (_,decl,i2) ->
+	    let newenv = (process_decl env decl)@env in
+	    let sem = Ast0_cocci.make_mcode ";" in
+	    let dummy =
+	      Ast0.rewrap first (Ast0.ForExp (None,sem,None,sem,None)) in
+	    bind
+	      ((propagate_types newenv).VT0.combiner_rec_initialiser i2)
+	      ((propagate_types newenv).VT0.combiner_rec_statement
+		 (Ast0.rewrap s (Ast0.For(a,b,dummy,e,f,g)))))
     | Ast0.Switch(_,_,exp,_,_,decls,cases,_) ->
 	let senv = process_statement_list r env (Ast0.unwrap decls) in
 	let res =
@@ -465,17 +516,12 @@ let rec propagate_types env =
 	None
     | _ -> k c in
 
-  V0.combiner bind option_default
-    {V0.combiner_functions with
-      VT0.combiner_dotsstmtfn = statement_dots;
-      VT0.combiner_identfn = ident;
-      VT0.combiner_exprfn = expression;
-      VT0.combiner_stmtfn = statement;
-      VT0.combiner_casefn = case_line}
+  V0.combiner_default bind option_default
+    ~dotsstmt:statement_dots ~ident:ident ~expr:expression ~stmt:statement ~case:case_line ()
 
 let type_infer code =
   let unknown = Ast0.wrap (Ast0.BaseType (Ast.Unknown, [])) in
-  let unknown_ptr = Ast0.wrap (Ast0.Pointer (unknown, dummy)) in
+  let unknown_ptr = Ast0.wrap (Ast0.Pointer (unknown, dummy_star)) in
   let prop = propagate_types [(Id("NULL"), unknown_ptr)] in
   let fn = prop.VT0.combiner_rec_top_level in
   let _ = List.map fn code in
