@@ -251,6 +251,8 @@ let rec expression e =
 	  expression tn; mcode print_string_box lp;
 	  let _ = dots (function _ -> ()) expression args in
 	  close_box(); mcode print_string rp
+      | Ast0.TupleExpr(init) ->
+	  initialiser init
       | Ast0.TypeExp(ty) -> typeC ty
       | Ast0.Constructor(lp,ty,rp,init) ->
 	  mcode print_string_box lp; typeC ty; close_box();
@@ -466,7 +468,7 @@ and declaration d =
           print_attribute_list endattr;
 	  print_string " ";
 	  mcode print_string eq; print_string " "; initialiser ini;
-	  mcode print_string sem
+	  print_option (mcode print_string) sem
       | Ast0.UnInit(al,stg,ty,id,endattr,sem) ->
 	  print_option alignas al;
 	  print_option (mcode U.storage) stg; print_named_type ty id;
@@ -546,6 +548,7 @@ and field d =
       | Ast0.ConjField(_,fields,_,_) ->
 	  do_disj fields field "&"
       | Ast0.OptField(decl) -> print_string "?"; field decl
+      | Ast0.AccSpec(decl,dd) -> mcode print_string decl; mcode print_string dd
       | Ast0.Fdots(dots,Some (_,_,whencode)) ->
 	  mcode print_string dots; print_string "   when != ";
 	  field whencode
@@ -578,9 +581,9 @@ and initialiser i =
   print_context i
     (function _ ->
       match Ast0.unwrap i with
-	Ast0.MetaInit(name,_,_)-> mcode print_meta name; print_string " "
-      |	Ast0.MetaInitList(name,_,_,_)-> mcode print_meta name; print_string " "
-      |	Ast0.InitExpr(exp) -> expression exp
+	Ast0.MetaInit(name,_,_) -> mcode print_meta name; print_string " "
+      | Ast0.MetaInitList(name,_,_,_) -> mcode print_meta name; print_string " "
+      | Ast0.InitExpr(exp) -> expression exp
       | Ast0.InitList(lb,initlist,rb,ordered) ->
           (*doesn't show commas dropped in unordered case*)
 	  mcode print_string lb; open_box 0;
@@ -699,10 +702,10 @@ and statement arity s =
 	  print_string arity; mcode print_string els; print_string " ";
 	  statement arity branch2;
 	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
-      | Ast0.While(whl,lp,exp,rp,body,(info,aft,adj)) ->
+      | Ast0.While(whl,lp,cond,rp,body,(info,aft,adj)) ->
 	  print_string arity;
 	  mcode print_string whl; print_string " "; mcode print_string_box lp;
-	  expression exp; close_box(); mcode print_string rp; print_string " ";
+	  whileinfo cond; close_box(); mcode print_string rp; print_string " ";
 	  statement arity body;
 	  mcode (function _ -> ()) ((),(),info,aft,ref [],adj)
       | Ast0.Do(d,body,whl,lp,exp,rp,sem) ->
@@ -821,6 +824,10 @@ and statement arity s =
       | Ast0.AsStmt(stm,asstm) -> statement arity stm; print_string "@";
 	  statement arity asstm)
 
+and whileinfo = function
+    Ast0.WhileExp(e) -> expression e
+  | Ast0.WhileDecl(_,d) -> declaration d
+
 and print_define_parameters params =
   match Ast0.unwrap params with
     Ast0.NoParams -> ()
@@ -1053,7 +1060,3 @@ let unparse_x_to_string fn x =
   let res = Common.format_to_string (function _ -> fn x) in
   quiet := q;
   res
-
-let show_cocci_parse_tree comment parse_tree =
-  Printf.printf "%s\n" comment;
-  top_level parse_tree; Format.print_newline()
