@@ -22,7 +22,9 @@ type 'n all_functions =
       initialiser : (Ast0.initialiser,'n) inout;
       initialiser_list : (Ast0.initialiser_list,'n) inout;
       parameter : (Ast0.parameterTypeDef,'n) inout;
+      template_parameter : (Ast0.templateParameterTypeDef,'n) inout;
       parameter_list : (Ast0.parameter_list,'n) inout;
+      template_parameter_list : (Ast0.template_parameter_list,'n) inout;
       statement : (Ast0.statement,'n) inout;
       forinfo : (Ast0.forinfo,'n) inout;
       case_line : (Ast0.case_line,'n) inout;
@@ -59,7 +61,9 @@ type 'n combiner_rec_functions =
       combiner_rec_initialiser_list :
 	(Ast0.initialiser_list,'n) combiner_inout;
       combiner_rec_parameter : (Ast0.parameterTypeDef,'n) combiner_inout;
+      combiner_rec_template_parameter : (Ast0.templateParameterTypeDef,'n) combiner_inout;
       combiner_rec_parameter_list : (Ast0.parameter_list,'n) combiner_inout;
+      combiner_rec_template_parameter_list : (Ast0.template_parameter_list,'n) combiner_inout;
       combiner_rec_statement : (Ast0.statement,'n) combiner_inout;
       combiner_rec_forinfo   : (Ast0.forinfo,'n) combiner_inout;
       combiner_rec_case_line : (Ast0.case_line,'n) combiner_inout;
@@ -94,8 +98,8 @@ type 'n combiner_functions =
    combiner_const_mcode : (Ast.constant,'n) cmcode;
    combiner_simpleAssign_mcode : (Ast0.simpleAssignOp,'n) cmcode;
    combiner_opAssign_mcode : (Ast.arithOp,'n) cmcode;
-   combiner_fix_mcode : (Ast.fixOp,'n) cmcode;
-   combiner_unary_mcode : (Ast.unaryOp,'n) cmcode;
+   combiner_fixOp_mcode : (Ast.fixOp,'n) cmcode;
+   combiner_unaryOp_mcode : (Ast.unaryOp,'n) cmcode;
    combiner_arithOp_mcode : (Ast.arithOp,'n) cmcode;
    combiner_logicalOp_mcode : (Ast.logicalOp,'n) cmcode;
    combiner_cv_mcode : (Ast.const_vol,'n) cmcode;
@@ -106,6 +110,7 @@ type 'n combiner_functions =
    combiner_dotsexprfn : (Ast0.expression Ast0.dots,'n) ccode;
    combiner_dotsinitfn : (Ast0.initialiser Ast0.dots,'n) ccode;
    combiner_dotsparamfn : (Ast0.parameterTypeDef Ast0.dots,'n) ccode;
+   combiner_dotstemplateparamfn : (Ast0.templateParameterTypeDef Ast0.dots,'n) ccode;
    combiner_dotsstmtfn : (Ast0.statement Ast0.dots,'n) ccode;
    combiner_dotsdeclfn : (Ast0.declaration Ast0.dots,'n) ccode;
    combiner_dotsfieldfn : (Ast0.field Ast0.dots,'n) ccode;
@@ -119,6 +124,7 @@ type 'n combiner_functions =
    combiner_tyfn : (Ast0.typeC,'n) ccode;
    combiner_initfn : (Ast0.initialiser,'n) ccode;
    combiner_paramfn : (Ast0.parameterTypeDef,'n) ccode;
+   combiner_template_paramfn : (Ast0.templateParameterTypeDef,'n) ccode;
    combiner_declfn : (Ast0.declaration,'n) ccode;
    combiner_fieldfn : (Ast0.field,'n) ccode;
    combiner_enumdeclfn : (Ast0.enum_decl,'n) ccode;
@@ -149,7 +155,9 @@ type rebuilder_rec_functions =
       rebuilder_rec_initialiser_list :
 	Ast0.initialiser_list rebuilder_inout;
       rebuilder_rec_parameter : Ast0.parameterTypeDef rebuilder_inout;
+      rebuilder_rec_template_parameter : Ast0.templateParameterTypeDef rebuilder_inout;
       rebuilder_rec_parameter_list : Ast0.parameter_list rebuilder_inout;
+      rebuilder_rec_template_parameter_list : Ast0.template_parameter_list rebuilder_inout;
       rebuilder_rec_statement : Ast0.statement rebuilder_inout;
       rebuilder_rec_forinfo : Ast0.forinfo rebuilder_inout;
       rebuilder_rec_case_line : Ast0.case_line rebuilder_inout;
@@ -183,8 +191,8 @@ type rebuilder_functions =
    rebuilder_const_mcode : Ast.constant rmcode;
    rebuilder_simpleAssign_mcode : Ast0.simpleAssignOp rmcode;
    rebuilder_opAssign_mcode : Ast.arithOp rmcode;
-   rebuilder_fix_mcode : Ast.fixOp rmcode;
-   rebuilder_unary_mcode : Ast.unaryOp rmcode;
+   rebuilder_fixOp_mcode : Ast.fixOp rmcode;
+   rebuilder_unaryOp_mcode : Ast.unaryOp rmcode;
    rebuilder_arithOp_mcode : Ast.arithOp rmcode;
    rebuilder_logicalOp_mcode : Ast.logicalOp rmcode;
    rebuilder_cv_mcode : Ast.const_vol rmcode;
@@ -195,6 +203,7 @@ type rebuilder_functions =
    rebuilder_dotsexprfn : Ast0.expression Ast0.dots rcode;
    rebuilder_dotsinitfn : Ast0.initialiser Ast0.dots rcode;
    rebuilder_dotsparamfn : Ast0.parameterTypeDef Ast0.dots rcode;
+   rebuilder_dotstemplateparamfn : Ast0.templateParameterTypeDef Ast0.dots rcode;
    rebuilder_dotsstmtfn : Ast0.statement Ast0.dots rcode;
    rebuilder_dotsdeclfn : Ast0.declaration Ast0.dots rcode;
    rebuilder_dotsfieldfn : Ast0.field Ast0.dots rcode;
@@ -209,6 +218,7 @@ type rebuilder_functions =
    rebuilder_tyfn : Ast0.typeC rcode;
    rebuilder_initfn : Ast0.initialiser rcode;
    rebuilder_paramfn : Ast0.parameterTypeDef rcode;
+   rebuilder_template_paramfn : Ast0.templateParameterTypeDef rcode;
    rebuilder_declfn : Ast0.declaration rcode;
    rebuilder_fieldfn : Ast0.field rcode;
    rebuilder_enumdeclfn : Ast0.enum_decl rcode;
@@ -224,6 +234,7 @@ type rebuilder_functions =
 (* combiner_rebuilder *)
 
 type ('mc,'a) rcmcode = 'a -> 'mc Ast0.mcode -> ('a * 'mc Ast0.mcode)
+type ('mc,'a) flat_rcmcode = 'mc Ast0.mcode -> ('a * 'mc Ast0.mcode)
 type ('cd,'a) rccode =
     'a all_functions -> ('cd -> ('a * 'cd)) -> 'cd -> ('a * 'cd)
 
@@ -233,8 +244,8 @@ type 'n combiner_rebuilder_functions =
    combiner_rebuilder_const_mcode : (Ast.constant,'n) rcmcode;
    combiner_rebuilder_simpleAssign_mcode : (Ast0.simpleAssignOp,'n) rcmcode;
    combiner_rebuilder_opAssign_mcode : (Ast.arithOp,'n) rcmcode;
-   combiner_rebuilder_fix_mcode : (Ast.fixOp,'n) rcmcode;
-   combiner_rebuilder_unary_mcode : (Ast.unaryOp,'n) rcmcode;
+   combiner_rebuilder_fixOp_mcode : (Ast.fixOp,'n) rcmcode;
+   combiner_rebuilder_unaryOp_mcode : (Ast.unaryOp,'n) rcmcode;
    combiner_rebuilder_arithOp_mcode : (Ast.arithOp,'n) rcmcode;
    combiner_rebuilder_logicalOp_mcode : (Ast.logicalOp,'n) rcmcode;
    combiner_rebuilder_cv_mcode : (Ast.const_vol,'n) rcmcode;
@@ -246,6 +257,8 @@ type 'n combiner_rebuilder_functions =
    combiner_rebuilder_dotsinitfn : (Ast0.initialiser Ast0.dots,'n) rccode;
    combiner_rebuilder_dotsparamfn :
       (Ast0.parameterTypeDef Ast0.dots,'n) rccode;
+   combiner_rebuilder_dotstemplateparamfn :
+      (Ast0.templateParameterTypeDef Ast0.dots,'n) rccode;
    combiner_rebuilder_dotsstmtfn : (Ast0.statement Ast0.dots,'n) rccode;
    combiner_rebuilder_dotsdeclfn : (Ast0.declaration Ast0.dots,'n) rccode;
    combiner_rebuilder_dotsfieldfn : (Ast0.field Ast0.dots,'n) rccode;
@@ -259,6 +272,7 @@ type 'n combiner_rebuilder_functions =
    combiner_rebuilder_tyfn : (Ast0.typeC,'n) rccode;
    combiner_rebuilder_initfn : (Ast0.initialiser,'n) rccode;
    combiner_rebuilder_paramfn : (Ast0.parameterTypeDef,'n) rccode;
+   combiner_rebuilder_template_paramfn : (Ast0.templateParameterTypeDef,'n) rccode;
    combiner_rebuilder_declfn : (Ast0.declaration,'n) rccode;
    combiner_rebuilder_fieldfn : (Ast0.field,'n) rccode;
    combiner_rebuilder_enumdeclfn : (Ast0.enum_decl,'n) rccode;
