@@ -163,8 +163,6 @@ let wrap_files args =
 
 let pyoutputinstance = ref Py.none
 
-let get_cocci_file args = Py.String.of_string !cocci_file_name
-
 (* initialisation routines *)
 let _pycocci_setargs argv0 =
   let argv =
@@ -174,7 +172,7 @@ let _pycocci_setargs argv0 =
 
 let initialize_python_path () =
   let sep = ":" in
-  let python_libdir = Filename.concat Config.path "python" in
+  let python_libdir = Filename.concat Cocciconfig.path "python" in
   match Common.optionise (fun () -> Sys.getenv "PYTHONPATH") with
     None -> Unix.putenv "PYTHONPATH" python_libdir
   | Some paths ->
@@ -199,7 +197,7 @@ let pycocci_init () =
     initialize_python_path ();
     let _ = if not (Py.is_initialized ()) then
       (if !Flag.show_misc then Common.pr2 "Initializing python\n%!";
-	Py.initialize ~interpreter:!Config.python_interpreter ()) in
+	Py.initialize ~interpreter:!Cocciconfig.python_interpreter ()) in
 
   (* set argv *)
   let argv0 = Sys.executable_name in
@@ -225,6 +223,7 @@ let pycocci_init () =
 	("make_stmt", wrap_make Coccilib.make_stmt);
 	("make_stmt_with_env", wrap_make_stmt_with_env);
 	("make_type", wrap_make Coccilib.make_type);
+	("make_pragmainfo", wrap_make Coccilib.make_pragmainfo);
 	("make_listlen", wrap_make_listlen);
 	("make_position", wrap_make_position);
         ("files", wrap_files);
@@ -278,12 +277,6 @@ let get_variable name =
   let mx = !coccinelle_module in
   Py.Module.get mx name
 
-let contains_binding e (_,(r,m),_) =
-  try
-    let _ = List.find (function ((re, rm), _) -> r = re && m = rm) e in
-    true
-  with Not_found -> false
-
 let construct_variables mv e =
   let find_binding (r,m) =
     try
