@@ -1,4 +1,4 @@
-int main() {
+int main(signed *m, unsigned *p) {
   signed x;
   unsigned a;
   return x;
