@@ -1,4 +1,6 @@
 
 static void
 fas216_log_command(FAS216_Info *info, int level, Scsi_Cmnd *SCpnt, char *fmt, ...)
-{}
+{
+  ty_id ty_id;
+}
