@@ -174,6 +174,8 @@ let rec constraints_to_buffer ~rn buffer cstr =
     Ast.CstrTrue | Ast.CstrSub [] -> ()
   | Ast.CstrFalse ->
       failwith "Ast_cocci.string_of_constraints: no syntax for Ast.CstrFalse"
+  | Ast.CstrExit ->
+      failwith "Ast_cocci.string_of_constraints: no syntax for Ast.CstrExit"
   | Ast.CstrAnd list -> ignore (List.fold_left (print_sub " && ") true list)
   | Ast.CstrOr list ->
       begin
@@ -269,7 +271,7 @@ let types ~rn = function
               get_meta_id_opt acc
                 (Common.default None Ast_cocci.ident_of_expression_opt nm2) in
             acc
-        | Ast.EnumName (_, Some n)
+        | Ast.EnumName (_, _, Some n)
         | Ast.StructUnionName(_, Some n) -> get_meta_id acc n
         | Ast.SignedT (_, Some t) -> get_meta_type acc t
         | Ast.Pointer (t, _)
@@ -386,6 +388,7 @@ let metavar_combiner rn =
   let dotsexprfn = donothing in
   let dotsinitfn = donothing in
   let dotsparamfn = donothing in
+  let dotstemplateparamfn = donothing in
   let dotsstmtfn = donothing in
   let dotsdeclfn = donothing in
   let dotsfieldfn = donothing in
@@ -523,6 +526,23 @@ let metavar_combiner rn =
         as_format ptd ex par expr
     | _ -> fn v in
 
+  let templateparamfn c fn v =
+    match Ast0.unwrap v with
+    (* FIXME: incomplete *)
+      Ast0.TypenameOrClassParam(tyorcl,id,Some (eq,ty)) ->
+           fn v
+    | Ast0.TypenameOrClassParam(tyorcl,id,None) ->
+           fn v
+    | Ast0.VarNameParam(ty,id,Some (eq,exp)) ->
+           fn v
+    | Ast0.VarNameParam(ty,id,None) ->
+           fn v
+    | Ast0.TPComma(comma) ->
+           fn v
+    | Ast0.TPDots(dots) ->
+           fn v
+    in
+
 (*
   Not used for now, visitor not parameterized by this...
   let define_paramfn c fn v =
@@ -539,10 +559,10 @@ let metavar_combiner rn =
         meta_mc_format ~mc ~typ:"declaration " ~constr
     | Ast0.AsDecl(dc1, dc2) ->
         let dec = c.VT0.combiner_rec_declaration in as_format dc1 dc2 dec dec
-    | Ast0.MacroDecl(_, id, _, expdots, _, _, _) ->
+    | Ast0.MacroDecl(_, _, id, _, expdots, _, _, _) ->
         let expids = c.VT0.combiner_rec_expression_dots expdots in
         MVSet.union (ids ~rn ~typ:"declarer" ~id) expids
-    | Ast0.MacroDeclInit(_, id, _, expdots, _, _, ini, _) ->
+    | Ast0.MacroDeclInit(_, _, id, _, expdots, _, _, _, ini, _) ->
         let expids = c.VT0.combiner_rec_expression_dots expdots in
         let inid = MVSet.union expids (c.VT0.combiner_rec_initialiser ini) in
         let declids = ids ~rn ~typ:"declarer" ~id in
@@ -565,7 +585,7 @@ let metavar_combiner rn =
     | Ast0.FormatFragment(_, format) ->
       (match Ast0.unwrap format with
         | Ast0.MetaFormat(mc, idconstr) ->
-            let constr = constraints rn idconstr in
+            let constr = constraints ~rn:rn idconstr in
             meta_mc_format ~mc ~typ:"format " ~constr
         | _ -> fn v
        )
@@ -578,14 +598,21 @@ let metavar_combiner rn =
         meta_mc_format ~mc ~typ:"parameter " ~constr
     | _ -> fn v in
 
-  V0.flat_combiner bind option_default
-    meta_mcode string_mcode const_mcode simpleAssign_mcode opAssign_mcode
-    fix_mcode unary_mcode arithOp_mcode logicalOp_mcode cv_mcode sign_mcode
-    struct_mcode storage_mcode inc_mcode
-    dotsexprfn dotsinitfn dotsparamfn dotsstmtfn dotsdeclfn dotsfieldfn
-    dotsenumdeclfn dotscasefn dotsdefparfn
-    identfn exprfn assignOpfn binaryOpfn tyfn initfn paramfn declfn fieldfn
-    enumdeclfn stmtfn forinfofn casefn string_fragmentfn attributefn attr_argfn topfn
+  V0.combiner_default bind option_default
+    ~meta_mcode:meta_mcode ~string_mcode:string_mcode ~const_mcode:const_mcode
+    ~simpleAssign_mcode:simpleAssign_mcode ~opAssign_mcode:opAssign_mcode
+    ~fixOp_mcode:fix_mcode ~unaryOp_mcode:unary_mcode ~arithOp_mcode:arithOp_mcode
+    ~logicalOp_mcode:logicalOp_mcode ~cv_mcode:cv_mcode ~sign_mcode:sign_mcode
+    ~struct_mcode:struct_mcode ~storage_mcode:storage_mcode ~inc_mcode:inc_mcode
+    ~dotsexpr:dotsexprfn ~dotsinit:dotsinitfn ~dotsparam:dotsparamfn
+    ~dotstemplateparam:dotstemplateparamfn ~dotsstmt:dotsstmtfn ~dotsdecl:dotsdeclfn
+    ~dotsfield:dotsfieldfn ~dotsenumdecl:dotsenumdeclfn ~dotscase:dotscasefn
+    ~dotsdefpar:dotsdefparfn ~ident:identfn ~expr:exprfn ~assignOp:assignOpfn
+    ~binaryOp:binaryOpfn ~ty:tyfn ~init:initfn ~param:paramfn
+    ~template_param:templateparamfn ~decl:declfn ~field:fieldfn
+    ~enumdecl:enumdeclfn ~stmt:stmtfn ~forinfo:forinfofn ~case:casefn
+    ~string_fragment:string_fragmentfn ~attribute:attributefn ~attr_arg:attr_argfn
+    ~top:topfn ()
 
 
 (* ------------------------------------------------------------------------- *)
