@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -12,6 +12,7 @@
  *  - interactive: if true, draw user input interactively
  *  - default: if true, generate without user input (using default values)
  *  - hide: if true, do not output the generated file
+ *  - year: current year (for copyright in generated header)
  *)
 
 type options
@@ -22,6 +23,7 @@ val make_options :
   ?interactive:bool ->
   ?default:bool ->
   ?hide:bool ->
+  ?year:int ->
   string -> (* filename of cocci file to generate *)
   options
 
